/*******************************************************************************
 * Copyright (c) 2006 Cisco Technology, Inc and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Author: Andrew Cheadle, CrossCore Optimization Limited
 *
 * Contributors:
 *     Cisco Technology, Inc - Initial implementation
 *******************************************************************************/

package org.eclipseclp.saros.core.preferences;

import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.widgets.Link;
import org.eclipse.ui.dialogs.PreferencesUtil;

public class CompilerOutput extends Composite implements SelectionListener {

	public CompilerOutput(Composite parent, int style) {
		super(parent, style);
		initialise(false);
	}

	public CompilerOutput(Composite parent, int style,
			boolean showProjectSpecific) {
		super(parent, style);
		initialise(showProjectSpecific);
	}

	public void commit() {
		compilationTargets = new boolean[] { fcompileCheckbox.getSelection(),
				icompileCheckbox.getSelection(),
				icompileCheckbox.getSelection() && htmlCheckbox.getSelection() };
	}

	public void revertToDefaults() {
		setWidgetValues(PreferenceManager.getDefaultOutputCompilationTargets());
	}

	private void initialise(boolean showProjectSpecific) {
		this.showProjectSpecific = showProjectSpecific;
		GridLayout layout = new GridLayout();
		GridData gridData = new GridData();
		gridData.horizontalAlignment = org.eclipse.swt.layout.GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		gridData.verticalAlignment = org.eclipse.swt.layout.GridData.FILL;
		this.setLayout(layout);
		this.setLayoutData(gridData);
		createCompilationGroup(this);
		setWidgetValues(PreferenceManager.getOutputCompilationTargets());
		if (showProjectSpecific) {
			defaultsRadio.setSelection(true);
			projectSpecificRadio.setSelection(false);
			fcompileCheckbox.setEnabled(false);
			icompileCheckbox.setEnabled(false);
			htmlCheckbox.setEnabled(false);
		}
		setSize(this.computeSize(SWT.DEFAULT, SWT.DEFAULT));
	}

	private void createCompilationGroup(Composite composite) {
		GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		GridLayout layout = new GridLayout();
		layout.verticalSpacing = 8;
		layout.numColumns = 2;
		compilationGroup = new Group(composite, SWT.NONE);
		compilationGroup.setLayout(layout);
		compilationGroup.setLayoutData(gridData);
		compilationGroup.setText(PreferenceMessages
				.getString("ECLiPSeStrings.ProjectCompilation")); //$NON-NLS-1$
		int horizontalIndent = 0;
		if (showProjectSpecific) {
			defaultsRadio = new Button(compilationGroup, SWT.RADIO);
			defaultsRadio.setText(PreferenceMessages
					.getString("ECLiPSeStrings.UseDefaultCompilationTargets"));
			defaultsRadio.addSelectionListener(this);
			gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
			gridData.grabExcessHorizontalSpace = true;
			defaultsRadio.setLayoutData(gridData);
			configureDefaultsLink = new Link(compilationGroup, SWT.NONE);
			configureDefaultsLink.setText("<A>"
					+ PreferenceMessages
							.getString("ECLiPSeStrings.ConfigureDefaults")
					+ "..." + "</A>");
			configureDefaultsLink.addSelectionListener(this);
			gridData = new GridData(GridData.HORIZONTAL_ALIGN_END);
			configureDefaultsLink.setLayoutData(gridData);
			projectSpecificRadio = new Button(compilationGroup, SWT.RADIO);
			projectSpecificRadio
					.setText(PreferenceMessages
							.getString("ECLiPSeStrings.UseProjectSpecificCompilationTargets"));
			projectSpecificRadio.addSelectionListener(this);
			gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
			gridData.horizontalSpan = 2;
			gridData.grabExcessHorizontalSpace = true;
			projectSpecificRadio.setLayoutData(gridData);
			horizontalIndent = 22;
		}

		fcompileCheckbox = new Button(compilationGroup, SWT.CHECK);
		fcompileCheckbox.setText(PreferenceMessages
				.getString("ECLiPSeStrings.fcompile"));
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		gridData.horizontalSpan = 2;
		gridData.horizontalIndent = horizontalIndent;
		fcompileCheckbox.setLayoutData(gridData);
		icompileCheckbox = new Button(compilationGroup, SWT.CHECK);
		icompileCheckbox.setText(PreferenceMessages
				.getString("ECLiPSeStrings.icompile"));
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		gridData.horizontalSpan = 2;
		gridData.horizontalIndent = horizontalIndent;
		icompileCheckbox.setLayoutData(gridData);
		icompileCheckbox.addSelectionListener(this);
		htmlCheckbox = new Button(compilationGroup, SWT.CHECK);
		htmlCheckbox.setText(PreferenceMessages
				.getString("ECLiPSeStrings.GenerateHtmlDoc"));
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		gridData.horizontalSpan = 2;
		gridData.horizontalIndent = horizontalIndent;
		htmlCheckbox.setLayoutData(gridData);
	}

	private void setWidgetValues(boolean[] targets) {
		if (targets[0]) {
			fcompileCheckbox.setSelection(true);
		} else {
			fcompileCheckbox.setSelection(false);
		}

		if (targets[1]) {
			icompileCheckbox.setSelection(true);
			htmlCheckbox.setEnabled(true);
		} else {
			icompileCheckbox.setSelection(false);
			htmlCheckbox.setEnabled(false);
		}

		if (targets[2]) {
			htmlCheckbox.setSelection(true);
		} else {
			htmlCheckbox.setSelection(false);
		}
	}

	public void widgetDefaultSelected(SelectionEvent event) {
	}

	public void widgetSelected(SelectionEvent event) {
		if (event.getSource() == icompileCheckbox) {
			if (icompileCheckbox.getSelection()) {
				htmlCheckbox.setEnabled(true);
			} else {
				htmlCheckbox.setEnabled(false);
			}
		} else if (event.getSource() == defaultsRadio) {
			fcompileCheckbox.setEnabled(false);
			icompileCheckbox.setEnabled(false);
			htmlCheckbox.setEnabled(false);
			configureDefaultsLink.setEnabled(true);
		} else if (event.getSource() == projectSpecificRadio) {
			fcompileCheckbox.setEnabled(true);
			icompileCheckbox.setEnabled(true);
			htmlCheckbox.setEnabled(true);
			configureDefaultsLink.setEnabled(false);
		} else if (event.getSource() == configureDefaultsLink) {
			configureWorkspacePreferences();
		}
	}

	public void configureWorkspacePreferences() {
		PreferencesUtil.createPreferenceDialogOn(getShell(),
				PreferenceConstants.CompilerPreferencePageId,
				new String[] { PreferenceConstants.CompilerPreferencePageId },
				null).open();
		setWidgetValues(PreferenceManager.getOutputCompilationTargets());
	}

	public boolean[] getOutputCompilationTargets() {
		return compilationTargets;
	}

	public void setEnabled(boolean enabled) {
		compilationGroup.setEnabled(enabled);
		fcompileCheckbox.setEnabled(enabled);
		icompileCheckbox.setEnabled(enabled);
		htmlCheckbox.setEnabled(enabled);
	}
	
	public void setIcompile(boolean icompile) {
		icompileCheckbox.setSelection(icompile);
		htmlCheckbox.setEnabled(icompile);
	}

	public void setFcompile(boolean fcompile) {
		fcompileCheckbox.setSelection(fcompile);
	}

	public void setEciToHtml(boolean eciToHtml) {
		htmlCheckbox.setSelection(eciToHtml);
	}

	private Group compilationGroup = null;

	private Button fcompileCheckbox = null;

	private Button icompileCheckbox = null;

	private Button htmlCheckbox = null;

	private Link configureDefaultsLink = null;

	private boolean showProjectSpecific = false;

	private Button defaultsRadio = null;

	private Button projectSpecificRadio = null;

	private boolean[] compilationTargets = new boolean[0];

	public boolean isProjectSpecificCompilationTarget() {
		return projectSpecificRadio.getSelection();
	}
}
