/*******************************************************************************
 * Copyright (c) 2006 Cisco Technology, Inc and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Author: Andrew Cheadle, CrossCore Optimization Limited
 *
 * Contributors:
 *     Cisco Technology, Inc - Initial implementation
 *******************************************************************************/

package org.eclipseclp.saros.core.preferences;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Scale;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;

public class ECLiPSeChangeDepth extends Composite implements SelectionListener {

	private String title = null;

	private Button cancelButton = null;

	private Button okButton = null;

	private Composite buttonComposite = null;

	private Group depthGroup = null;

	private Scale depthScale = null;

	private Text depthText = null;

	private Button fullCheckBox = null;

	private Composite parentComposite = null;

	private static int maxDepth = 100;

	private static int minDepth = 0;

	private int printDepth = 20;

	private int oldPrintDepth = 20;

	public ECLiPSeChangeDepth(Composite parent, int style) {
		super(parent, style);
		title = PreferenceMessages.getString("ECLiPSeStrings.PrintDepth");
		parentComposite = parent;
		initialize(false, false);
	}

	public ECLiPSeChangeDepth(Composite parent, int style, String newTitle,
			int value, boolean fullDepth) {
		super(parent, style);
		title = newTitle;
		parentComposite = parent;
		oldPrintDepth = printDepth = value;
		initialize(fullDepth, false);
	}

	public ECLiPSeChangeDepth(Composite parent, int style, String newTitle,
			int value, boolean fullDepth, boolean shellButtons) {
		super(parent, style);
		title = newTitle;
		parentComposite = parent;
		oldPrintDepth = printDepth = value;
		initialize(fullDepth, shellButtons);
	}

	/**
	 * This method initializes the ECLiPSeChangeDepth widget
	 * 
	 */
	private void initialize(boolean fullDepth, boolean shellButtons) {
		this.setLayout(new GridLayout());
		createDepthGroup();
		if (fullDepth) {
			oldPrintDepth = printDepth = -1;
			depthScale.setEnabled(false);
			depthText.setEnabled(false);
			fullCheckBox.setSelection(true);
		}
		if (shellButtons) {
			createButtonComposite();
		}
		setSize(this.computeSize(SWT.DEFAULT, SWT.DEFAULT));
	}

	/**
	 * This method initializes depthGroup
	 * 
	 * The composite contains a slider, ranging from minDepth to maxDepth and a
	 * checkBox, which will set the slider to maxDepth when selected, and return
	 * it to the previous depth when deselected
	 * 
	 */
	private void createDepthGroup() {
		depthGroup = new Group(this, SWT.NONE);
		depthGroup.setText(title);
		GridLayout layout = new GridLayout();
		layout.numColumns = 3;
		depthGroup.setLayout(layout);
		GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		depthGroup.setLayoutData(gridData);
		depthScale = new Scale(depthGroup, SWT.NONE);
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		gridData.minimumWidth = depthScale.computeSize(120, SWT.DEFAULT).x;
		depthScale.setLayoutData(gridData);
		depthScale.setMinimum(minDepth);
		depthScale.setMaximum(maxDepth);
		depthScale.setIncrement(1);
		depthScale.setPageIncrement(5);
		depthScale.setSelection(printDepth);
		depthScale.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				printDepth = depthScale.getSelection();
				depthText.setText("" + printDepth); //$NON-NLS-1$
			}
		});
		depthText = new Text(depthGroup, SWT.BORDER | SWT.SINGLE | SWT.CENTER);
		depthText.setText("" + maxDepth); //$NON-NLS-1$
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		gridData.minimumWidth = depthText.computeSize(SWT.DEFAULT, SWT.DEFAULT).x;
		depthText.setLayoutData(gridData);
		depthText.setText("" + printDepth); //$NON-NLS-1$
		fullCheckBox = new Button(depthGroup, SWT.CHECK);
		fullCheckBox.setSelection(false);
		fullCheckBox.addSelectionListener(this);
		fullCheckBox.setText(PreferenceMessages.getString("ECLiPSeStrings.Full"));
		//fullLabel = new Label(depthGroup, SWT.NONE);
		//fullLabel.setText(PreferenceMessages.getString("ECLiPSeStrings.Full")); //$NON-NLS-1$
	}

	/**
	 * This method constructs the Cancel/OK button group
	 * 
	 */
	private void createButtonComposite() {
		buttonComposite = new Composite(this, SWT.NONE);
		GridLayout layout = new GridLayout();
		layout.numColumns = 2;
		layout.makeColumnsEqualWidth = true;
		buttonComposite.setLayout(layout);
		GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		buttonComposite.setLayoutData(gridData);
		okButton = new Button(buttonComposite, SWT.PUSH);
		okButton.setText(PreferenceMessages.getString("ECLiPSeStrings.OK")); //$NON-NLS-1$
		cancelButton = new Button(buttonComposite, SWT.PUSH);
		cancelButton.setText(PreferenceMessages
				.getString("ECLiPSeStrings.Cancel")); //$NON-NLS-1$
		cancelButton.addSelectionListener(this);
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL
				| GridData.VERTICAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		cancelButton.setLayoutData(gridData);
		okButton.addSelectionListener(this);
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL
				| GridData.VERTICAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		okButton.setLayoutData(gridData);
	}

	public int getPrintDepth() {
		return printDepth;
	}

	public void widgetDefaultSelected(SelectionEvent event) {
	}

	public void widgetSelected(SelectionEvent event) {
		if (event.getSource() == okButton) {
			if (fullCheckBox.getSelection()) {
				printDepth = -1;
			} else {
				printDepth = (new Integer(depthText.getText())).intValue();
			}
			parentComposite.dispose();
		} else if (event.getSource() == cancelButton) {
			printDepth = oldPrintDepth;
			parentComposite.dispose();
		} else if (event.getSource() == fullCheckBox) {
			if (fullCheckBox.getSelection()) {
				printDepth = -1;
				depthScale.setEnabled(false);
				depthText.setEnabled(false);
			} else {
				printDepth = (new Integer(depthText.getText())).intValue();
				depthScale.setEnabled(true);
				depthText.setEnabled(true);
			}
		}
	}
}
