/*******************************************************************************
 * Copyright (c) 2006 Cisco Technology, Inc and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Author: Andrew Cheadle, CrossCore Optimization Limited
 *
 * Contributors:
 *     Cisco Technology, Inc - Initial implementation
 *******************************************************************************/

package org.eclipseclp.saros.core.preferences;

import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.jface.dialogs.MessageDialog;
import java.io.File;
import java.util.List;

public class ECLiPSeEngineInstallation extends Composite implements
		SelectionListener {

	public ECLiPSeEngineInstallation(Composite parent, int style,
			List installations, EnginePreferences initialisedInstallation) {
		super(parent, style);
		this.installations = installations;
		parentComposite = parent;
		initialise(initialisedInstallation, false);
	}

	public ECLiPSeEngineInstallation(Composite parent, int style,
			boolean shellButtons, List installations,
			EnginePreferences initialisedInstallation) {
		super(parent, style);
		this.installations = installations;
		parentComposite = parent;
		initialise(initialisedInstallation, shellButtons);
	}

	private void initialise(EnginePreferences initialisedInstallation,
			boolean shellButtons) {
		if (initialisedInstallation != null) {
			installation = initialisedInstallation;
		} else {
			installation = new EnginePreferences();
		}

		GridLayout layout = new GridLayout();
		GridData gridData = new GridData();
		gridData.horizontalAlignment = org.eclipse.swt.layout.GridData.FILL;
		gridData.grabExcessVerticalSpace = true;
		gridData.verticalAlignment = org.eclipse.swt.layout.GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		this.setLayout(layout);
		this.setLayoutData(gridData);
		createInstallationGroup();
		if (shellButtons) {
			createButtonComposite();
		}
		setSize(this.computeSize(SWT.DEFAULT, SWT.DEFAULT));
	}

	/**
	 * This method initializes installationGroup
	 * 
	 */
	private void createInstallationGroup() {
		installationGroup = new Group(this, SWT.NONE);
		GridLayout layout = new GridLayout();
		layout.numColumns = 3;
		installationGroup.setLayout(layout);
		GridData gridData = new GridData();
		gridData.widthHint = 350;
		gridData.horizontalAlignment = org.eclipse.swt.layout.GridData.FILL;
		gridData.grabExcessVerticalSpace = true;
		gridData.verticalAlignment = org.eclipse.swt.layout.GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		installationGroup.setLayoutData(gridData);
		installationGroup.setText(PreferenceMessages
				.getString("ECLiPSeStrings.ECLiPSeEngineInstallation")); //$NON-NLS-1$
		installationNameLabel = new Label(installationGroup, SWT.NONE);
		installationNameLabel.setText(PreferenceMessages
				.getString("ECLiPSeStrings.Name") + ":"); //$NON-NLS-1$
		RadioCompositeBuilder.buildLabel(installationNameLabel);
		installationNameText = new Text(installationGroup, SWT.BORDER);
		installationNameText.setText(installation.getInstallationName());
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		gridData.horizontalSpan = 2;
		installationNameText.setLayoutData(gridData);
		installationPathLabel = new Label(installationGroup, SWT.NONE);
		installationPathLabel.setText(PreferenceMessages
				.getString("ECLiPSeStrings.InstallationDirectory") + ":"); //$NON-NLS-1$
		RadioCompositeBuilder.buildLabel(installationPathLabel);
		installationPathText = new Text(installationGroup, SWT.BORDER);
		installationPathText.setText(installation.getInstallationPath());
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		installationPathText.setLayoutData(gridData);
		installationPathButton = new Button(installationGroup, SWT.NONE);
		installationPathButton.setText(PreferenceMessages
				.getString("ECLiPSeStrings.Browse") + "..."); //$NON-NLS-1$
		installationPathButton.addSelectionListener(this);
		libPathLabel = new Label(installationGroup, SWT.NONE);
		libPathLabel.setText(PreferenceMessages
				.getString("ECLiPSeStrings.LibraryPath") + ":"); //$NON-NLS-1$
		RadioCompositeBuilder.buildLabel(libPathLabel);
		libraryPathText = new Text(installationGroup, SWT.BORDER);
		libraryPathText.setText(installation.getLibraryPath());
		libraryPathButton = new Button(installationGroup, SWT.NONE);
		libraryPathButton.setText(PreferenceMessages
				.getString("ECLiPSeStrings.Browse") + "..."); //$NON-NLS-1$
		libraryPathButton.addSelectionListener(this);
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		libraryPathText.setLayoutData(gridData);
		engineArgumentsLabel = new Label(installationGroup, SWT.NONE);
		engineArgumentsLabel.setText(PreferenceMessages
				.getString("ECLiPSeStrings.EngineArguments") + ":"); //$NON-NLS-1$
		RadioCompositeBuilder.buildLabel(engineArgumentsLabel);
		engineArgumentsText = new Text(installationGroup, SWT.BORDER);
		engineArgumentsText.setText(installation.getEngineArguments());
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		gridData.horizontalSpan = 2;
		engineArgumentsText.setLayoutData(gridData);
	}

	private void createButtonComposite() {
		buttonComposite = new Composite(this, SWT.NONE);
		GridLayout layout = new GridLayout();
		layout.numColumns = 2;
		layout.makeColumnsEqualWidth = true;
		buttonComposite.setLayout(layout);
		GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		buttonComposite.setLayoutData(gridData);
		okButton = new Button(buttonComposite, SWT.PUSH);
		okButton.setText(PreferenceMessages.getString("ECLiPSeStrings.OK")); //$NON-NLS-1$
		cancelButton = new Button(buttonComposite, SWT.PUSH);
		cancelButton.setText(PreferenceMessages
				.getString("ECLiPSeStrings.Cancel")); //$NON-NLS-1$
		cancelButton.addSelectionListener(this);
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL
				| GridData.VERTICAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		cancelButton.setLayoutData(gridData);
		okButton.addSelectionListener(this);
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL
				| GridData.VERTICAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		okButton.setLayoutData(gridData);
	}

	public void widgetDefaultSelected(SelectionEvent e) {
	}

	public void widgetSelected(SelectionEvent event) {
		if (event.getSource() == okButton) {
			if (createInstallation(true)) {
				parentComposite.dispose();
			}
		} else if (event.getSource() == cancelButton) {
			installation.setInstallationName("");
			parentComposite.dispose();
		} else if (event.getSource() == installationPathButton) {
			DirectoryDialog dialog = new DirectoryDialog(
					(Shell) parentComposite, SWT.DIALOG_TRIM
							| SWT.APPLICATION_MODAL);
			dialog.setText(PreferenceMessages
					.getString("ECLiPSeStrings.BrowseFolder"));
			dialog.setMessage(PreferenceMessages
					.getString("ECLiPSeStrings.SelectInstallDir")
					+ ":");
			dialog.setFilterPath(installationPathText.getText());
			String newInstallationPath = dialog.open();
			if (newInstallationPath != null) {
				installationPathText.setText(newInstallationPath);
				installationPathText.update();
				if (installationNameText.getText().equals("")) {
					installationNameText
							.setText((new File(newInstallationPath)).getName());
				}
			}
		} else if (event.getSource() == libraryPathButton) {
			DirectoryDialog dialog = new DirectoryDialog(
					(Shell) parentComposite, SWT.DIALOG_TRIM
							| SWT.APPLICATION_MODAL);
			dialog.setText(PreferenceMessages
					.getString("ECLiPSeStrings.BrowseFolder"));
			dialog.setMessage(PreferenceMessages
					.getString("ECLiPSeStrings.SelectLibraryDir")
					+ ":");
			dialog.setFilterPath("");
			String libraryDir = dialog.open();
			if (libraryDir != null) {
				String[] subDirs = libraryPathText.getText().split(",");
				for (int cnt = 0; cnt < subDirs.length; cnt++) {
					if (subDirs[cnt].equals(libraryDir)) {
						return;
					}
				}

				if (!libraryPathText.getText().equals("")) {
					libraryPathText.append(", ");
				}
				libraryPathText.append(libraryDir);
				libraryPathText.update();
			}
		}
	}

	private boolean createInstallation(boolean create) {
		boolean success = true;
		installation.setInstallationName(installationNameText.getText());
		installation.setInstallationPath(installationPathText.getText());
		installation.setLibraryPath(libraryPathText.getText());
		installation.setEngineArguments(engineArgumentsText.getText());
		if (installation.getInstallationName().equals("") && success) {
			MessageDialog.openError(this.getShell(), PreferenceMessages
					.getString("ECLiPSeStrings.Error"), PreferenceMessages
					.getString("ECLiPSeStrings.InstallationNameError")
					+ ".");
			success = false;
		}
		if (installation.getInstallationPath().equals("") && success) {
			MessageDialog.openError(this.getShell(), PreferenceMessages
					.getString("ECLiPSeStrings.Error"), PreferenceMessages
					.getString("ECLiPSeStrings.InstallationDirectoryError")
					+ ".");
			success = false;
		}

		if (success) {
			String installationName = installation.getInstallationName();
			for (int engine = 0; engine < installations.size(); engine++) {
				EnginePreferences existingInstallation = (EnginePreferences) installations
						.get(engine);
				if (installation == existingInstallation) {
					continue;
				}
				if (installationName.equals(existingInstallation
						.getInstallationName())) {
					MessageDialog
							.openError(
									this.getShell(),
									PreferenceMessages
											.getString("ECLiPSeStrings.Error"),
									PreferenceMessages
											.getString("ECLiPSeStrings.InstallationExistsError")
											+ ".");
					success = false;
					break;
				}
			}
		}

		if (success) {
			if (create) {
				installation = PreferenceManager.initialiseInstallation(
						installation, true);
				if (installation == null) {
					success = false;
					installation = new EnginePreferences();
				}
			} else {
				// Verify the installation isn't broken!
				if (PreferenceManager.initialiseInstallation(
						new EnginePreferences(installation
								.getInstallationName(), installation
								.getInstallationPath(), installation
								.getLibraryPath(), ""), true) == null) {
					return false;
				}
			}
		}

		return success;
	}

	public EnginePreferences commit(EnginePreferences installation) {
		this.installation = installation;
		if (!createInstallation(false)) {
			return null;
		}
		return installation;
	}

	public EnginePreferences getInstallation() {
		return installation;
	}

	private Group installationGroup = null;

	private Label installationNameLabel = null;

	private Text installationNameText = null;

	private Label installationPathLabel = null;

	private Text installationPathText = null;

	private Button installationPathButton = null;

	private Label libPathLabel = null;

	private Text libraryPathText = null;

	private Button libraryPathButton = null;

	private Label engineArgumentsLabel = null;

	private Text engineArgumentsText = null;

	private Button okButton = null;

	private Button cancelButton = null;

	private Composite buttonComposite = null;

	private Composite parentComposite = null;

	private EnginePreferences installation = null;

	private List installations = null;
}
