/*******************************************************************************
 * Copyright (c) 2006 Cisco Technology, Inc and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Author: Andrew Cheadle, CrossCore Optimization Limited
 *
 * Contributors:
 *     Cisco Technology, Inc - Initial implementation
 *******************************************************************************/

package org.eclipseclp.saros.core.preferences;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.graphics.Point;
import java.util.List;
import java.util.ArrayList;

public class ECLiPSeEngineInstallationShell {

	private List installations = null;

	private EnginePreferences initialisedInstallation = null;

	private Shell engineInstallationShell = null;

	private ECLiPSeEngineInstallation engineSettingsComposite = null;

	public ECLiPSeEngineInstallationShell(List installations,
			EnginePreferences initialisedInstallation) {
		this.installations = installations;
		this.initialisedInstallation = initialisedInstallation;
	}

	/**
	 * This method initializes engineInstallationShell
	 * 
	 * The shell is a modal window, so that the parent window is blocked until
	 * the selection is made or cancelled The input parameter title contains the
	 * string that is displayed as the window title
	 */
	private void createEngineInstallationShell(Shell parent) {
		engineInstallationShell = new Shell(parent, SWT.DIALOG_TRIM
				| SWT.APPLICATION_MODAL);
		engineInstallationShell.setText(PreferenceMessages
				.getString("ECLiPSeStrings.ECLiPSe")
				+ PreferenceMessages
						.getString("ECLiPSeStrings.ECLiPSeEngineInstallation")); //$NON-NLS-1$
		GridLayout layout = new GridLayout();
		engineInstallationShell.setLayout(layout);
		engineSettingsComposite = new ECLiPSeEngineInstallation(
				engineInstallationShell, SWT.NONE, true, installations,
				initialisedInstallation);
		GridData gridData = new GridData();
		gridData.horizontalAlignment = org.eclipse.swt.layout.GridData.FILL;
		gridData.grabExcessVerticalSpace = true;
		gridData.verticalAlignment = org.eclipse.swt.layout.GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		engineSettingsComposite.setLayoutData(gridData);
		engineInstallationShell.setSize(engineInstallationShell.computeSize(
				SWT.DEFAULT, SWT.DEFAULT));
	}

	/**
	 * This method opens a modal ECLiPSeengineInstallationShell
	 * 
	 * @param parent
	 *            the parent window
	 */
	public EnginePreferences open(Shell parent) {
		// open shell
		createEngineInstallationShell(parent);
		Point parentSize = parent.getSize();
		Point size = engineInstallationShell.getSize();
		int x = (parentSize.x - size.x) / 2;
		int y = (parentSize.y - size.y) / 2;
		engineInstallationShell.setLocation(parent.toDisplay(x, y));
		engineInstallationShell.open();
		// block display until shell is closed
		Display display = Display.getDefault();
		while (!engineInstallationShell.isDisposed()) {
			if (!display.readAndDispatch())
				display.sleep();
		}

		return engineSettingsComposite.getInstallation();
	}

	/**
	 * main method, for debugging purposes
	 * 
	 * @param args
	 */
	public static void main(String[] args) {
		/*
		 * Before this is run, be sure to set up the launch configuration
		 * (Arguments->VM Arguments) for the correct SWT library path in order
		 * to run with the SWT dlls. The dlls are located in the SWT plugin jar.
		 * For example, on Windows the Eclipse SWT 3.1 plugin jar is:
		 * installation_directory\plugins\org.eclipse.swt.win32_3.1.0.jar
		 */
		Display display = Display.getDefault();
		ECLiPSeEngineInstallationShell thisClass = new ECLiPSeEngineInstallationShell(
				new ArrayList(), null);
		thisClass.open(null);
		while (!thisClass.engineInstallationShell.isDisposed()) {
			if (!display.readAndDispatch())
				display.sleep();
		}
		display.dispose();
	}
}
