/*******************************************************************************
 * Copyright (c) 2006 Cisco Technology, Inc and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Author: Andrew Cheadle, CrossCore Optimization Limited
 *
 * Contributors:
 *     Cisco Technology, Inc - Initial implementation
 *******************************************************************************/

package org.eclipseclp.saros.core.preferences;

import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import java.util.List;

public class ECLiPSeEnginePreferences extends Composite implements
		SelectionListener {

	/**
	 * 
	 * @param parent
	 * @param style
	 * @param installations
	 * @param installation
	 */
	public ECLiPSeEnginePreferences(Composite parent, int style,
			List installations, EnginePreferences installation) {
		super(parent, style);
		parentComposite = parent;
		this.installations = installations;
		this.installation = installation;
		initialize(true);
	}

	/**
	 * 
	 * @param parent
	 * @param style
	 * @param shellButtons
	 *            indicates whether the Close and OK buttons should be shown
	 * @param installations
	 * @param installation
	 */
	public ECLiPSeEnginePreferences(Composite parent, int style,
			boolean shellButtons, List installations,
			EnginePreferences installation) {
		super(parent, style);
		parentComposite = parent;
		this.installations = installations;
		this.installation = installation;
		initialize(shellButtons);
	}

	/**
	 * This method initializes the Engine Preferences widget
	 * 
	 * @param shellButtons:
	 *            true == show button composite
	 * 
	 * The widget contains a scrolled composite, since the list of radio buttons
	 * is quite long
	 * 
	 */
	private void initialize(boolean shellButtons) {
		GridLayout layout = new GridLayout();
		GridData gridData = new GridData();
		gridData.horizontalAlignment = org.eclipse.swt.layout.GridData.FILL;
		gridData.grabExcessVerticalSpace = true;
		gridData.verticalAlignment = org.eclipse.swt.layout.GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		this.setLayout(layout);
		this.setLayoutData(gridData);
		layout = new GridLayout();
		gridData = new GridData();
		gridData.horizontalAlignment = org.eclipse.swt.layout.GridData.FILL;
		gridData.grabExcessVerticalSpace = true;
		gridData.verticalAlignment = org.eclipse.swt.layout.GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		scrolledComposite = new ScrolledComposite(this, SWT.V_SCROLL
				| SWT.H_SCROLL);
		scrolledComposite.setLayout(layout);
		scrolledComposite.setLayoutData(gridData);
		scrolledComposite.setExpandHorizontal(true);
		scrolledComposite.setExpandVertical(true);
		layout = new GridLayout();
		gridData = new GridData();
		gridData.horizontalAlignment = org.eclipse.swt.layout.GridData.FILL;
		gridData.grabExcessVerticalSpace = true;
		gridData.verticalAlignment = org.eclipse.swt.layout.GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		composite = new Composite(scrolledComposite, SWT.NONE);
		composite.setLayout(layout);
		composite.setLayoutData(gridData);
		if (installations != null) {
			engineInstallation = new ECLiPSeEngineInstallation(composite,
					SWT.None, installations, installation);
		}
		engineSettings = new ECLiPSeEngineSettings(composite, SWT.NONE, true,
				installation);
		gcSettings = new ECLiPSeGCSettings(composite, SWT.NONE, installation);
		if (shellButtons) {
			createButtonComposite();
		}
		setSize(this.computeSize(560, 560, true));
		scrolledComposite.setContent(composite);
		composite.setSize(composite.computeSize(500, 856));
		scrolledComposite.setSize(500, 856);
		scrolledComposite.setMinSize(500, 856);
	}

	/**
	 * This method initializes buttonComposite
	 * 
	 */
	private void createButtonComposite() {
		buttonComposite = new Composite(this, SWT.NONE);
		GridLayout layout = new GridLayout();
		layout.numColumns = 3;
		buttonComposite.setLayout(layout);
		GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		buttonComposite.setLayoutData(gridData);
		okButton = new Button(buttonComposite, SWT.PUSH);
		okButton.setText(PreferenceMessages.getString("ECLiPSeStrings.OK")); //$NON-NLS-1$
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL
				| GridData.VERTICAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		okButton.setLayoutData(gridData);
		okButton.addSelectionListener(this);
		cancelButton = new Button(buttonComposite, SWT.PUSH);
		cancelButton.setText(PreferenceMessages
				.getString("ECLiPSeStrings.Cancel")); //$NON-NLS-1$
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL
				| GridData.VERTICAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		cancelButton.setLayoutData(gridData);
		cancelButton.addSelectionListener(this);
	}

	public void widgetDefaultSelected(SelectionEvent event) {
	}

	public void widgetSelected(SelectionEvent event) {
		if (event.getSource() == okButton) {
			if (engineInstallation != null) {
				EnginePreferences newInstallation = engineInstallation
						.commit(installation);
				if (newInstallation == null) {
					return;
				} else {
					installation = newInstallation;
				}
			}
			installation = engineSettings.commit(installation);
			installation = gcSettings.commit(installation);
			parentComposite.dispose();
		} else if (event.getSource() == cancelButton) {
			parentComposite.dispose();
		}
	}

	public EnginePreferences getInstallation() {
		return installation;
	}

	private ScrolledComposite scrolledComposite = null;

	private Composite composite = null;

	private Button okButton = null;

	private Button cancelButton = null;

	private Composite buttonComposite = null;

	private Composite parentComposite = null;

	private EnginePreferences installation = null;

	private List installations = null;

	private ECLiPSeEngineInstallation engineInstallation = null;

	private ECLiPSeEngineSettings engineSettings = null;

	private ECLiPSeGCSettings gcSettings = null;
}
