/*******************************************************************************
 * Copyright (c) 2006 Cisco Technology, Inc and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Author: Andrew Cheadle, CrossCore Optimization Limited
 *
 * Contributors:
 *     Cisco Technology, Inc - Initial implementation
 *******************************************************************************/

package org.eclipseclp.saros.core.preferences;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import java.util.List;

public class ECLiPSeEnginePreferencesShell {

	private Shell enginePreferencesShell = null;

	private ECLiPSeEnginePreferences enginePreferencesComposite = null;

	private EnginePreferences installation = null;

	private List installations = null;

	public ECLiPSeEnginePreferencesShell(List installations,
			EnginePreferences installation) {
		this.installations = installations;
		this.installation = installation;
	}
	
	public ECLiPSeEnginePreferencesShell(EnginePreferences installation) {
		this.installations = null;
		this.installation = installation;
	}

	/**
	 * This method initializes enginePreferencesShell
	 * 
	 * The shell is a modal window, so that the parent window is blocked until
	 * the selection is made or cancelled The input parameter title contains the
	 * string that is displayed as the window title
	 */
	private void createEnginePreferencesShell(Shell parent) {
		enginePreferencesShell = new Shell(parent, SWT.DIALOG_TRIM
				| SWT.APPLICATION_MODAL);
		enginePreferencesShell.setText(PreferenceMessages
				.getString("ECLiPSeStrings.ECLiPSe")
				+ PreferenceMessages
						.getString("ECLiPSeStrings.ECLiPSeEnginePreferences")); //$NON-NLS-1$
		GridLayout layout = new GridLayout();
		GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL
				| GridData.VERTICAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		enginePreferencesShell.setLayout(layout);
		enginePreferencesShell.setLayoutData(gridData);
		enginePreferencesComposite = new ECLiPSeEnginePreferences(
				enginePreferencesShell, SWT.NONE, true, installations,
				installation);
		enginePreferencesShell.setSize(enginePreferencesShell.computeSize(
		enginePreferencesComposite.getSize().x + 8,
		enginePreferencesComposite.getSize().y + 4));
	}

	/**
	 * This method opens a modal ECLiPSeEnginePreferencesShell
	 * 
	 * @param parent
	 *            the parent window
	 */
	public EnginePreferences open(Shell parent) {
		// open shell
		createEnginePreferencesShell(parent);
		Point parentSize = parent.getSize();
		Point size = enginePreferencesShell.getSize();
		int x = (parentSize.x - size.x) / 2;
		int y = (parentSize.y - size.y) / 2;
		enginePreferencesShell.setLocation(parent.toDisplay(x, y));
		enginePreferencesShell.open();
		installation = enginePreferencesComposite.getInstallation();
		// block display until shell is closed
		Display display = Display.getDefault();
		while (!enginePreferencesShell.isDisposed()) {
			if (!display.readAndDispatch())
				display.sleep();
		}

		return installation;
	}

	/**
	 * main method, for debugging purposes
	 * 
	 * @param args
	 */
	public static void main(String[] args) {
		/*
		 * Before this is run, be sure to set up the launch configuration
		 * (Arguments->VM Arguments) for the correct SWT library path in order
		 * to run with the SWT dlls. The dlls are located in the SWT plugin jar.
		 * For example, on Windows the Eclipse SWT 3.1 plugin jar is:
		 * installation_directory\plugins\org.eclipse.swt.win32_3.1.0.jar
		 */
		Display display = Display.getDefault();
		ECLiPSeEnginePreferencesShell thisClass = new ECLiPSeEnginePreferencesShell(
				null, null);
		thisClass.open(null);
		while (!thisClass.enginePreferencesShell.isDisposed()) {
			if (!display.readAndDispatch())
				display.sleep();
		}
		display.dispose();
	}
}
