/*******************************************************************************
 * Copyright (c) 2006 Cisco Technology, Inc and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Author: Andrew Cheadle, CrossCore Optimization Limited
 *
 * Contributors:
 *     Cisco Technology, Inc - Initial implementation
 *******************************************************************************/

package org.eclipseclp.saros.core.preferences;

import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;

public class ECLiPSeGCSettings extends Composite {

	/**
	 * @param parent
	 * @param style
	 * @param installation
	 */
	public ECLiPSeGCSettings(Composite parent, int style,
			EnginePreferences installation) {
		super(parent, style);
		initialise(installation);
	}

	public EnginePreferences commit(EnginePreferences installation) {
		if (gcOnRadio.getSelection()) {
			installation
					.setFlag(PreferenceConstants.gc, PreferenceConstants.on);
		} else if (gcOffRadio.getSelection()) {
			installation.setFlag(PreferenceConstants.gc,
					PreferenceConstants.off);
		} else if (gcVerboseRadio.getSelection()) {
			installation.setFlag(PreferenceConstants.gc,
					PreferenceConstants.verbose);
		}

		if (gcPolicyAdaptiveRadio.getSelection()) {
			installation.setFlag(PreferenceConstants.gcPolicy,
					PreferenceConstants.adaptive);
		} else if (gcPolicyFixedRadio.getSelection()) {
			installation.setFlag(PreferenceConstants.gcPolicy,
					PreferenceConstants.fixed);
		}
		Integer value = new Integer(gcIntervalText.getText());
		installation.setFlag(PreferenceConstants.gcInterval, value);
		value = new Integer(gcIntervalDictText.getText());
		installation.setFlag(PreferenceConstants.gcIntervalDict, value);
		return installation;
	}

	public EnginePreferences revertToDefaults(EnginePreferences installation) {
		String[] flagNames = new String[] { PreferenceConstants.gc,
				PreferenceConstants.gcPolicy, PreferenceConstants.gcInterval,
				PreferenceConstants.gcIntervalDict };
		PreferenceManager.revertToDefault(installation, flagNames);
		setWidgetValues(installation);
		return installation;
	}

	/**
	 * This method initializes the Engine Preferences widget
	 * 
	 * @param flag:
	 *            true == show button composite
	 * 
	 * The widget contains a scrolled composite, since the list of radio buttons
	 * is quite long
	 * 
	 */
	private void initialise(EnginePreferences installation) {
		GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL
				| GridData.VERTICAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		this.setLayout(new GridLayout());
		this.setLayoutData(gridData);
		createGCSettingsGroup();
		setWidgetValues(installation);
		setSize(this.computeSize(SWT.DEFAULT, SWT.DEFAULT));
	}

	private void createGCSettingsGroup() {
		GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL
				| GridData.VERTICAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		gcSettingsGroup = new Group(this, SWT.NONE);
		gcSettingsGroup.setText(PreferenceMessages
				.getString("ECLiPSeStrings.ECLiPSeGarbageCollectorSettings")); //$NON-NLS-1$
		gcSettingsGroup.setLayout(new GridLayout());
		gcSettingsGroup.setLayoutData(gridData);
		createGCEntry();
		createGCpolicyEntry();
		createGCintervalEntry();
		createGCintervalDictEntry();
	}

	private void createGCEntry() {
		gcComposite = new Composite(gcSettingsGroup, SWT.NONE);
		gcLabel = new Label(gcComposite, SWT.NONE);
		gcLabel.setText(PreferenceConstants.gc + ":"); //$NON-NLS-1$
		gcOffRadio = new Button(gcComposite, SWT.RADIO);
		gcOffRadio.setText(PreferenceConstants.off); //$NON-NLS-1$
		gcOnRadio = new Button(gcComposite, SWT.RADIO);
		gcOnRadio.setText(PreferenceConstants.on); //$NON-NLS-1$
		gcVerboseRadio = new Button(gcComposite, SWT.RADIO);
		gcVerboseRadio.setText(PreferenceConstants.verbose); //$NON-NLS-1$
		RadioCompositeBuilder.build(gcComposite, gcLabel, gcOffRadio,
				gcOnRadio, gcVerboseRadio);
	}

	private void createGCpolicyEntry() {
		gcPolicyComposite = new Composite(gcSettingsGroup, SWT.NONE);
		gcPolicyLabel = new Label(gcPolicyComposite, SWT.NONE);
		gcPolicyLabel.setText(PreferenceConstants.gcPolicy + ":"); //$NON-NLS-1$
		gcPolicyAdaptiveRadio = new Button(gcPolicyComposite, SWT.RADIO);
		gcPolicyAdaptiveRadio.setText(PreferenceConstants.adaptive); //$NON-NLS-1$
		gcPolicyFixedRadio = new Button(gcPolicyComposite, SWT.RADIO);
		gcPolicyFixedRadio.setText(PreferenceConstants.fixed); //$NON-NLS-1$
		RadioCompositeBuilder.build(gcPolicyComposite, gcPolicyLabel,
				gcPolicyAdaptiveRadio, gcPolicyFixedRadio, null);
	}

	private void createGCintervalEntry() {
		gcIntervalComposite = new Composite(gcSettingsGroup, SWT.NONE);
		GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		GridLayout layout = new GridLayout();
		layout.numColumns = 4;
		gcIntervalComposite.setLayout(layout);
		gcIntervalComposite.setLayoutData(gridData);
		gcIntervalLabel = new Label(gcIntervalComposite, SWT.NONE);
		gcIntervalLabel.setText(PreferenceConstants.gcInterval + ":"); //$NON-NLS-1$
		RadioCompositeBuilder.buildLabel(gcIntervalLabel);
		gridData = new GridData();
		gridData.grabExcessHorizontalSpace = true;
		gridData.horizontalAlignment = org.eclipse.swt.layout.GridData.FILL;
		gridData.horizontalSpan = 3;
		gcIntervalText = new Text(gcIntervalComposite, SWT.BORDER);
		gcIntervalText.setLayoutData(gridData);
	}

	private void createGCintervalDictEntry() {
		gcIntervalDictComposite = new Composite(gcSettingsGroup, SWT.NONE);
		GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		GridLayout layout = new GridLayout();
		layout.numColumns = 4;
		gcIntervalDictComposite.setLayout(layout);
		gcIntervalDictComposite.setLayoutData(gridData);
		gcIntervalDictLabel = new Label(gcIntervalDictComposite, SWT.NONE);
		gcIntervalDictLabel.setText(PreferenceConstants.gcIntervalDict + ":"); //$NON-NLS-1$
		RadioCompositeBuilder.buildLabel(gcIntervalDictLabel);
		gridData = new GridData();
		gridData.grabExcessHorizontalSpace = true;
		gridData.horizontalAlignment = org.eclipse.swt.layout.GridData.FILL;
		gridData.horizontalSpan = 3;
		gcIntervalDictText = new Text(gcIntervalDictComposite, SWT.BORDER);
		gcIntervalDictText.setLayoutData(gridData);
	}

	public void setWidgetValues(EnginePreferences installation) {
		String gc = installation.getStringFlag(PreferenceConstants.gc);
		if (gc.equals(PreferenceConstants.on)) {
			gcOnRadio.setSelection(true);
			gcOffRadio.setSelection(false);
			gcVerboseRadio.setSelection(false);
		} else if (gc.equals(PreferenceConstants.off)) {
			gcOnRadio.setSelection(false);
			gcOffRadio.setSelection(true);
			gcVerboseRadio.setSelection(false);
		} else if (gc.equals(PreferenceConstants.verbose)) {
			gcOnRadio.setSelection(false);
			gcOffRadio.setSelection(false);
			gcVerboseRadio.setSelection(true);
		} else {
			gcOnRadio.setSelection(false);
			gcOffRadio.setSelection(false);
			gcVerboseRadio.setSelection(false);
		}
		String gcPolicy = installation
				.getStringFlag(PreferenceConstants.gcPolicy);
		if (gcPolicy.equals(PreferenceConstants.adaptive)) {
			gcPolicyAdaptiveRadio.setSelection(true);
			gcPolicyFixedRadio.setSelection(false);
		} else if (gcPolicy.equals(PreferenceConstants.fixed)) {
			gcPolicyAdaptiveRadio.setSelection(false);
			gcPolicyFixedRadio.setSelection(true);
		} else {
			gcPolicyAdaptiveRadio.setSelection(false);
			gcPolicyFixedRadio.setSelection(false);
		}
		gcIntervalText.setText(""
				+ installation.getIntFlag(PreferenceConstants.gcInterval));
		gcIntervalDictText.setText(""
				+ installation.getIntFlag(PreferenceConstants.gcIntervalDict));
	}

	public void setEnabled(boolean enabled) {
		gcSettingsGroup.setEnabled(enabled);
		gcLabel.setEnabled(enabled);
		gcOffRadio.setEnabled(enabled);
		gcOnRadio.setEnabled(enabled);
		gcVerboseRadio.setEnabled(enabled);
		gcPolicyLabel.setEnabled(enabled);
		gcPolicyAdaptiveRadio.setEnabled(enabled);
		gcPolicyFixedRadio.setEnabled(enabled);
		gcIntervalLabel.setEnabled(enabled);
		gcIntervalDictLabel.setEnabled(enabled);
		gcIntervalText.setEnabled(enabled);
		gcIntervalDictText.setEnabled(enabled);
	}

	private Group gcSettingsGroup = null;

	private Composite gcComposite = null;

	private Label gcLabel = null;

	private Button gcOffRadio = null;

	private Button gcOnRadio = null;

	private Button gcVerboseRadio = null;

	private Composite gcPolicyComposite = null;

	private Label gcPolicyLabel = null;

	private Button gcPolicyAdaptiveRadio = null;

	private Button gcPolicyFixedRadio = null;

	private Composite gcIntervalComposite = null;

	private Label gcIntervalLabel = null;

	private Composite gcIntervalDictComposite = null;

	private Label gcIntervalDictLabel = null;

	private Text gcIntervalText = null;

	private Text gcIntervalDictText = null;
}
