/*******************************************************************************
 * Copyright (c) 2006 Cisco Technology, Inc and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Author: Andrew Cheadle, CrossCore Optimization Limited
 *
 * Contributors:
 *     Cisco Technology, Inc - Initial implementation
 *******************************************************************************/

package org.eclipseclp.saros.core.preferences;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.preference.PreferencePage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;

/**
 * @author Andrew Cheadle
 * 
 */
public class ECLiPSeOutputOptionsPage extends PreferencePage implements
		IWorkbenchPreferencePage {

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.preference.PreferencePage#createContents(org.eclipse.swt.widgets.Composite)
	 */
	protected Control createContents(Composite parent) {
		localInstallation = PreferenceManager.getDefaultInstallation();
		if (localInstallation == null) {
			localInstallation = PreferenceManager
					.getStoreDefaultInstallation();
		}
		outputOptions = new ECLiPSeOutputOptions(parent, SWT.NONE, true,
				localInstallation);
		return outputOptions;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.IWorkbenchPreferencePage#init(org.eclipse.ui.IWorkbench)
	 */
	public void init(IWorkbench workbench) {
	}

	protected void performDefaults() {
		EnginePreferences installation = PreferenceManager
				.getDefaultInstallation();
		if (installation == null) {
			MessageDialog
					.openError(
							this.getShell(),
							PreferenceMessages
									.getString("ECLiPSeStrings.Error"),
							PreferenceMessages
									.getString("ECLiPSeStrings.InstallationDefaultNotFoundError")
									+ ".");
			localInstallation = PreferenceManager
					.getStoreDefaultInstallation();
		} else {
			installation = outputOptions.revertToDefaults(installation);
		}
		super.performDefaults();
	}

	public boolean performOk() {
		EnginePreferences installation = PreferenceManager
				.getDefaultInstallation();

		if (installation == null) {
			MessageDialog
					.openError(
							this.getShell(),
							PreferenceMessages
									.getString("ECLiPSeStrings.Error"),
							PreferenceMessages
									.getString("ECLiPSeStrings.InstallationDefaultNotFoundError")
									+ ".");
			localInstallation = PreferenceManager
					.getStoreDefaultInstallation();
		} else {
			EnginePreferences installationClone = (EnginePreferences) installation
					.clone();
			installationClone = outputOptions.commit(installationClone);
			localInstallation = outputOptions.commit(localInstallation);
			if (!installation.equals(installationClone)
					&& !installationClone.equals(localInstallation)) {
				if (!queryApplyToDefaultInstallation()) {
					localInstallation = installation;
					return true;
				}
			}

			localInstallation = installationClone;
			PreferenceManager.setDefaultInstallation(installationClone);
		}
		return true;
	}

	private boolean queryApplyToDefaultInstallation() {
		MessageDialog dialog = new MessageDialog(
				this.getShell(),
				PreferenceMessages.getString("ECLiPSeStrings.ApplyChanges"),
				null,
				PreferenceMessages
						.getString("ECLiPSeStrings.InstallationDefaultChanged"),
				MessageDialog.QUESTION,
				new String[] {
						PreferenceMessages.getString("ECLiPSeStrings.Apply"),
						PreferenceMessages.getString("ECLiPSeStrings.Discard") },
				0);
		if (dialog.open() == 0) {
			return true;
		}

		return false;
	}

	private EnginePreferences localInstallation = null;

	private ECLiPSeOutputOptions outputOptions = null;
}
