/*******************************************************************************
 * Copyright (c) 2006 Cisco Technology, Inc and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Author: Andrew Cheadle, CrossCore Optimization Limited
 *
 * Contributors:
 *     Cisco Technology, Inc - Initial implementation
 *******************************************************************************/

package org.eclipseclp.saros.core.preferences;

import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipseclp.saros.core.Saros;
import java.io.File;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.List;
import java.util.Hashtable;

public class InstalledECLiPSeEngines extends Composite implements
		SelectionListener {

	private Table engineTable = null;

	private Group engineGroup = null;

	private Composite buttonComposite = null;

	private Button addButton = null;

	private Button editButton = null;

	private Button copyButton = null;

	private Button removeButton = null;

	private Button searchButton = null;

	private boolean searchCancelled = false;

	private int checkedItem = -1;

	private List installations = null;

	public InstalledECLiPSeEngines(Composite parent, int style) {
		super(parent, style);
		initialize();
	}

	public boolean commit() {
		EnginePreferences installation = null;
		TableItem items[] = engineTable.getItems();
		for (int item = 0; item < items.length; item++) {
			for (int engine = 0; engine < installations.size(); engine++) {
				installation = (EnginePreferences) installations.get(engine);
				if (installation.getInstallationName().equals(
						items[item].getText(0))) {
					PreferenceManager.storeInstallation(installation, item);
					break;
				}
			}
		}
		PreferenceManager.setDefaultInstallationNumber(checkedItem);
		PreferenceManager.setNumberOfInstallations(items.length);
		return true;
	}

	public void revertToDefaults() {
		EnginePreferences installation = null;
		TableItem items[] = engineTable.getSelection();
		for (int item = 0; item < items.length; item++) {
			for (int engine = 0; engine < installations.size(); engine++) {
				installation = (EnginePreferences) installations.get(engine);
				if (installation.getInstallationName().equals(
						items[item].getText(0))) {
					restoreInstallationDefaults(installation);
					break;
				}
			}
		}
	}

	private void initialize() {
		createEngineGroup();
		this.setLayout(new FillLayout());
		setSize(new org.eclipse.swt.graphics.Point(479, 403));
	}

	/**
	 * This method initializes engineTable
	 * 
	 */
	private void createEngineTable() {
		GridData gridData = new GridData();
		gridData.grabExcessHorizontalSpace = true;
		gridData.horizontalAlignment = org.eclipse.swt.layout.GridData.FILL;
		gridData.verticalAlignment = org.eclipse.swt.layout.GridData.FILL;
		gridData.grabExcessVerticalSpace = true;
		engineTable = new Table(engineGroup, SWT.BORDER | SWT.CHECK | SWT.MULTI
				| SWT.FULL_SELECTION);
		engineTable.setHeaderVisible(true);
		engineTable.setLayoutData(gridData);
		engineTable.setLinesVisible(true);
		TableColumn engineNameColumn = new TableColumn(engineTable, SWT.NONE);
		engineNameColumn.setWidth(100);
		engineNameColumn.setText(PreferenceMessages
				.getString("ECLiPSeStrings.Name"));
		TableColumn engineVersionColumn = new TableColumn(engineTable, SWT.NONE);
		engineVersionColumn.setWidth(60);
		engineVersionColumn.setText(PreferenceMessages
				.getString("ECLiPSeStrings.Version"));
		TableColumn engineLocationColumn = new TableColumn(engineTable,
				SWT.NONE);
		engineLocationColumn.setWidth(200);
		engineLocationColumn.setText(PreferenceMessages
				.getString("ECLiPSeStrings.Location"));
		installations = PreferenceManager.getInstallations();
		for (int engine = 0; engine < installations.size(); engine++) {
			TableItem item = new TableItem(engineTable, SWT.LEFT);
			EnginePreferences installation = (EnginePreferences) installations
					.get(engine);
			item.setText(0, installation.getInstallationName());
			item.setText(1, installation.getInstallationVersion());
			item.setText(2, installation.getInstallationPath());
		}

		int defaultEngine = PreferenceManager
				.getDefaultInstallationNumber();
		if (defaultEngine != -1) {
			TableItem item = engineTable.getItem(defaultEngine);
			item.setChecked(true);
			checkedItem = defaultEngine;
		}
		engineTable.addSelectionListener(this);
	}

	/**
	 * This method initializes engineGroup
	 * 
	 */
	private void createEngineGroup() {
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
		engineGroup = new Group(this, SWT.NONE);
		engineGroup.setText(PreferenceMessages
				.getString("ECLiPSeStrings.InstalledECLiPSeEngines"));
		engineGroup.setLayout(gridLayout);
		createEngineTable();
		createButtonComposite();
	}

	/**
	 * This method initializes buttonComposite
	 * 
	 */
	private void createButtonComposite() {
		GridData gridData3 = new GridData();
		gridData3.horizontalAlignment = org.eclipse.swt.layout.GridData.FILL;
		gridData3.grabExcessHorizontalSpace = true;
		GridLayout gridLayout1 = new GridLayout();
		gridLayout1.verticalSpacing = 5;
		GridData gridData2 = new GridData();
		gridData2.grabExcessHorizontalSpace = true;
		gridData2.horizontalAlignment = org.eclipse.swt.layout.GridData.FILL;
		GridData gridData1 = new GridData();
		gridData1.horizontalAlignment = org.eclipse.swt.layout.GridData.FILL;
		gridData1.grabExcessHorizontalSpace = true;
		buttonComposite = new Composite(engineGroup, SWT.NONE);
		buttonComposite.setLayout(gridLayout1);
		addButton = new Button(buttonComposite, SWT.NONE);
		addButton.setText(PreferenceMessages.getString("ECLiPSeStrings.Add")
				+ "...");
		addButton.setLayoutData(gridData2);
		addButton.addSelectionListener(this);
		editButton = new Button(buttonComposite, SWT.NONE);
		editButton.setText(PreferenceMessages.getString("ECLiPSeStrings.Edit")
				+ "...");
		editButton.setLayoutData(gridData1);
		editButton.setEnabled(false);
		editButton.addSelectionListener(this);
		copyButton = new Button(buttonComposite, SWT.NONE);
		copyButton.setText(PreferenceMessages.getString("ECLiPSeStrings.Copy")
				+ "...");
		copyButton.setLayoutData(gridData1);
		copyButton.setEnabled(false);
		copyButton.addSelectionListener(this);
		removeButton = new Button(buttonComposite, SWT.NONE);
		removeButton.setText(PreferenceMessages
				.getString("ECLiPSeStrings.Remove")
				+ "...");
		removeButton.addSelectionListener(this);
		removeButton.setEnabled(false);
		searchButton = new Button(buttonComposite, SWT.NONE);
		searchButton.setText(PreferenceMessages
				.getString("ECLiPSeStrings.Search")
				+ "...");
		searchButton.setLayoutData(gridData3);
		searchButton.addSelectionListener(this);
	}

	private void restoreInstallationDefaults(EnginePreferences installation) {
		EnginePreferences newInstallation = new EnginePreferences(installation
				.getInstallationName(), installation.getInstallationPath(),
				installation.getLibraryPath(), installation
						.getEngineArguments());
		installations.remove(installation);
		if ((newInstallation = PreferenceManager.initialiseInstallation(
				newInstallation, true)) != null) {
			installations.add(newInstallation);
		}
	}

	public void widgetDefaultSelected(SelectionEvent event) {
	}

	public void widgetSelected(SelectionEvent event) {
		if (event.getSource() == addButton) {
			addInstallation(null);
		} else if (event.getSource() == editButton) {
			// Only one item can be selected if the edit button has been
			// selected
			String installationName = engineTable.getSelection()[0].getText(0);
			EnginePreferences installation = findInstallation(installationName);
			if (installation == null) {
				MessageDialog
						.openError(
								this.getShell(),
								PreferenceMessages
										.getString("ECLiPSeStrings.InternalError"),
								PreferenceMessages
										.getString("ECLiPSeStrings.InstallationNotFoundError"
												+ ": " + installationName)
										+ ".");
				return;
			}
			installation = (new ECLiPSeEnginePreferencesShell(installations,
					installation)).open(this.getShell());
			engineTable.getSelection()[0].setText(0, installation
					.getInstallationName());
		} else if (event.getSource() == copyButton) {
			// Only one item can be selected if the copy button has been
			// selected
			String installationName = engineTable.getSelection()[0].getText(0);
			EnginePreferences installation = findInstallation(installationName);
			if (installation == null) {
				MessageDialog
						.openError(
								this.getShell(),
								PreferenceMessages
										.getString("ECLiPSeStrings.InternalError"),
								PreferenceMessages
										.getString("ECLiPSeStrings.InstallationNotFoundError"
												+ ": " + installationName)
										+ ".");
				return;
			}
			EnginePreferences newInstallation = new EnginePreferences(
					installation.getInstallationName(), installation
							.getInstallationPath(), installation
							.getLibraryPath(), installation
							.getEngineArguments());
			addInstallation(newInstallation);
			newInstallation.setFlags((Hashtable) installation.getFlags()
					.clone());
		} else if (event.getSource() == removeButton) {
			editButton.setEnabled(false);
			copyButton.setEnabled(false);
			removeButton.setEnabled(false);
			TableItem items[] = engineTable.getSelection();
			EnginePreferences installation = null;
			boolean removedDefault = false;
			for (int cnt = 0; cnt < items.length; cnt++) {
				for (int engine = 0; engine < installations.size(); engine++) {
					installation = (EnginePreferences) installations
							.get(engine);
					if (installation.getInstallationName().equals(
							items[cnt].getText(0))) {
						installations.remove(engine);
						if (items[cnt].getChecked()) {
							removedDefault = true;
						}
						break;
					}
				}
			}
			engineTable.remove(engineTable.getSelectionIndices());
			if (removedDefault) {
				if (engineTable.getItemCount() > 0) {
					TableItem item = engineTable.getItem(0);
					item.setChecked(true);
					checkedItem = 0;
				} else {
					checkedItem = -1;
				}
			}
		} else if (event.getSource() == searchButton) {
			searchForECLiPSeEngineInstallations();
		} else if (event.getSource() == engineTable) {
			if (event.detail == SWT.CHECK) {
				TableItem item = (TableItem) event.item;
				int index = engineTable.indexOf(item);
				if (index == checkedItem) {
					if (!item.getChecked()) {
						checkedItem = -1;
					}
				} else {
					if (checkedItem != -1) {
						TableItem checked = engineTable.getItem(checkedItem);
						checked.setChecked(false);
						checkedItem = -1;
					}
					checkedItem = index;
				}
			} else if (engineTable.getSelectionCount() > 0) {
				if (engineTable.getSelectionCount() == 1) {
					copyButton.setEnabled(true);
					editButton.setEnabled(true);
				} else {
					copyButton.setEnabled(false);
					editButton.setEnabled(false);
				}
				removeButton.setEnabled(true);
			}
		}
	}

	private EnginePreferences findInstallation(String installationName) {
		EnginePreferences installation = null;
		for (int engine = 0; engine < installations.size(); engine++) {
			installation = (EnginePreferences) installations.get(engine);
			if (installation.getInstallationName().equals(installationName)) {
				return installation;
			}
		}
		return null;
	}

	private void addInstallation(EnginePreferences initialisedInstallation) {
		ECLiPSeEngineInstallationShell eis = new ECLiPSeEngineInstallationShell(
				installations, initialisedInstallation);
		EnginePreferences installation = eis.open(this.getShell());
		if (installation.getInstallationName().equals("")) {
			return;
		}
		TableItem item = new TableItem(engineTable, SWT.LEFT);
		item.setText(0, installation.getInstallationName());
		item.setText(1, installation.getInstallationVersion());
		item.setText(2, installation.getInstallationPath());
		if (engineTable.getItemCount() == 1) {
			item.setChecked(true);
			checkedItem = 0;
		}
		installations.add(installation);
	}

	private void searchForECLiPSeEngineInstallations() {
		final List existingNames = new ArrayList();
		final List existingPaths = new ArrayList();
		final List newInstallations = new ArrayList();

		for (int engine = 0; engine < installations.size(); engine++) {
			EnginePreferences installation = (EnginePreferences) installations
					.get(engine);
			existingNames.add(installation.getInstallationName());
			existingPaths.add(installation.getInstallationPath());
		}

		DirectoryDialog dialog = new DirectoryDialog(this.getShell(),
				SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL);
		dialog.setText("Select a directory to search in:");
		dialog.setFilterPath("");
		String path = dialog.open();
		if (path == null) {
			return;
		}
		searchCancelled = false;

		final File rootDir = new File(path);
		ProgressMonitorDialog pm = new ProgressMonitorDialog(this.getShell());

		IRunnableWithProgress r = new IRunnableWithProgress() {
			public void run(IProgressMonitor monitor) {
				monitor.beginTask(PreferenceMessages
						.getString("ECLiPSeStrings.SearchingECLiPSeEngines"),
						IProgressMonitor.UNKNOWN);
				search(rootDir, existingNames, existingPaths, newInstallations,
						monitor);
				monitor.done();
				if (monitor.isCanceled())
					searchCancelled = true;
			}
		};

		try {
			pm.run(true, true, r);
		} catch (InvocationTargetException ite) {
			Status status = new Status(IStatus.ERROR, "Saros", 0,
					"Invocation Exception occurred running monitor:", ite);

			ErrorDialog.openError(Saros.getDefault().getWorkbench()
					.getDisplay().getActiveShell(), "Saros Core Plugin Error",
					ite.getMessage(), status);

			System.err
					.println("Invoctation Exception occurred running monitor: "
							+ ite.getMessage());
			ite.printStackTrace();
			return;
		} catch (InterruptedException ite) {
			Status status = new Status(IStatus.ERROR, "Saros", 0,
					"Interrupted Exception occurred running monitor:", ite);

			ErrorDialog.openError(Saros.getDefault().getWorkbench()
					.getDisplay().getActiveShell(), "Saros Core Plugin Error",
					ite.getMessage(), status);

			System.err
					.println("Interrupted Exception occurred running monitor: "
							+ ite.getMessage());
			ite.printStackTrace();
			return;
		}

		if (searchCancelled) {
			return;
		}

		installations.addAll(newInstallations);
		for (int engine = 0; engine < newInstallations.size(); engine++) {
			final EnginePreferences installation = (EnginePreferences) newInstallations
					.get(engine);
			Runnable ir = new Runnable() {
				public void run() {
					TableItem item = new TableItem(engineTable, SWT.LEFT);
					item.setText(0, installation.getInstallationName());
					item.setText(1, installation.getInstallationVersion());
					item.setText(2, installation.getInstallationPath());
					if (engineTable.getItemCount() == 1) {
						item.setChecked(true);
						checkedItem = 0;
					}
				}
			};
			this.getDisplay().syncExec(ir);
		}
	}

	private void search(File directory, List existingNames, List existingPaths,
			List newInstallations, IProgressMonitor monitor) {
		if (monitor.isCanceled())
			return;

		String[] names = directory.list();
		List subDirs = new ArrayList();

		for (int i = 0; i < names.length; i++) {
			if (monitor.isCanceled())
				return;

			File file = new File(directory, names[i]);

			if (file.getName().equals(PreferenceConstants.joopBootEco)) {
				File parentFile = file.getParentFile();
				if (parentFile != null) {
					if (parentFile.getName().equals(PreferenceConstants.lib)) {
						parentFile = parentFile.getParentFile();
						if (parentFile != null) {
							try {
								String iname = parentFile.getName();
								if (existingPaths.contains(parentFile
										.getCanonicalPath())) {
									continue;
								}

								if (existingNames.contains(iname)) {
									int cnt = 1;
									do {
										iname = PreferenceMessages
												.getString("ECLiPSeStrings.ECLiPSeInstallation")
												+ " " + cnt;
										cnt++;
									} while (existingNames.contains(iname));
								}
								existingNames.add(iname);
								existingPaths
										.add(parentFile.getCanonicalPath());
								EnginePreferences newInstallation = new EnginePreferences(
										iname, parentFile.getCanonicalPath(),
										"", "");
								if ((newInstallation = PreferenceManager
										.initialiseInstallation(
												newInstallation, false)) != null) {
									newInstallations.add(newInstallation);
								}
							} catch (IOException ioe) {
							}
						}
					}
				}
			}

			try {
				monitor.subTask(PreferenceMessages
						.getString("ECLiPSeStrings.Found")
						+ " "
						+ newInstallations.size()
						+ ". "
						+ PreferenceMessages
								.getString("ECLiPSeStrings.Searching")
						+ ": "
						+ file.getCanonicalPath());
			} catch (IOException ioe) {
			}

			if (file.isDirectory()) {
				if (monitor.isCanceled())
					return;
				subDirs.add(file);

			}
		}
		while (!subDirs.isEmpty()) {
			File subDir = (File) subDirs.remove(0);
			search(subDir, existingNames, existingPaths, newInstallations,
					monitor);
			if (monitor.isCanceled()) {
				return;
			}
		}

	}
} // @jve:decl-index=0:visual-constraint="10,10"
