/*******************************************************************************
 * Copyright (c) 2006 Cisco Technology, Inc and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Author: Andrew Cheadle, CrossCore Optimization Limited
 *
 * Contributors:
 *     Cisco Technology, Inc - Initial implementation
 *******************************************************************************/

package org.eclipseclp.saros.core.preferences;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.QualifiedName;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Link;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.dialogs.PreferencesUtil;
import org.eclipseclp.saros.core.Saros;
import org.eclipseclp.saros.core.preferences.PreferenceMessages;
import org.eclipseclp.saros.core.preferences.PreferenceConstants;

public class LaunchQueryComposite extends Composite implements
		SelectionListener {

	public LaunchQueryComposite(Composite parent, int style, boolean fileLocal) {
		super(parent, style);
		initialise(fileLocal);
	}

	public LaunchQueryComposite(Composite parent, int style, boolean fileLocal,
			IResource resource) {
		super(parent, style);
		this.resource = resource;
		setProject(resource.getProject());
		initialise(fileLocal);
	}

	private void initialise(boolean fileLocal) {
		this.fileLocal = fileLocal;
		GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL
				| GridData.VERTICAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		GridLayout layout = new GridLayout();
		layout.numColumns = 2;
		this.setLayout(layout);
		this.setLayoutData(gridData);
		if (fileLocal) {
			createFileLocalWidgets();
			createQueryGroup();
			setEnabled(false);
		} else {
			createQueryGroup();
		}
		setWidgetValues();
		this.setSize(this.computeSize(SWT.DEFAULT, SWT.DEFAULT));
	}

	private void createFileLocalWidgets() {
		fileSpecificCheckbox = new Button(this, SWT.CHECK);
		fileSpecificCheckbox.setText(PreferenceMessages
				.getString("ECLiPSeStrings.ProjectLaunchSettings"));
		fileSpecificCheckbox.addSelectionListener(this);
		GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		fileSpecificCheckbox.setLayoutData(gridData);
		configureProjectPropertiesLink = new Link(this, SWT.NONE);
		configureProjectPropertiesLink.setText("<A>"
				+ PreferenceMessages
						.getString("ECLiPSeStrings.ConfigureProjectSettings")
				+ "...</A>");
		configureProjectPropertiesLink.addSelectionListener(this);
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_END);
		configureProjectPropertiesLink.setLayoutData(gridData);
		Label separator = new Label(this, SWT.SEPARATOR | SWT.HORIZONTAL);
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		gridData.horizontalSpan = 2;
		separator.setLayoutData(gridData);
	}

	private void createQueryGroup() {
		GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		gridData.horizontalSpan = 2;
		GridLayout layout = new GridLayout();
		layout.numColumns = 2;
		layout.verticalSpacing = 8;
		launchQueryGroup = new Group(this, SWT.NONE);
		launchQueryGroup.setText(PreferenceMessages
				.getString("ECLiPSeStrings.LaunchQuery")); //$NON-NLS-1$
		launchQueryGroup.setLayout(layout);
		launchQueryGroup.setLayoutData(gridData);
		noQueryRadio = new Button(launchQueryGroup, SWT.RADIO);
		noQueryRadio.setText(PreferenceMessages
				.getString("ECLiPSeStrings.None"));
		gridData = new GridData();
		gridData.grabExcessHorizontalSpace = true;
		gridData.horizontalSpan = 2;
		gridData.horizontalAlignment = org.eclipse.swt.layout.GridData.FILL;
		noQueryRadio.setLayoutData(gridData);
		noQueryRadio.addSelectionListener(this);
		fileQueryRadio = new Button(launchQueryGroup, SWT.RADIO);
		fileQueryRadio.setText(PreferenceMessages
				.getString("ECLiPSeStrings.FileName")
				+ ":");
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		fileQueryRadio.setLayoutData(gridData);
		fileQueryRadio.addSelectionListener(this);
		fileNameText = new Text(launchQueryGroup, SWT.BORDER);
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		fileNameText.setLayoutData(gridData);
		moduleQueryRadio = new Button(launchQueryGroup, SWT.RADIO);
		moduleQueryRadio.setText(PreferenceMessages
				.getString("ECLiPSeStrings.ModuleName")
				+ ":");
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		moduleQueryRadio.setLayoutData(gridData);
		moduleQueryRadio.addSelectionListener(this);
		moduleNameText = new Text(launchQueryGroup, SWT.BORDER);
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		moduleNameText.setLayoutData(gridData);
		runMethodCombo = new Combo(launchQueryGroup, SWT.DROP_DOWN
				| SWT.READ_ONLY);
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		runMethodCombo.setLayoutData(gridData);
		runMethodCombo.setBackground(Display.getDefault().getSystemColor(
				SWT.COLOR_WIDGET_BACKGROUND));
		runMethodCombo.add(PreferenceMessages.getString("ECLiPSeStrings.Run"));
		runMethodCombo.add(PreferenceMessages
				.getString("ECLiPSeStrings.TimeProfile"));
		runMethodCombo.add(PreferenceMessages
				.getString("ECLiPSeStrings.PortProfile"));
		runMethodCombo.addSelectionListener(this);
		queryText = new Text(launchQueryGroup, SWT.BORDER);
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		queryText.setLayoutData(gridData);
	}

	private void setWidgetValues() {
		if (resource == null) {
			noQueryRadio.setSelection(true);
			fileQueryRadio.setSelection(false);
			moduleQueryRadio.setSelection(false);
			moduleNameText.setEnabled(false);
			fileNameText.setEnabled(false);
			queryText.setEnabled(false);
			runMethodCombo.setText(PreferenceMessages
					.getString("ECLiPSeStrings.Run"));
			runMethodCombo.setEnabled(false);
		} else {
			try {
				fileNameText.setText(resource
						.getPersistentProperty(new QualifiedName(
								PreferenceConstants.SarosPlugin,
								PreferenceConstants.LaunchFile)));
				moduleNameText.setText(resource
						.getPersistentProperty(new QualifiedName(
								PreferenceConstants.SarosPlugin,
								PreferenceConstants.LaunchModule)));
				if (resource.getPersistentProperty(
						new QualifiedName(PreferenceConstants.SarosPlugin,
								PreferenceConstants.HasLaunchQuery)).equals(
						PreferenceConstants.False)) {
					noQueryRadio.setSelection(true);
					moduleQueryRadio.setSelection(false);
					fileQueryRadio.setSelection(false);
					moduleNameText.setEnabled(false);
					fileNameText.setEnabled(false);
					queryText.setEnabled(false);
				} else {
					noQueryRadio.setSelection(false);
					if (fileNameText.getText().equals("")) {
						fileQueryRadio.setSelection(false);
						fileNameText.setEnabled(false);
						moduleQueryRadio.setSelection(true);
					} else {
						fileQueryRadio.setSelection(true);
						fileNameText.setEnabled(true);
						moduleQueryRadio.setSelection(false);
					}
					moduleNameText.setEnabled(true);
					queryText.setEnabled(true);
				}
				queryText.setText(resource
						.getPersistentProperty(new QualifiedName(
								PreferenceConstants.SarosPlugin,
								PreferenceConstants.LaunchQuery)));
				String runMethod = resource
						.getPersistentProperty(new QualifiedName(
								PreferenceConstants.SarosPlugin,
								PreferenceConstants.LaunchRunMode));
				if (runMethod.equals("")) {
					runMethod = PreferenceMessages
							.getString("ECLiPSeStrings.Run");
				}
				runMethodCombo.setText(runMethod);
				if (fileLocal) {
					if (resource.getPersistentProperty(
							new QualifiedName(PreferenceConstants.SarosPlugin,
									PreferenceConstants.FileLaunchQuery))
							.equals(PreferenceConstants.True)) {
						setEnabled(true);
						configureProjectPropertiesLink.setEnabled(false);
						fileSpecificCheckbox.setSelection(true);
					} else {
						setEnabled(false);
						configureProjectPropertiesLink.setEnabled(true);
						fileSpecificCheckbox.setSelection(false);
					}
				}
			} catch (CoreException e) {
				Status status = new Status(
						IStatus.ERROR,
						"Saros",
						0,
						"Exception caught whilst retrieving resource properties: ",
						e);
				ErrorDialog.openError(Saros.getDefault().getWorkbench()
						.getDisplay().getActiveShell(),
						"Saros Development Plugin Error", e.getMessage(),
						status);
			}
		}
	}

	public void setWidgetValues(boolean hasLaunchQuery, String launchFile,
			String launchModule, String runMethod, String launchQuery) {
		if (!hasLaunchQuery) {
			noQueryRadio.setSelection(true);
			moduleQueryRadio.setSelection(false);
			fileQueryRadio.setSelection(false);
			moduleNameText.setEnabled(false);
			fileNameText.setEnabled(false);
			queryText.setEnabled(false);
		} else {
			noQueryRadio.setSelection(false);
			if (launchFile.equals("")) {
				fileQueryRadio.setSelection(false);
				fileNameText.setEnabled(false);
				moduleQueryRadio.setSelection(true);
			} else {
				fileQueryRadio.setSelection(true);
				fileNameText.setEnabled(true);
				moduleQueryRadio.setSelection(false);
			}
			moduleNameText.setEnabled(true);
			queryText.setEnabled(true);
		}
		queryText.setText(launchQuery);
		fileNameText.setText(launchFile);
		moduleNameText.setText(launchModule);

		if (runMethod.equals("")) {
			runMethod = PreferenceMessages.getString("ECLiPSeStrings.Run");
		}
		runMethodCombo.setText(runMethod);
	}

	public void widgetDefaultSelected(SelectionEvent event) {
	}

	public void widgetSelected(SelectionEvent event) {
		if (event.getSource() == fileSpecificCheckbox) {
			if (fileSpecificCheckbox.getSelection()) {
				setEnabled(true);
				configureProjectPropertiesLink.setEnabled(false);
			} else {
				setEnabled(false);
				configureProjectPropertiesLink.setEnabled(true);
			}
		} else if (event.getSource() == configureProjectPropertiesLink) {
			PreferencesUtil
					.createPropertyDialogOn(
							getShell(),
							project,
							PreferenceConstants.ProjectLaunchDefaultsPropertyPageId,
							new String[] { PreferenceConstants.ProjectLaunchDefaultsPropertyPageId },
							null).open();
		} else if (event.getSource() == noQueryRadio) {
			noQueryRadio.setSelection(true);
			moduleQueryRadio.setSelection(false);
			fileQueryRadio.setSelection(false);
			moduleNameText.setEnabled(false);
			fileNameText.setEnabled(false);
			queryText.setEnabled(false);
			runMethodCombo.setEnabled(false);
		} else if (event.getSource() == moduleQueryRadio) {
			noQueryRadio.setSelection(false);
			moduleQueryRadio.setSelection(true);
			fileQueryRadio.setSelection(false);
			moduleNameText.setEnabled(true);
			fileNameText.setEnabled(false);
			fileNameText.setText("");
			queryText.setEnabled(true);
			runMethodCombo.setEnabled(true);
		} else if (event.getSource() == fileQueryRadio) {
			noQueryRadio.setSelection(false);
			moduleQueryRadio.setSelection(false);
			fileQueryRadio.setSelection(true);
			moduleNameText.setEnabled(true);
			fileNameText.setEnabled(true);
			queryText.setEnabled(true);
			runMethodCombo.setEnabled(true);
		} else if (event.getSource() == runMethodCombo) {
			if (runMethodCombo.getText().equals(
					PreferenceMessages.getString("ECLiPSeStrings.Run"))) {
			} else if (runMethodCombo.getText().equals(
					PreferenceMessages.getString("ECLiPSeStrings.TimeProfile"))) {
			} else if (runMethodCombo.getText().equals(
					PreferenceMessages.getString("ECLiPSeStrings.PortProfile"))) {
			}
		}
	}

	public void commit() {
		hasLaunchQuery = !noQueryRadio.getSelection();
		if (noQueryRadio.getSelection()) {
			fileName = "";
			moduleName = "";
			query = "";
			runMethod = "";
			return;
		}
		fileName = fileNameText.getText();
		moduleName = moduleNameText.getText();
		query = queryText.getText();
		runMethod = runMethodCombo.getText();
		if (resource != null) {

		}
	}

	public void revertToDefaults() {
		// Resource can't be null when reverting defaults
		setEnabled(false);
		if (fileLocal) {
			try {
				configureProjectPropertiesLink.setEnabled(true);
				fileSpecificCheckbox.setSelection(false);
				fileNameText.setText(project
						.getPersistentProperty(new QualifiedName(
								PreferenceConstants.SarosPlugin,
								PreferenceConstants.LaunchFile)));
				moduleNameText.setText(project
						.getPersistentProperty(new QualifiedName(
								PreferenceConstants.SarosPlugin,
								PreferenceConstants.LaunchModule)));
				if (project.getPersistentProperty(
						new QualifiedName(PreferenceConstants.SarosPlugin,
								PreferenceConstants.HasLaunchQuery)).equals(
						PreferenceConstants.False)) {
					noQueryRadio.setSelection(true);
					moduleQueryRadio.setSelection(false);
					fileQueryRadio.setSelection(false);
				} else {
					noQueryRadio.setSelection(false);
					if (fileNameText.getText().equals("")) {
						fileQueryRadio.setSelection(false);
						moduleQueryRadio.setSelection(true);
					} else {
						fileQueryRadio.setSelection(true);
						moduleQueryRadio.setSelection(false);
					}
				}
				queryText.setText(project
						.getPersistentProperty(new QualifiedName(
								PreferenceConstants.SarosPlugin,
								PreferenceConstants.LaunchQuery)));
				runMethodCombo.setText(project
						.getPersistentProperty(new QualifiedName(
								PreferenceConstants.SarosPlugin,
								PreferenceConstants.LaunchRunMode)));
			} catch (CoreException e) {
				Status status = new Status(
						IStatus.ERROR,
						"Saros",
						0,
						"Exception caught whilst retrieving project properties: ",
						e);
				ErrorDialog.openError(Saros.getDefault().getWorkbench()
						.getDisplay().getActiveShell(),
						"Saros Development Plugin Error", e.getMessage(),
						status);
			}
		} else {
			noQueryRadio.setSelection(true);
			fileQueryRadio.setSelection(false);
			moduleQueryRadio.setSelection(false);
			runMethodCombo.setText(PreferenceMessages
					.getString("ECLiPSeStrings.Run"));
		}
	}

	public String getFileName() {
		return fileName;
	}

	public String getModuleName() {
		return moduleName;
	}

	public String getQuery() {
		return query;
	}

	public String getRunMethod() {
		return runMethod;
	}

	public void setEnabled(boolean enabled) {
		launchQueryGroup.setEnabled(enabled);
		noQueryRadio.setEnabled(enabled);
		if (enabled) {
			if (noQueryRadio.getSelection()) {
				moduleQueryRadio.setEnabled(true);
				fileQueryRadio.setEnabled(true);
				return;
			} else if (fileQueryRadio.getSelection()) {
				fileNameText.setEnabled(true);
			}
			moduleQueryRadio.setEnabled(true);
			fileQueryRadio.setEnabled(true);
			runMethodCombo.setEnabled(true);
			moduleNameText.setEnabled(true);
			queryText.setEnabled(true);
		} else {
			moduleQueryRadio.setEnabled(false);
			fileQueryRadio.setEnabled(false);
			runMethodCombo.setEnabled(false);
			moduleNameText.setEnabled(false);
			fileNameText.setEnabled(false);
			queryText.setEnabled(false);
		}
	}

	public void addModifyListener(ModifyListener listener,
			SelectionAdapter adapter) {
		moduleQueryRadio.addSelectionListener(adapter);
		fileQueryRadio.addSelectionListener(adapter);
		runMethodCombo.addSelectionListener(adapter);
		moduleNameText.addModifyListener(listener);
		fileNameText.addModifyListener(listener);
		queryText.addModifyListener(listener);
	}

	public static String getRunMethodData(String runMethod) {
		if (runMethod
				.equals(PreferenceMessages.getString("ECLiPSeStrings.Run"))) {
			return PreferenceConstants.call;
		} else if (runMethod.equals(PreferenceMessages
				.getString("ECLiPSeStrings.TimeProfile"))) {
			return PreferenceConstants.profile + ":"
					+ PreferenceConstants.profile;
		} else if (runMethod.equals(PreferenceMessages
				.getString("ECLiPSeStrings.PortProfile"))) {
			return PreferenceConstants.portProfiler + ":"
					+ PreferenceConstants.portProfile;
		}
		return null;
	}

	public static String getToplevelRunMethodData(String runMethod) {
		if (runMethod
				.equals(PreferenceMessages.getString("ECLiPSeStrings.Run"))) {
			return PreferenceConstants.call;
		} else if (runMethod.equals(PreferenceMessages
				.getString("ECLiPSeStrings.TimeProfile"))) {
			return PreferenceConstants.profile;
		} else if (runMethod.equals(PreferenceMessages
				.getString("ECLiPSeStrings.PortProfile"))) {
			return PreferenceConstants.portProfile;
		}
		return null;
	}

	public boolean hasLaunchQuery() {
		return hasLaunchQuery;
	}

	public void setProject(IProject project) {
		this.project = project;
	}

	public boolean isFileSpecificLaunchQuery() {
		return fileSpecificCheckbox.getSelection();
	}

	private Button fileSpecificCheckbox = null;

	private Link configureProjectPropertiesLink = null;

	private Group launchQueryGroup = null;

	private Button noQueryRadio = null;

	private Button moduleQueryRadio = null;

	private Button fileQueryRadio = null;

	private Combo runMethodCombo = null;

	private Text fileNameText = null;

	private Text moduleNameText = null;

	private Text queryText = null;

	private String fileName = "";

	private String moduleName = "";

	private String query = "";

	private String runMethod = "";

	private boolean hasLaunchQuery = false;

	private IResource resource = null;

	private IProject project = null;

	private boolean fileLocal = false;
}
