/*******************************************************************************
 * Copyright (c) 2006 Cisco Technology, Inc and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Author: Andrew Cheadle, CrossCore Optimization Limited
 *
 * Contributors:
 *     Cisco Technology, Inc - Initial implementation
 *******************************************************************************/

package org.eclipseclp.saros.core.preferences;

import org.eclipse.core.runtime.preferences.AbstractPreferenceInitializer;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipseclp.saros.core.Saros;
import com.parctechnologies.eclipse.*;
import org.eclipse.jface.util.IPropertyChangeListener;
import java.io.File;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.dialogs.ErrorDialog;
import java.util.Collection;
import java.util.ArrayList;
import java.util.List;
import org.eclipse.jface.dialogs.MessageDialog;
import java.io.IOException;

/**
 * Class used to initialize default preference values.
 */
public class PreferenceManager extends AbstractPreferenceInitializer {

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.core.runtime.preferences.AbstractPreferenceInitializer#initializeDefaultPreferences()
	 */
	public void initializeDefaultPreferences() {
		IPreferenceStore store = Saros.getDefault().getPreferenceStore();
		store.setDefault(PreferenceConstants.ECLiPSeEngineType,
				PreferenceConstants.outOfProcess);
		store.setDefault(PreferenceConstants.ECLiPSeLauncher, false);
		store.setDefault(PreferenceConstants.ECLiPSeFcompileTarget, true);
		store.setDefault(PreferenceConstants.ECLiPSeIcompileTarget, true);
		store.setDefault(PreferenceConstants.ECLiPSeEciToHtml, true);
		store.setDefault(PreferenceConstants.ECLiPSeOutputFolders,
				PreferenceConstants.ECLiPSeProject);
		store.setDefault(PreferenceConstants.ECLiPSeSourceFolderName,
				PreferenceConstants.source);
		store.setDefault(PreferenceConstants.ECLiPSeObjectFolderName,
				PreferenceConstants.object);
		store.setDefault(PreferenceConstants.ECLiPSeDocFolderName,
				PreferenceConstants.documentation);
		store.setDefault(PreferenceConstants.ECLiPSeHtmlFolderName,
				PreferenceConstants.html);
		store.setDefault(PreferenceConstants.numberInstalledEngines, 0);
		store.setDefault(PreferenceConstants.defaultInstalledEngine, -1);
		store.setDefault(PreferenceConstants.ECLiPSeInstallationName, "");
		store.setDefault(PreferenceConstants.ECLiPSeInstallationPath, "");
		store.setDefault(PreferenceConstants.ECLiPSeLibraryPath, "");
		store.setDefault(PreferenceConstants.ECLiPSeEngineArguments, "");
		store.setDefault(PreferenceConstants.version, "0.0 #0");
		store.setDefault(PreferenceConstants.afterEventTimer,
				PreferenceConstants.real);
		store.setDefault(PreferenceConstants.allDynamic,
				PreferenceConstants.off);
		store.setDefault(PreferenceConstants.brealExceptions,
				PreferenceConstants.off);
		store
				.setDefault(PreferenceConstants.coroutine,
						PreferenceConstants.off);
		store.setDefault(PreferenceConstants.debugging,
				PreferenceConstants.nodebug);
		store.setDefault(PreferenceConstants.debugCompile,
				PreferenceConstants.on);
		store.setDefault(PreferenceConstants.enableInterrupts,
				PreferenceConstants.on);
		store.setDefault(PreferenceConstants.goalExpansion,
				PreferenceConstants.on);
		store.setDefault(PreferenceConstants.macroExpansion,
				PreferenceConstants.on);
		store.setDefault(PreferenceConstants.preferRationals,
				PreferenceConstants.off);
		store.setDefault(PreferenceConstants.variableNames,
				PreferenceConstants.checkSingletons);
		store.setDefault(PreferenceConstants.outputMode,
				PreferenceConstants.outputModeDefault);
		store.setDefault(PreferenceConstants.printDepth,
				PreferenceConstants.printDepthDefault);
		store.setDefault(PreferenceConstants.gc, PreferenceConstants.on);
		store.setDefault(PreferenceConstants.gcPolicy,
				PreferenceConstants.adaptive);
		store.setDefault(PreferenceConstants.gcInterval,
				PreferenceConstants.gcIntervalDefault);
		store.setDefault(PreferenceConstants.gcIntervalDict,
				PreferenceConstants.gcIntervalDictDefault);
	}

	public static int getEngineType() {
		IPreferenceStore store = Saros.getDefault().getPreferenceStore();
		return store.getInt(PreferenceConstants.ECLiPSeEngineType);
	}

	public static boolean launchActionSelection() {
		IPreferenceStore store = Saros.getDefault().getPreferenceStore();
		return store.getBoolean(PreferenceConstants.ECLiPSeLauncher);
	}

	public static void setLaunchActionSelection(boolean launchSelection) {
		IPreferenceStore store = Saros.getDefault().getPreferenceStore();
		store.setValue(PreferenceConstants.ECLiPSeLauncher, launchSelection);
	}

	public static boolean launchActionSelectionDefault() {
		IPreferenceStore store = Saros.getDefault().getPreferenceStore();
		return store.getDefaultBoolean(PreferenceConstants.ECLiPSeLauncher);
	}

	public static boolean defaultProjectFolderOutput() {
		IPreferenceStore store = Saros.getDefault().getPreferenceStore();
		if (store.getDefaultString(PreferenceConstants.ECLiPSeOutputFolders)
				.equals(PreferenceConstants.ECLiPSeProject)) {
			return true;
		}

		return false;
	}

	public static boolean projectFolderOutput() {
		IPreferenceStore store = Saros.getDefault().getPreferenceStore();
		if (store.getString(PreferenceConstants.ECLiPSeOutputFolders).equals(
				PreferenceConstants.ECLiPSeProject)) {
			return true;
		}

		return false;
	}

	public static void setProjectFolderOutput(boolean project) {
		IPreferenceStore store = Saros.getDefault().getPreferenceStore();
		if (project) {
			store.setValue(PreferenceConstants.ECLiPSeOutputFolders,
					PreferenceConstants.ECLiPSeProject);
		} else {
			store.setValue(PreferenceConstants.ECLiPSeOutputFolders,
					PreferenceConstants.ECLiPSeFolders);
		}
	}

	public static String[] getDefaultOutputFolders() {
		IPreferenceStore store = Saros.getDefault().getPreferenceStore();
		String[] outputFolders = new String[4];
		outputFolders[0] = store
				.getDefaultString(PreferenceConstants.ECLiPSeSourceFolderName);
		outputFolders[1] = store
				.getDefaultString(PreferenceConstants.ECLiPSeObjectFolderName);
		outputFolders[2] = store
				.getDefaultString(PreferenceConstants.ECLiPSeDocFolderName);
		outputFolders[3] = store
				.getDefaultString(PreferenceConstants.ECLiPSeHtmlFolderName);

		return outputFolders;
	}

	public static String[] getOutputFolders() {
		IPreferenceStore store = Saros.getDefault().getPreferenceStore();
		String[] outputFolders = new String[4];
		outputFolders[0] = store
				.getString(PreferenceConstants.ECLiPSeSourceFolderName);
		outputFolders[1] = store
				.getString(PreferenceConstants.ECLiPSeObjectFolderName);
		outputFolders[2] = store
				.getString(PreferenceConstants.ECLiPSeDocFolderName);
		outputFolders[3] = store
				.getString(PreferenceConstants.ECLiPSeHtmlFolderName);

		return outputFolders;
	}

	public static void setOutputFolders(String[] outputFolders) {
		IPreferenceStore store = Saros.getDefault().getPreferenceStore();
		store.setValue(PreferenceConstants.ECLiPSeSourceFolderName,
				outputFolders[0]);
		store.setValue(PreferenceConstants.ECLiPSeObjectFolderName,
				outputFolders[1]);
		store.setValue(PreferenceConstants.ECLiPSeDocFolderName,
				outputFolders[2]);
		store.setValue(PreferenceConstants.ECLiPSeHtmlFolderName,
				outputFolders[3]);
	}

	public static boolean[] getDefaultOutputCompilationTargets() {
		IPreferenceStore store = Saros.getDefault().getPreferenceStore();
		boolean[] targets = new boolean[3];
		targets[0] = store
				.getDefaultBoolean(PreferenceConstants.ECLiPSeFcompileTarget);
		targets[1] = store
				.getDefaultBoolean(PreferenceConstants.ECLiPSeIcompileTarget);
		targets[2] = store
				.getDefaultBoolean(PreferenceConstants.ECLiPSeEciToHtml);

		return targets;
	}

	public static boolean[] getOutputCompilationTargets() {
		IPreferenceStore store = Saros.getDefault().getPreferenceStore();
		boolean[] targets = new boolean[3];
		targets[0] = store
				.getBoolean(PreferenceConstants.ECLiPSeFcompileTarget);
		targets[1] = store
				.getBoolean(PreferenceConstants.ECLiPSeIcompileTarget);
		targets[2] = store.getBoolean(PreferenceConstants.ECLiPSeEciToHtml);

		return targets;
	}

	public static void setOutputCompilationTargets(boolean[] targets) {
		IPreferenceStore store = Saros.getDefault().getPreferenceStore();
		store.setValue(PreferenceConstants.ECLiPSeFcompileTarget, targets[0]);
		store.setValue(PreferenceConstants.ECLiPSeIcompileTarget, targets[1]);
		store.setValue(PreferenceConstants.ECLiPSeEciToHtml, targets[2]);
	}

	public static EnginePreferences getDefaultInstallation() {
		IPreferenceStore store = Saros.getDefault().getPreferenceStore();
		int defaultEngine = store
				.getInt(PreferenceConstants.defaultInstalledEngine);
		if (defaultEngine == -1) {
			return null;
		}
		return getInstallation(defaultEngine);
	}

	public static void setDefaultInstallation(EnginePreferences installation) {
		IPreferenceStore store = Saros.getDefault().getPreferenceStore();
		int defaultEngine = store
				.getInt(PreferenceConstants.defaultInstalledEngine);
		if (defaultEngine == -1) {
			return;
		}
		setInstallation(defaultEngine, installation);
	}

	public static String getDefaultInstallationName() {
		IPreferenceStore store = Saros.getDefault().getPreferenceStore();
		int defaultEngine = store
				.getInt(PreferenceConstants.defaultInstalledEngine);
		if (defaultEngine == -1) {
			return "";
		}
		return store.getString(PreferenceConstants.ECLiPSeInstallationName
				+ defaultEngine);
	}

	public static String getDefaultInstallationPath() {
		IPreferenceStore store = Saros.getDefault().getPreferenceStore();
		int defaultEngine = store
				.getInt(PreferenceConstants.defaultInstalledEngine);
		if (defaultEngine == -1) {
			return "";
		}
		return store.getString(PreferenceConstants.ECLiPSeInstallationPath
				+ defaultEngine);
	}

	public static int getDefaultInstallationNumber() {
		IPreferenceStore store = Saros.getDefault().getPreferenceStore();
		return store.getInt(PreferenceConstants.defaultInstalledEngine);
	}

	public static void setDefaultInstallationNumber(int number) {
		IPreferenceStore store = Saros.getDefault().getPreferenceStore();
		store.setValue(PreferenceConstants.defaultInstalledEngine, number);
	}

	public static void setNumberOfInstallations(int number) {
		IPreferenceStore store = Saros.getDefault().getPreferenceStore();
		store.setValue(PreferenceConstants.numberInstalledEngines, number);
	}

	public static int getNumberOfInstallations() {
		IPreferenceStore store = Saros.getDefault().getPreferenceStore();
		return store.getInt(PreferenceConstants.numberInstalledEngines);
	}

	public static List getInstallations() {
		IPreferenceStore store = Saros.getDefault().getPreferenceStore();
		List installations = new ArrayList();
		int numEngines = store
				.getInt(PreferenceConstants.numberInstalledEngines);

		for (int engine = 0; engine < numEngines; engine++) {
			installations.add(getInstallation(engine));
		}

		return installations;
	}

	public static EnginePreferences getInstallationByName(String name) {
		IPreferenceStore store = Saros.getDefault().getPreferenceStore();
		int numEngines = store
				.getInt(PreferenceConstants.numberInstalledEngines);

		for (int engine = 0; engine < numEngines; engine++) {
			if (name.equals(store
					.getString(PreferenceConstants.ECLiPSeInstallationName
							+ engine))) {
				return getInstallation(engine);
			}
		}

		return null;
	}

	public static String[] getInstallationNames() {
		IPreferenceStore store = Saros.getDefault().getPreferenceStore();
		int numEngines = store
				.getInt(PreferenceConstants.numberInstalledEngines);
		String[] installationNames = new String[numEngines];

		for (int engine = 0; engine < numEngines; engine++) {
			installationNames[engine] = store
					.getString(PreferenceConstants.ECLiPSeInstallationName
							+ engine);
		}

		return installationNames;
	}

	public static String[] getInstallationPaths() {
		IPreferenceStore store = Saros.getDefault().getPreferenceStore();
		int numEngines = store
				.getInt(PreferenceConstants.numberInstalledEngines);
		String[] installationPaths = new String[numEngines];

		for (int engine = 0; engine < numEngines; engine++) {
			installationPaths[engine] = store
					.getString(PreferenceConstants.ECLiPSeInstallationPath
							+ engine);
		}

		return installationPaths;
	}

	public static EnginePreferences getInstallation(int engineNum) {
		IPreferenceStore store = Saros.getDefault().getPreferenceStore();
		if (engineNum < 0
				|| engineNum >= store
						.getInt(PreferenceConstants.numberInstalledEngines)) {
			return null;
		}

		EnginePreferences installation = new EnginePreferences(store
				.getString(PreferenceConstants.ECLiPSeInstallationName
						+ engineNum), store
				.getString(PreferenceConstants.ECLiPSeInstallationPath
						+ engineNum), store
				.getString(PreferenceConstants.ECLiPSeLibraryPath + engineNum),
				store.getString(PreferenceConstants.ECLiPSeEngineArguments
						+ engineNum));

		installation.setInstallationVersion(store
				.getString(PreferenceConstants.version + engineNum));

		for (int flag = 0; flag < EnginePreferences.flagNames.length; flag++) {
			switch (EnginePreferences.flagTypes[flag]) {
			case EnginePreferences.ATOM:
			case EnginePreferences.STRING:
				installation.setFlag(EnginePreferences.flagNames[flag], store
						.getString(EnginePreferences.flagNames[flag]
								+ engineNum));
				break;
			case EnginePreferences.INTEGER:
				Integer value = new Integer(store
						.getInt(EnginePreferences.flagNames[flag] + engineNum));
				installation.setFlag(EnginePreferences.flagNames[flag], value);
				break;
			}
		}

		return installation;
	}

	public static void setInstallation(int engineNum,
			EnginePreferences installation) {
		IPreferenceStore store = Saros.getDefault().getPreferenceStore();
		if (engineNum < 0
				|| engineNum >= store
						.getInt(PreferenceConstants.numberInstalledEngines)) {
			return;
		}

		store.setValue(PreferenceConstants.ECLiPSeInstallationName + engineNum,
				installation.getInstallationName());
		store.setValue(PreferenceConstants.ECLiPSeInstallationPath + engineNum,
				installation.getInstallationPath());
		store.setValue(PreferenceConstants.ECLiPSeLibraryPath + engineNum,
				installation.getLibraryPath());
		store.setValue(PreferenceConstants.ECLiPSeEngineArguments + engineNum,
				installation.getEngineArguments());
		store.setValue(PreferenceConstants.version + engineNum, installation
				.getInstallationVersion());

		for (int flag = 0; flag < EnginePreferences.flagNames.length; flag++) {
			switch (EnginePreferences.flagTypes[flag]) {
			case EnginePreferences.ATOM:
			case EnginePreferences.STRING:
				store
						.setValue(
								EnginePreferences.flagNames[flag] + engineNum,
								installation
										.getStringFlag(EnginePreferences.flagNames[flag]));
				break;
			case EnginePreferences.INTEGER:
				store.setValue(EnginePreferences.flagNames[flag] + engineNum,
						installation
								.getIntFlag(EnginePreferences.flagNames[flag]));
				break;
			}
		}
	}

	public static EnginePreferences initialiseInstallation(
			EnginePreferences installation, boolean reportExceptions) {
		IPreferenceStore store = Saros.getDefault().getPreferenceStore();
		try {
			String version = null;
			EclipseEngineOptions eclipseEngineOptions = new EclipseEngineOptions(
					new File(installation.getInstallationPath()));
			eclipseEngineOptions.setUseQueues(false);
			EclipseEngine eclipseEngine = new OutOfProcessEclipse(
					eclipseEngineOptions);
			Collection versionCollection = (Collection) queryEngineFlag(store,
					PreferenceConstants.versionAsList, EnginePreferences.LIST,
					eclipseEngine, reportExceptions);
			Object[] versionInfo = versionCollection.toArray();
			if (versionInfo.length != 3) {
				version = versionCollection.toString();
			} else {
				version = versionInfo[0] + "." + versionInfo[1] + " #"
						+ versionInfo[2];
				int[] versionInts = new int[3];
				versionInts[0] = new Integer(versionInfo[0].toString())
						.intValue();
				versionInts[1] = new Integer(versionInfo[1].toString())
						.intValue();
				versionInts[2] = new Integer(versionInfo[2].toString())
						.intValue();
				if (versionInts[0] < PreferenceConstants.ECLiPSeSupportedMajorVersion
						|| (versionInts[0] == PreferenceConstants.ECLiPSeSupportedMajorVersion && versionInts[1] < PreferenceConstants.ECLiPSeSupportedMinorVersion)
						|| (versionInts[0] == PreferenceConstants.ECLiPSeSupportedMajorVersion
								&& versionInts[1] == PreferenceConstants.ECLiPSeSupportedMinorVersion && versionInts[2] < PreferenceConstants.ECLiPSeSupportedBuildVersion)) {
					if (reportExceptions) {
						Status status = new Status(
								IStatus.ERROR,
								"Saros",
								0,
								"The ECLiPSe version number is not supported. Please upgrade your version of ECLiPSe.",
								null);

						ErrorDialog
								.openError(
										Saros.getDefault().getWorkbench()
												.getDisplay().getActiveShell(),
										"Saros Plugin Error",
										"The selected ECLiPSe installation is not supported for use with the Saros Plugin",
										status);
					}
					return null;
				}
			}

			List libraryPathItems = getEngineLibraryPath(store, eclipseEngine,
					reportExceptions);
			if (libraryPathItems == null) {
				return null;
			}

			String libraryPath = processLibraryPath(installation,
					libraryPathItems);
			if (libraryPath == null) {
				return null;
			}

			for (int flag = 0; flag < EnginePreferences.flagNames.length; flag++) {
				Object value = queryEngineFlag(store,
						EnginePreferences.flagNames[flag],
						EnginePreferences.flagTypes[flag], eclipseEngine,
						reportExceptions);
				installation.setFlag(EnginePreferences.flagNames[flag], value);
			}

			((OutOfProcessEclipse) eclipseEngine).destroy();

			installation.setLibraryPath(libraryPath);
			installation.setInstallationVersion(version);

		} catch (Exception e) {
			if (reportExceptions) {
				Status status = new Status(IStatus.ERROR, "Saros", 0,
						"Exception caught in Saros ECLiPSe "
								+ "engine while initialising installation: ", e);

				ErrorDialog.openError(Saros.getDefault().getWorkbench()
						.getDisplay().getActiveShell(),
						"Saros Core Plugin Error", e.getMessage(), status);

				System.err.println("Exception caught in Saros ECLiPSe "
						+ "engine while initialising installation: "
						+ e.getMessage());
				e.printStackTrace();
			}
			installation = null;
		}
		return installation;
	}

	private static List getEngineLibraryPath(IPreferenceStore store,
			EclipseEngine eclipseEngine, boolean reportExceptions) {
		ArrayList libraryPath = null;

		try {
			CompoundTerm result = eclipseEngine.rpc(":",
					new Atom("tracer_tcl"), new CompoundTermImpl(
							"saros_get_library_path", (Object) null));
			libraryPath = new ArrayList((Collection) ((CompoundTerm) result
					.arg(2)).arg(1));
		} catch (Exception exception) {
			if (reportExceptions) {
				Status status = new Status(
						IStatus.ERROR,
						"Saros",
						0,
						"Exception caught in Saros ECLiPSe "
								+ "engine while retrieving engine library path: ",
						exception);

				ErrorDialog.openError(Saros.getDefault().getWorkbench()
						.getDisplay().getActiveShell(),
						"Saros Core Plugin Error", exception.getMessage(),
						status);

				System.err.println("Exception caught in Saros ECLiPSe "
						+ "engine while retrieving engine library path: "
						+ exception.getMessage());
				exception.printStackTrace();
			}
		}
		return libraryPath;
	}

	private static String processLibraryPath(EnginePreferences installation,
			List libraryPathItems) {
		String libraryPath = "";
		String canonicalPath = null;
		if (libraryPathItems.size() > 0) {
			for (int dir = 0; dir < libraryPathItems.size(); dir++) {
				String libraryPathItem = (String) libraryPathItems.get(dir);
				if (libraryPathItem.equals("")) {
					continue;
				}
				File path = new File(libraryPathItem);
				try {
					canonicalPath = path.getCanonicalPath();
				} catch (IOException ioe) {
				}
				if (!path.isDirectory()) {
					MessageDialog
							.openError(
									null,
									PreferenceMessages
											.getString("ECLiPSeStrings.Error"),
									canonicalPath
											+ " "
											+ PreferenceMessages
													.getString("ECLiPSeStrings.LibraryPathError")
											+ ".");
					return null;
				}
				libraryPathItems.set(dir, canonicalPath);

				libraryPath = libraryPath.concat(libraryPathItem + ",");
			}
		}
		String[] userLibraryPaths = installation.getLibraryPath().split(",");
		for (int dir = 0; dir < userLibraryPaths.length; dir++) {
			if (userLibraryPaths[dir].equals("")) {
				continue;
			}
			File path = new File(userLibraryPaths[dir]);
			try {
				canonicalPath = path.getCanonicalPath();
			} catch (IOException ioe) {
			}
			if (!path.isDirectory()) {
				MessageDialog.openError(null, PreferenceMessages
						.getString("ECLiPSeStrings.Error"), canonicalPath
						+ " "
						+ PreferenceMessages
								.getString("ECLiPSeStrings.LibraryPathError")
						+ ".");
				return null;
			}

			if (libraryPathItems.contains(canonicalPath)) {
				continue;
			}

			libraryPath = libraryPath.concat(canonicalPath + ",");
		}

		// Tidy up the libraryPath
		if (libraryPath.endsWith(",")) {
			libraryPath = libraryPath.substring(0, libraryPath.length() - 1);
		}

		return libraryPath;
	}

	private static Object queryEngineFlag(IPreferenceStore store, String flag,
			int type, EclipseEngine eclipseEngine, boolean reportExceptions) {
		Object value = null;

		try {
			switch (type) {
			case EnginePreferences.ATOM:
				value = ((Atom) (eclipseEngine.rpc("get_flag", new Atom(flag),
						null).arg(2))).functor();
				break;
			case EnginePreferences.INTEGER:
			case EnginePreferences.STRING:
			case EnginePreferences.LIST:
				value = eclipseEngine.rpc("get_flag", new Atom(flag), null)
						.arg(2);
				break;
			}
		} catch (Exception exception) {
			value = store.getDefaultString(flag);
			if (reportExceptions) {
				Status status = new Status(
						IStatus.ERROR,
						"Saros",
						0,
						"Exception caught in Saros ECLiPSe "
								+ "engine while initialising engine settings: ",
						exception);

				ErrorDialog.openError(Saros.getDefault().getWorkbench()
						.getDisplay().getActiveShell(),
						"Saros Core Plugin Error", exception.getMessage(),
						status);

				System.err.println("Exception caught in Saros ECLiPSe "
						+ "engine while initialising engine settings: "
						+ exception.getMessage());
				exception.printStackTrace();
			}
		}

		return value;
	}

	public static void storeInstallation(EnginePreferences installation,
			int engineNumber) {
		IPreferenceStore store = Saros.getDefault().getPreferenceStore();
		store.setValue(PreferenceConstants.ECLiPSeInstallationName
				+ engineNumber, installation.getInstallationName());
		store.setValue(PreferenceConstants.ECLiPSeInstallationPath
				+ engineNumber, installation.getInstallationPath());
		store.setValue(PreferenceConstants.ECLiPSeLibraryPath + engineNumber,
				installation.getLibraryPath());
		store.setValue(PreferenceConstants.ECLiPSeEngineArguments
				+ engineNumber, installation.getEngineArguments());
		store.setValue(PreferenceConstants.version + engineNumber, installation
				.getInstallationVersion());
		for (int flag = 0; flag < EnginePreferences.flagNames.length; flag++) {
			Object value = installation
					.getFlag(EnginePreferences.flagNames[flag]);
			if (EnginePreferences.flagTypes[flag] == EnginePreferences.INTEGER) {
				store.setValue(
						EnginePreferences.flagNames[flag] + engineNumber,
						((Integer) value).intValue());
			} else {
				store.setValue(
						EnginePreferences.flagNames[flag] + engineNumber,
						(String) value);
			}
		}
	}

	public static void removeInstallation(String installationName) {
		IPreferenceStore store = Saros.getDefault().getPreferenceStore();
		int numEngines = store
				.getInt(PreferenceConstants.numberInstalledEngines);
		boolean relocate = false;
		boolean defaultRemoved = false;

		for (int engine = 0; engine < numEngines; engine++) {
			if (relocate) {
				int place = engine - 1;
				store.setValue(PreferenceConstants.ECLiPSeInstallationName
						+ place, store
						.getString(PreferenceConstants.ECLiPSeInstallationName
								+ engine));
				store.setValue(PreferenceConstants.ECLiPSeInstallationPath
						+ place, store
						.getString(PreferenceConstants.ECLiPSeInstallationPath
								+ engine));
				store.setValue(PreferenceConstants.ECLiPSeLibraryPath + place,
						store.getString(PreferenceConstants.ECLiPSeLibraryPath
								+ engine));
				store.setValue(PreferenceConstants.ECLiPSeEngineArguments
						+ place, store
						.getString(PreferenceConstants.ECLiPSeEngineArguments
								+ engine));
				store.setValue(PreferenceConstants.version + place, store
						.getString(PreferenceConstants.version + engine));
				for (int flag = 0; flag < EnginePreferences.flagNames.length; flag++) {
					if (EnginePreferences.flagTypes[flag] == EnginePreferences.ATOM
							|| EnginePreferences.flagTypes[flag] == EnginePreferences.STRING) {
						store.setValue(EnginePreferences.flagNames[flag]
								+ place, store
								.getString(EnginePreferences.flagNames[flag]
										+ engine));
					} else {
						store.setValue(EnginePreferences.flagNames[flag]
								+ place, store
								.getInt(EnginePreferences.flagNames[flag]
										+ engine));
					}
				}
			} else {
				String retrievedName = store
						.getString(PreferenceConstants.ECLiPSeInstallationName
								+ engine);
				if (retrievedName.equals(installationName)) {
					relocate = true;
					if (engine == store
							.getInt(PreferenceConstants.defaultInstalledEngine)) {
						defaultRemoved = true;
					}
					continue;
				}
			}
		}

		numEngines--;

		if (defaultRemoved && numEngines > 0) {
			// Always set first entry as default if there is one
			store.setValue(PreferenceConstants.defaultInstalledEngine, 1);
		} else {
			store.setValue(PreferenceConstants.defaultInstalledEngine, -1);
		}
		store.setValue(PreferenceConstants.numberInstalledEngines, numEngines);
	}

	public static void revertToDefault(EnginePreferences installation,
			String[] flagNames) {
		EnginePreferences defaultInstallation = new EnginePreferences(
				installation.getInstallationName(), installation
						.getInstallationPath(), "", "");
		defaultInstallation = initialiseInstallation(defaultInstallation, true);
		if (defaultInstallation == null) {
			return;
		}

		for (int flag = 0; flag < flagNames.length; flag++) {
			installation.setFlag(flagNames[flag], defaultInstallation
					.getFlag(flagNames[flag]));
		}
	}

	public static EnginePreferences getStoreDefaultInstallation() {
		IPreferenceStore store = Saros.getDefault().getPreferenceStore();

		EnginePreferences installation = new EnginePreferences(
				store
						.getDefaultString(PreferenceConstants.ECLiPSeInstallationName),
				store
						.getDefaultString(PreferenceConstants.ECLiPSeInstallationPath),
				store.getDefaultString(PreferenceConstants.ECLiPSeLibraryPath),
				store
						.getDefaultString(PreferenceConstants.ECLiPSeEngineArguments));

		installation.setInstallationVersion(store
				.getDefaultString(PreferenceConstants.version));

		for (int flag = 0; flag < EnginePreferences.flagNames.length; flag++) {
			switch (EnginePreferences.flagTypes[flag]) {
			case EnginePreferences.ATOM:
			case EnginePreferences.STRING:
				installation.setFlag(EnginePreferences.flagNames[flag], store
						.getDefaultString(EnginePreferences.flagNames[flag]));
				break;
			case EnginePreferences.INTEGER:
				Integer value = new Integer(store
						.getDefaultInt(EnginePreferences.flagNames[flag]));
				installation.setFlag(EnginePreferences.flagNames[flag], value);
				break;
			}
		}

		return installation;
	}

	public static void addPreferenceStoreChangeListener(
			IPropertyChangeListener listener) {
		IPreferenceStore store = Saros.getDefault().getPreferenceStore();
		store.addPropertyChangeListener(listener);
	}
}
