/*******************************************************************************
 * Copyright (c) 2006 Cisco Technology, Inc and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Author: Andrew Cheadle, CrossCore Optimization Limited
 *
 * Contributors:
 *     Cisco Technology, Inc - Initial implementation
 *******************************************************************************/

package org.eclipseclp.saros.core.preferences;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

public class PrintDepthOutputOptionsComposite extends Composite implements
		SelectionListener {

	private EnginePreferences installation = null;

	public PrintDepthOutputOptionsComposite(Composite parent, int style,
			EnginePreferences installation) {
		super(parent, style);
		parentComposite = parent;
		this.installation = installation;
		this.outputDisplay = true;
		initialise();
	}

	public PrintDepthOutputOptionsComposite(Composite parent, int style,
			boolean outputDisplay, EnginePreferences installation) {
		super(parent, style);
		parentComposite = parent;
		this.installation = installation;
		this.outputDisplay = outputDisplay;
		initialise();
	}

	public EnginePreferences commit(EnginePreferences installation) {
		Integer printDepth = new Integer(printDepthText.getText());
		installation.setFlag(PreferenceConstants.printDepth, printDepth);

		String outputModeString = outputModeText.getText();
		if (!outputDisplay) {
			if (outputModeString.indexOf(PreferenceConstants.fullDepthChar) != -1) {
				outputModeString = installation
						.getStringFlag(PreferenceConstants.outputMode);
				outputModeString += PreferenceConstants.fullDepthChar;
			}
		}
		installation.setFlag(PreferenceConstants.outputMode, outputModeString);
		this.installation = installation;
		return installation;
	}

	public EnginePreferences revertToDefaults(EnginePreferences installation) {
		String[] flagNames = new String[] { PreferenceConstants.printDepth,
				PreferenceConstants.outputMode };
		PreferenceManager.revertToDefault(installation, flagNames);
		this.installation = installation;
		setWidgetValues();
		return installation;
	}

	private void initialise() {
		GridLayout layout = new GridLayout();
		GridData gridData = new GridData();
		gridData.horizontalAlignment = org.eclipse.swt.layout.GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		gridData.verticalAlignment = org.eclipse.swt.layout.GridData.FILL;
		this.setLayout(layout);
		this.setLayoutData(gridData);
		// Bit of a hack to that blank canvas isn't left with a hidden output
		// mode display
		if (outputDisplay) {
			createOutputModeEntry();
			createPrintDepthEntry();
		} else {
			createPrintDepthEntry();
			createOutputModeEntry();
		}
		setWidgetValues();
		setSize(this.computeSize(SWT.DEFAULT, SWT.DEFAULT, true));
	}

	/**
	 * This method creates the printDepth entry
	 * 
	 */
	private void createPrintDepthEntry() {
		Composite radioGroupComposite = new Composite(this, SWT.NONE);
		GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		GridLayout layout = new GridLayout();
		layout.numColumns = 4;
		radioGroupComposite.setLayout(layout);
		radioGroupComposite.setLayoutData(gridData);
		printDepthLabel = new Label(radioGroupComposite, SWT.NONE);
		printDepthLabel.setText(PreferenceConstants.printDepth + ":"); //$NON-NLS-1$
		RadioCompositeBuilder.buildLabel(printDepthLabel);
		printDepthText = new Text(radioGroupComposite, SWT.BORDER | SWT.CENTER);
		gridData = new GridData();
		gridData.grabExcessHorizontalSpace = true;
		gridData.horizontalAlignment = org.eclipse.swt.layout.GridData.FILL;
		gridData.horizontalSpan = 2;
		printDepthText.setLayoutData(gridData);
		printDepthButton = new Button(radioGroupComposite, SWT.NONE);
		printDepthButton.setText(PreferenceMessages
				.getString("ECLiPSeStrings.Change") + "..."); //$NON-NLS-1$
		printDepthButton.addSelectionListener(this);
	}

	/**
	 * This method creates the outputMode entry
	 * 
	 */
	private void createOutputModeEntry() {
		Composite radioGroupComposite = new Composite(this, SWT.NONE);
		GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		GridLayout layout = new GridLayout();
		layout.numColumns = 4;
		radioGroupComposite.setLayout(layout);
		radioGroupComposite.setLayoutData(gridData);
		outputModeLabel = new Label(radioGroupComposite, SWT.NONE);
		outputModeLabel.setText(PreferenceConstants.outputMode + ":"); //$NON-NLS-1$
		RadioCompositeBuilder.buildLabel(outputModeLabel);
		gridData = new GridData();
		gridData.grabExcessHorizontalSpace = true;
		gridData.horizontalAlignment = org.eclipse.swt.layout.GridData.FILL;
		gridData.horizontalSpan = 2;
		outputModeText = new Text(radioGroupComposite, SWT.BORDER | SWT.CENTER);
		outputModeText.setBackground(Display.getDefault().getSystemColor(
				SWT.COLOR_WIDGET_BACKGROUND));
		outputModeText.setEditable(false);
		outputModeText.setLayoutData(gridData);
		outputModeButton = new Button(radioGroupComposite, SWT.NONE);
		outputModeButton.setText(PreferenceMessages
				.getString("ECLiPSeStrings.Change") + "..."); //$NON-NLS-1$
		outputModeButton.addSelectionListener(this);
		if (!outputDisplay) {
			radioGroupComposite.setVisible(false);
		}
	}

	public void widgetDefaultSelected(SelectionEvent event) {
	}

	public void widgetSelected(SelectionEvent event) {
		if (event.getSource() == outputModeButton) {
			EnginePreferences dummyInstallation = new EnginePreferences();
			dummyInstallation.setFlag(PreferenceConstants.outputMode,
					outputModeText.getText());
			ECLiPSeOutputOptionsShell outputOptionsShell = new ECLiPSeOutputOptionsShell(
					dummyInstallation);
			String outputMode = outputOptionsShell.open(parentComposite
					.getShell());
			outputModeText.setText(outputMode);
		} else if (event.getSource() == printDepthButton) {
			boolean fullDepth = false;
			String outputModeString = outputModeText.getText();
			if (outputModeString.indexOf(PreferenceConstants.fullDepthChar) != -1) {
				fullDepth = true;
			}
			ECLiPSeChangeDepthShell printDepthShell = new ECLiPSeChangeDepthShell();
			int value = printDepthShell.open(parentComposite.getShell(),
					ECLiPSeChangeDepthShell.PRINT, (new Integer(printDepthText
							.getText())).intValue(), fullDepth);
			if (value == -1) {
				// Print depth is overridden by 'full' checkbox
				if (!fullDepth) {
					outputModeString += PreferenceConstants.fullDepthChar;
					outputModeText.setText(outputModeString);
					printDepthText.setEnabled(false);
				}
			} else {
				printDepthText.setText("" + value);
				// Remove the full print depth override
				if (fullDepth) {
					String newOutputModeString = "";
					for (int index = 0; index < outputModeString.length(); index++) {
						char chr = outputModeString.charAt(index);
						if (chr == PreferenceConstants.fullDepthChar) {
							continue;
						}
						newOutputModeString += chr;
					}
					outputModeText.setText(newOutputModeString);
					printDepthText.setEnabled(true);
				}
			}
		}
	}

	private void setWidgetValues() {
		printDepthText.setText(""
				+ installation.getIntFlag(PreferenceConstants.printDepth));
		outputModeText.setText(installation
				.getStringFlag(PreferenceConstants.outputMode));
		if (outputModeText.getText().indexOf(PreferenceConstants.fullDepthChar) != -1) {
			printDepthText.setEnabled(false);
		} else {
			printDepthText.setEnabled(true);
		}
	}

	public void setEnabled(boolean enabled) {
		outputModeLabel.setEnabled(enabled);
		outputModeText.setEnabled(enabled);
		outputModeButton.setEnabled(enabled);
		printDepthLabel.setEnabled(enabled);
		printDepthText.setEnabled(enabled);
		printDepthButton.setEnabled(enabled);
	}

	private Label outputModeLabel = null;

	private Text outputModeText = null;

	private Button outputModeButton = null;

	private Label printDepthLabel = null;

	private Text printDepthText = null;

	private Button printDepthButton = null;

	private Composite parentComposite = null;

	private boolean outputDisplay = true;
}
