/*******************************************************************************
 * Copyright (c) 2006 Cisco Technology, Inc and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Author: Andrew Cheadle, CrossCore Optimization Limited
 *
 * Contributors:
 *     Cisco Technology, Inc - Initial implementation
 *******************************************************************************/

package org.eclipseclp.saros.core.preferences;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;

public class PrintDepthOutputOptionsShell implements SelectionListener {

	private Shell outputOptionsShell = null;

	private ECLiPSeOutputOptions outputOptionsComposite = null;

	private EnginePreferences installation = null;

	private Button okButton = null;

	private Button restoreButton = null;

	private Button cancelButton = null;

	public PrintDepthOutputOptionsShell(EnginePreferences installation) {
		this.installation = installation;
	}

	/**
	 * This method initializes outputOptionsShell
	 * 
	 * The shell is a modal window, so that the parent window is blocked until
	 * the selection is made or cancelled The input parameter title contains the
	 * string that is displayed as the window title
	 */
	private void createOutputOptionsShell(Shell parent) {
		outputOptionsShell = new Shell(parent, SWT.DIALOG_TRIM
				| SWT.APPLICATION_MODAL);
		outputOptionsShell.setText(PreferenceMessages
				.getString("ECLiPSeStrings.OutputOptions")); //$NON-NLS-1$
		GridLayout layout = new GridLayout();
		layout.numColumns = 3;
		outputOptionsShell.setLayout(layout);
		outputOptionsComposite = new ECLiPSeOutputOptions(outputOptionsShell,
				SWT.NONE, true, installation);
		GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		gridData.horizontalSpan = 3;
		outputOptionsComposite.setLayoutData(gridData);
		okButton = new Button(outputOptionsShell, SWT.PUSH);
		okButton.setText(PreferenceMessages.getString("ECLiPSeStrings.OK"));
		okButton.addSelectionListener(this);
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		okButton.setLayoutData(gridData);
		restoreButton = new Button(outputOptionsShell, SWT.PUSH);
		restoreButton.setText(PreferenceMessages
				.getString("ECLiPSeStrings.RestoreDefaults"));
		restoreButton.addSelectionListener(this);
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		restoreButton.setLayoutData(gridData);
		cancelButton = new Button(outputOptionsShell, SWT.PUSH);
		cancelButton.setText(PreferenceMessages
				.getString("ECLiPSeStrings.Cancel"));
		cancelButton.addSelectionListener(this);
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		cancelButton.setLayoutData(gridData);
		outputOptionsShell.setSize(outputOptionsShell.computeSize(SWT.DEFAULT,
				SWT.DEFAULT));
	}

	/**
	 * This method opens a modal ECLiPSeEngineSettingsShell
	 * 
	 * @param parent
	 *            the parent window
	 */
	public EnginePreferences open(Shell parent) {
		// open shell
		createOutputOptionsShell(parent);
		outputOptionsShell.open();
		// block display until shell is closed
		Display display = Display.getDefault();
		while (!outputOptionsShell.isDisposed()) {
			if (!display.readAndDispatch())
				display.sleep();
		}
		return installation;
	}

	/**
	 * main method, for debugging purposes
	 * 
	 * @param args
	 */
	public static void main(String[] args) {
		/*
		 * Before this is run, be sure to set up the launch configuration
		 * (Arguments->VM Arguments) for the correct SWT library path in order
		 * to run with the SWT dlls. The dlls are located in the SWT plugin jar.
		 * For example, on Windows the Eclipse SWT 3.1 plugin jar is:
		 * installation_directory\plugins\org.eclipse.swt.win32_3.1.0.jar
		 */
		Display display = Display.getDefault();
		PrintDepthOutputOptionsShell thisClass = new PrintDepthOutputOptionsShell(
				null);
		thisClass.open(null);
		while (!thisClass.outputOptionsShell.isDisposed()) {
			if (!display.readAndDispatch())
				display.sleep();
		}
		display.dispose();
	}

	public void widgetDefaultSelected(SelectionEvent event) {
	}

	public void widgetSelected(SelectionEvent event) {
		if (event.getSource() == okButton) {
			installation = outputOptionsComposite.commit(installation);
			outputOptionsShell.dispose();
		} else if (event.getSource() == restoreButton) {
			installation = outputOptionsComposite
					.revertToDefaults(installation);
		} else if (event.getSource() == cancelButton) {
			outputOptionsShell.dispose();
		}
	}
}
