/*******************************************************************************
 * Copyright (c) 2006 Cisco Technology, Inc and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Author: Andrew Cheadle, CrossCore Optimization Limited
 *
 * Contributors:
 *     Cisco Technology, Inc - Initial implementation
 *******************************************************************************/

package org.eclipseclp.saros.development.editors;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.LinkedList;
import org.eclipseclp.saros.core.preferences.ECLiPSeEnginePreferencesShell;
import org.eclipseclp.saros.core.preferences.EnginePreferences;
import org.eclipseclp.saros.core.preferences.LaunchQueryComposite;
import org.eclipseclp.saros.core.preferences.PreferenceConstants;
import org.eclipseclp.saros.core.preferences.PreferenceManager;
import org.eclipseclp.saros.core.preferences.PreferenceMessages;
import org.eclipseclp.saros.development.ECLiPSeFilter;
import org.eclipseclp.saros.development.ECLiPSeFilterConfigurationShell;
import org.eclipseclp.saros.development.Saros;
import org.eclipseclp.saros.development.views.CallStack;
import org.eclipseclp.saros.development.views.DelayedGoals;
import org.eclipseclp.saros.development.views.Inspector;
import org.eclipseclp.saros.development.views.SarosViewNotification;
import org.eclipseclp.saros.development.views.SarosViewNotifier;
import org.eclipseclp.saros.development.views.TraceLog;
import org.eclipseclp.saros.runtime.AsynchronousRPCresult;
import org.eclipseclp.saros.runtime.ECLiPSeInteractiveEngineRuntime;
import org.eclipseclp.saros.runtime.ECLiPSeStdStreamDataListener;
import org.eclipseclp.saros.runtime.ECLiPSeTraceLine;
import org.eclipseclp.saros.runtime.Environment;
import org.eclipseclp.saros.runtime.RunnableInputDialog;
import org.eclipseclp.saros.runtime.ECLiPSeInteractionInterface;
import org.eclipse.swt.SWT;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.text.TextSelection;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.IPartListener;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchListener;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.dialogs.PreferencesUtil;
import org.eclipse.ui.part.EditorPart;
import org.eclipse.ui.part.FileEditorInput;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.CoolBar;
import org.eclipse.swt.widgets.CoolItem;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.ToolBar;
import org.eclipse.swt.widgets.ToolItem;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.Decorations;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.graphics.Point;
import com.parctechnologies.eclipse.Atom;
import com.parctechnologies.eclipse.CompoundTerm;
import com.parctechnologies.eclipse.CompoundTermImpl;
import com.parctechnologies.eclipse.QueueListener;

public class ECLiPSeExecutionEnvironment extends EditorPart implements
		SelectionListener, KeyListener, IPartListener, IWorkbenchListener,
		ECLiPSeInteractionInterface {

	public final static String ID = "org.eclipseclp.saros.development.editors.ECLiPSeExecutionEnvironment";

	private Composite top = null;

	private Group queryGroup = null;

	private CoolBar coolbar = null;

	private ToolItem runTypeToolItem = null;

	private ToolItem moreToolItem = null;

	private ToolItem creepToolItem = null;

	private ToolItem skipToolItem = null;

	private ToolItem upToolItem = null;

	private ToolItem leapToolItem = null;

	private ToolItem filterToolItem = null;

	private ToolItem abortToolItem = null;

	private ToolItem nodebugToolItem = null;

	private ToolItem toInvocToolItem = null;

	private Text toInvocText = null;

	private ToolItem toPortToolItem = null;

	private Combo toPortCombo = null;

	private ToolItem toDepthToolItem = null;

	private Text toDepthFromText = null;

	private Text toDepthToText = null;

	private Combo moduleCombo = null;

	private Label moduleSeperatorLabel = null;

	private Combo queryCombo = null;

	private Group resultsGroup = null;

	private StyledText resultsTextArea = null;

	private SashForm resultsOutputErrorSashForm = null;

	private Group outputErrorGroup = null;

	private StyledText outputErrorTextArea = null;

	private Decorations menuBarDecoration = null;

	private Menu menuBar = null;

	private Menu popupMenu = null;

	private ECLiPSeInteractiveEngineRuntime eclipseEngine = null;

	private Environment environment = null;

	private boolean isActiveEditor;

	private int engineState = -1;

	private String compileScratchpadData = "";

	private Float statisticsRefreshRate = null;

	private boolean statisticsReportingEnabled = false;

	private ECLiPSeStdStreamDataListener statisticsStreamListener = null;

	private ECLiPSeFilter tracerFilter = null;

	private ArrayList delayedGoals = null;

	private boolean autoRefreshDelayedGoals = true;

	private boolean autoRefreshTracer = false;

	private String traceLogData = "";

	private ArrayList callStackData = null;

	private ArrayList inspectors = null;

	public void doSave(IProgressMonitor monitor) {
		monitor.done();
	}

	public void doSaveAs() {
	}

	public void init(IEditorSite site, IEditorInput editorInput)
			throws PartInitException {
		if (!(editorInput instanceof ECLiPSeExecutionEnvironmentEditorInput)) {
			if (!(editorInput instanceof FileEditorInput)) {
				throw new PartInitException(
						"Editor input must have a valid ECLiPSe source file extension.");
			} else {
				editorInput = new ECLiPSeExecutionEnvironmentEditorInput(
						((FileEditorInput) editorInput).getFile());
			}
		}
		setSite(site);
		setInput(editorInput);
		statisticsRefreshRate = new Float(1);
		tracerFilter = new ECLiPSeFilter();
		callStackData = new ArrayList();
		inspectors = new ArrayList();
	}

	public boolean isDirty() {
		return false;
	}

	public boolean isSaveAsAllowed() {
		return false;
	}

	public boolean isSaveOnCloseNeeded() {
		return false;
	}

	public void createPartControl(Composite parent) {
		if (!Saros.getDefault()
				.getMultipleExecutionEnvironmentInstanceSupport()) {
			if (Saros.getDefault().getExecutionEnvironmentSingleton() != null) {
				Status status = new Status(IStatus.ERROR, "Saros", 0,
						"Failed to start execution environment instance: ",
						null);
				ErrorDialog
						.openError(
								Saros.getDefault().getWorkbench().getDisplay()
										.getActiveShell(),
								"Saros Execution Environment Error",
								"Saros 1.0 only supports a single execution environment instance. "
										+ "Please close all active environments before starting a new one.",
								status);
				return;
			}
			Saros.getDefault().setExecutionEnvironmentSingleton(this);
		}
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 1;
		GridData gridData = new GridData();
		gridData.grabExcessHorizontalSpace = true;
		gridData.horizontalAlignment = org.eclipse.swt.layout.GridData.FILL;
		gridData.grabExcessVerticalSpace = true;
		gridData.verticalAlignment = org.eclipse.swt.layout.GridData.FILL;
		menuBarDecoration = new Decorations(parent, SWT.NONE);
		menuBar = new Menu(menuBarDecoration, SWT.BAR);
		menuBarDecoration.setMenuBar(menuBar);
		menuBarDecoration.setLayout(gridLayout);
		popupMenu = new Menu(menuBarDecoration, SWT.POP_UP);
		top = new Composite(menuBarDecoration, SWT.NONE);
		top.setLayout(gridLayout);
		top.setLayoutData(gridData);
		createCoolbar();
		createQueryGroup();
		createResultsOutputErrorSashForm();
		createResultsGroup();
		createOutputErrorGroup();
		createContextMenu();
		ECLiPSeExecutionEnvironmentEditorInput editorInput = (ECLiPSeExecutionEnvironmentEditorInput) getEditorInput();
		if (editorInput.isLaunchActionInput()) {
			environment = new Environment(Environment.PROJECT_PROPERTIES,
					editorInput.getProject(), editorInput.getFile());
			if (!PreferenceManager.launchActionSelection()) {
				environment.setHasLaunchQuery(false);
			}
		} else if (editorInput.isLaunchInput()) {
			environment = new Environment(
					Environment.PROJECT_LAUNCH_PROPERTIES, editorInput
							.getProject(), null);
			environment.setLaunchQuery(editorInput.hasLaunchQuery(),
					editorInput.getLaunchFile(), editorInput.getLaunchModule(),
					editorInput.getLaunchRunMethod(), editorInput
							.getLaunchQuery());
		} else {
			environment = new Environment(Environment.FILE_PROPERTIES,
					editorInput.getProject(), editorInput.getFile());
		}
		try { // Spawn the ECLiPSe engine
			eclipseEngine = new ECLiPSeInteractiveEngineRuntime(this,
					environment);

			getSite().getWorkbenchWindow().getPartService().addPartListener(
					this);

			if (environment.hasLaunchQuery()) {
				String launchQuery = environment.getLaunchQuery();
				if (!launchQuery.equals("")) {
					if (!environment.getLaunchModuleName().equals("")) {
						launchQuery = environment.getLaunchModuleName() + ":"
								+ launchQuery;
					}
					if (queryCombo.indexOf(launchQuery) == -1) {
						queryCombo.add(launchQuery);
					}
					final String finalLaunchQuery = launchQuery;
					Runnable r = new Runnable() {
						public void run() {
							runTypeToolItem.setText(PreferenceMessages
									.getString("ECLiPSeStrings.Interrupt"));
							runTypeToolItem
									.setData(PreferenceConstants.intString);
							moreToolItem.setEnabled(false);
							eclipseEngine
									.sendToplevelInCommand(
											LaunchQueryComposite
													.getToplevelRunMethodData(environment
															.getLaunchRunMethod()),
											finalLaunchQuery);
						}
					};
					Display.getDefault().asyncExec(r);
				}
			}
		} catch (Exception e) {
			runTypeToolItem.setEnabled(false);
			Status status = new Status(
					IStatus.ERROR,
					"Saros",
					0,
					"Exception caught in Saros ECLiPSe execution environment: ",
					e);
			ErrorDialog
					.openError(Saros.getDefault().getWorkbench().getDisplay()
							.getActiveShell(),
							"Saros Execution Environment Error",
							e.getMessage(), status);
			System.err.println("Exception caught in Saros ECLiPSe "
					+ "execution environment: " + e.getMessage());
			e.printStackTrace();
		}

		moduleCombo.addSelectionListener(this);
		moreToolItem.addSelectionListener(this);
		RunTypeDropdownSelectionListener runTypeItemListener = new RunTypeDropdownSelectionListener();
		runTypeItemListener.add(PreferenceMessages
				.getString("ECLiPSeStrings.Run"), PreferenceConstants.call);
		runTypeItemListener.add(PreferenceMessages
				.getString("ECLiPSeStrings.Trace"), PreferenceConstants.trace);
		runTypeItemListener.add(PreferenceMessages
				.getString("ECLiPSeStrings.TimeProfile"),
				PreferenceConstants.profile);
		runTypeItemListener.add(PreferenceMessages
				.getString("ECLiPSeStrings.PortProfile"),
				PreferenceConstants.portProfile);
		runTypeToolItem.addSelectionListener(runTypeItemListener);
		queryCombo.addKeyListener(this);
		setFocus();
	}

	private void createContextMenu() {
		MenuItem item = new MenuItem(popupMenu, SWT.CASCADE);
		item.setText(PreferenceMessages.getString("ECLiPSeStrings.File"));
		Menu menu = new Menu(popupMenu);
		item.setMenu(menu);
		item = new MenuItem(menu, SWT.PUSH);
		item.setText(PreferenceMessages.getString("ECLiPSeStrings.Compile")
				+ "...");
		item.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				compileMenuAction();
			}
		});
		item = new MenuItem(menu, SWT.PUSH);
		item.setText(PreferenceMessages.getString("ECLiPSeStrings.UseModule")
				+ "...");
		item.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				useModuleMenuAction();
			}
		});
		item = new MenuItem(menu, SWT.PUSH);
		item.setText(PreferenceMessages
				.getString("ECLiPSeStrings.CrossReferencer")
				+ "...");
		item.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				crossReferenceMenuAction();
			}
		});
		item = new MenuItem(menu, SWT.PUSH);
		item.setText(PreferenceMessages
				.getString("ECLiPSeStrings.SourceChecker")
				+ "...");
		item.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				sourceCheckerMenuAction();
			}
		});
		item = new MenuItem(menu, SWT.SEPARATOR);
		item = new MenuItem(menu, SWT.PUSH);
		item.setText(PreferenceMessages
				.getString("ECLiPSeStrings.ChangeDirectory")
				+ "...");
		item.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				changeDirectoryMenuAction();
			}
		});
		item = new MenuItem(menu, SWT.PUSH);
		item.setText(PreferenceMessages
				.getString("ECLiPSeStrings.ChangeExampleDirectory"));
		item.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				changeToExampleDirectoryMenuAction();
			}
		});
		item = new MenuItem(menu, SWT.SEPARATOR);
		item = new MenuItem(menu, SWT.PUSH);
		item.setText(PreferenceMessages.getString("ECLiPSeStrings.NewModule")
				+ "...");
		item.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				newModuleMenuAction();
			}
		});
		item = new MenuItem(menu, SWT.PUSH);
		item.setText(PreferenceMessages
				.getString("ECLiPSeStrings.ClearToplevelModule"));
		item.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				clearToplevelModuleMenuAction();
			}
		});
		item = new MenuItem(popupMenu, SWT.CASCADE);
		item.setText(PreferenceMessages.getString("ECLiPSeStrings.Tools"));
		menu = new Menu(popupMenu);
		item.setMenu(menu);
		item = new MenuItem(menu, SWT.PUSH);
		item.setText(PreferenceMessages
				.getString("ECLiPSeStrings.CompileScratchpad")
				+ "...");
		item.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				openCompileScratchpadView();
			}
		});

		/*
		 * item = new MenuItem(menu, SWT.PUSH); item.setText(PreferenceMessages
		 * .getString("ECLiPSeStrings.SourceFileManager") + "...");
		 */
		item = new MenuItem(menu, SWT.PUSH);
		item.setText(PreferenceMessages
				.getString("ECLiPSeStrings.PredicateBrowser")
				+ "...");
		item.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				openPredicateBrowserView();
			}
		});
		item = new MenuItem(menu, SWT.SEPARATOR);
		item = new MenuItem(menu, SWT.PUSH);
		item.setText(PreferenceMessages
				.getString("ECLiPSeStrings.DelayedGoals")
				+ "...");
		item.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				openDelayedGoalsView();
			}
		});
		item = new MenuItem(menu, SWT.CASCADE);
		item.setText(PreferenceMessages.getString("ECLiPSeStrings.Tracer"));
		Menu tracerMenu = new Menu(menu);
		item.setMenu(tracerMenu);
		item = new MenuItem(tracerMenu, SWT.PUSH);
		item.setText(PreferenceMessages.getString("ECLiPSeStrings.CallStack")
				+ "...");
		item.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				openCallStackView();
			}
		});
		item = new MenuItem(tracerMenu, SWT.PUSH);
		item.setText(PreferenceMessages.getString("ECLiPSeStrings.TraceLog")
				+ "...");
		item.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				openTraceLogView();
			}
		});
		new MenuItem(tracerMenu, SWT.SEPARATOR);
		item = new MenuItem(tracerMenu, SWT.PUSH);
		item.setText(PreferenceMessages
				.getString("ECLiPSeStrings.ConfigureFilter")
				+ "...");
		item.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				openTraceFilter();
			}
		});
		item = new MenuItem(menu, SWT.PUSH);
		item.setText(PreferenceMessages.getString("ECLiPSeStrings.Inspector")
				+ "...");
		item.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				openInspectorView();
			}
		});
		item = new MenuItem(menu, SWT.PUSH);
		item.setText(PreferenceMessages
				.getString("ECLiPSeStrings.VisualisationClient")
				+ "...");
		item.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				visualisationClientMenuAction();
			}
		});
		item = new MenuItem(menu, SWT.SEPARATOR);
		item = new MenuItem(menu, SWT.PUSH);
		item.setText(PreferenceMessages
				.getString("ECLiPSeStrings.ECLiPSeEngineSettings")
				+ "...");
		item.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				engineSettingsMenuAction();
			}
		});
		item = new MenuItem(menu, SWT.PUSH);
		item.setText(PreferenceMessages.getString("ECLiPSeStrings.Statistics")
				+ "...");
		item.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				openStatisticsSummaryView();
			}
		});

		top.setMenu(popupMenu);
		queryGroup.setMenu(popupMenu);
		resultsGroup.setMenu(popupMenu);
		resultsTextArea.setMenu(popupMenu);
		outputErrorGroup.setMenu(popupMenu);
		outputErrorTextArea.setMenu(popupMenu);
	}

	protected void openTraceFilter() {
		tracerFilter = (new ECLiPSeFilterConfigurationShell(this, tracerFilter))
				.open(getSite().getShell());
	}

	protected void engineSettingsMenuAction() {
		IProject project = environment.getProject();
		if (project != null) {
			PreferencesUtil
					.createPropertyDialogOn(
							this.getSite().getShell(),
							project,
							PreferenceConstants.ProjectEngineSettingsPropertyPageId,
							new String[] {
									PreferenceConstants.ProjectEngineSettingsPropertyPageId,
									PreferenceConstants.ProjectGCSettingsPropertyPageId,
									PreferenceConstants.ProjectOutputOptionsPropertyPageId },
							null).open();
			return;
		}

		// No associated project - standalone execution environment
		environment
				.setInstallation((new ECLiPSeEnginePreferencesShell(environment
						.getInstallation()).open(this.getSite().getShell())));
	}

	protected void openInspectorView() {
		IWorkbenchPage page = PlatformUI.getWorkbench()
				.getActiveWorkbenchWindow().getActivePage();
		try {
			page.showView("org.eclipseclp.saros.development.views.Inspector");
		} catch (PartInitException e) {
			Status status = new Status(IStatus.ERROR, "Saros", 0,
					"Exception caught whilst opening the Term Inspector: ", e);
			ErrorDialog.openError(Saros.getDefault().getWorkbench()
					.getDisplay().getActiveShell(),
					"Saros Development Plugin Error", e.getMessage(), status);
		}
	}

	protected void openTraceLogView() {
		IWorkbenchPage page = PlatformUI.getWorkbench()
				.getActiveWorkbenchWindow().getActivePage();
		try {
			page.showView("org.eclipseclp.saros.development.views.TraceLog");
		} catch (PartInitException e) {
			Status status = new Status(IStatus.ERROR, "Saros", 0,
					"Exception caught whilst opening the Trace Log view: ", e);
			ErrorDialog.openError(Saros.getDefault().getWorkbench()
					.getDisplay().getActiveShell(),
					"Saros Development Plugin Error", e.getMessage(), status);
		}
	}

	protected void openCallStackView() {
		IWorkbenchPage page = PlatformUI.getWorkbench()
				.getActiveWorkbenchWindow().getActivePage();
		try {
			page.showView("org.eclipseclp.saros.development.views.CallStack");
		} catch (PartInitException e) {
			Status status = new Status(IStatus.ERROR, "Saros", 0,
					"Exception caught whilst opening the Call Stack view: ", e);
			ErrorDialog.openError(Saros.getDefault().getWorkbench()
					.getDisplay().getActiveShell(),
					"Saros Development Plugin Error", e.getMessage(), status);
		}
	}

	protected void openDelayedGoalsView() {
		IWorkbenchPage page = PlatformUI.getWorkbench()
				.getActiveWorkbenchWindow().getActivePage();
		try {
			page
					.showView("org.eclipseclp.saros.development.views.DelayedGoals");
		} catch (PartInitException e) {
			Status status = new Status(IStatus.ERROR, "Saros", 0,
					"Exception caught whilst opening the Delayed Goals view", e);
			ErrorDialog.openError(Saros.getDefault().getWorkbench()
					.getDisplay().getActiveShell(),
					"Saros Development Plugin Error", e.getMessage(), status);
		}
	}

	protected void openPredicateBrowserView() {
		IWorkbenchPage page = PlatformUI.getWorkbench()
				.getActiveWorkbenchWindow().getActivePage();
		try {
			page
					.showView("org.eclipseclp.saros.development.views.PredicateBrowser");
		} catch (PartInitException e) {
			Status status = new Status(IStatus.ERROR, "Saros", 0,
					"Exception caught whilst opening the Predicate Browser: ",
					e);
			ErrorDialog.openError(Saros.getDefault().getWorkbench()
					.getDisplay().getActiveShell(),
					"Saros Development Plugin Error", e.getMessage(), status);
		}
	}

	protected void openCompileScratchpadView() {
		IWorkbenchPage page = PlatformUI.getWorkbench()
				.getActiveWorkbenchWindow().getActivePage();
		try {
			page
					.showView("org.eclipseclp.saros.development.views.CompileScratchpad");
		} catch (PartInitException e) {
			Status status = new Status(IStatus.ERROR, "Saros", 0,
					"Exception caught whilst opening the Compile Scratchpad: ",
					e);
			ErrorDialog.openError(Saros.getDefault().getWorkbench()
					.getDisplay().getActiveShell(),
					"Saros Development Plugin Error", e.getMessage(), status);
		}
	}

	protected void openStatisticsSummaryView() {
		IWorkbenchPage page = PlatformUI.getWorkbench()
				.getActiveWorkbenchWindow().getActivePage();
		try {
			page
					.showView("org.eclipseclp.saros.development.views.StatisticsSummary");
		} catch (PartInitException e) {
			Status status = new Status(
					IStatus.ERROR,
					"Saros",
					0,
					"Exception caught whilst opening the Statistics Summary view: ",
					e);
			ErrorDialog.openError(Saros.getDefault().getWorkbench()
					.getDisplay().getActiveShell(),
					"Saros Development Plugin Error", e.getMessage(), status);
		}
	}

	protected void visualisationClientMenuAction() {
		CompoundTerm command = new CompoundTermImpl(":", new Atom("java_vc"),
				new CompoundTermImpl("start_vc", (Object) null));
		interruptingRpc(command, "Failed to start visualisation client.");
	}

	protected void clearToplevelModuleMenuAction() {
		MessageBox messageBox = new MessageBox(this.getSite().getShell(),
				SWT.YES | SWT.NO | SWT.ICON_QUESTION);
		messageBox.setMessage("Erase current contents of module '"
				+ eclipseEngine.getToplevelModule() + "'?");
		if (messageBox.open() == SWT.YES) {
			CompoundTerm command = new CompoundTermImpl(":", new Atom(
					"tracer_tcl"), new Atom("init_toplevel_module"));
			interruptingRpc(command, "Failed to erase contents of module '"
					+ eclipseEngine.getToplevelModule() + "'.");
		}

	}

	protected void newModuleMenuAction() {
		String moduleName = "";
		String moduleLanguage = "";
		InputDialog d = new InputDialog(this.getSite().getShell(),
				"Create New Module", "Module name:", null, null);
		d.setBlockOnOpen(true);
		if (d.open() == Window.OK) {
			moduleName = d.getValue();
		} else {
			return;
		}

		d = new InputDialog(this.getSite().getShell(), "Create New Module",
				"Module language:", "eclipse_language", null);
		d.setBlockOnOpen(true);
		if (d.open() == Window.OK) {
			moduleLanguage = d.getValue();
		} else {
			return;
		}
		CompoundTerm command = new CompoundTermImpl("create_module", new Atom(
				moduleName), new ArrayList(), new Atom(moduleLanguage));
		interruptingRpc(command, "Failed to create new module '" + moduleName
				+ "' with language '" + moduleLanguage + "'.");
	}

	protected void changeToExampleDirectoryMenuAction() {
		try {
			File path = new File(eclipseEngine.getInstallationPath()
					+ File.separator + "doc" + File.separator + "examples");
			CompoundTerm command = new CompoundTermImpl(":", new Atom(
					"tracer_tcl"), new CompoundTermImpl("saros_cd", path
					.getCanonicalPath()));
			interruptingRpc(command, "Failed to change to examples directory.");
		} catch (IOException ioe) {
			System.err
					.println("Exception caught whilst changing to examples directory: "
							+ ioe.getMessage());
			ioe.printStackTrace();
		}
	}

	protected void changeDirectoryMenuAction() {
		DirectoryDialog dialog = new DirectoryDialog(this.getSite().getShell(),
				SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL);
		dialog.setText(PreferenceMessages
				.getString("ECLiPSeStrings.SelectWorkingDirectory"));
		dialog.setMessage(PreferenceMessages
				.getString("ECLiPSeStrings.SelectDirectory")
				+ ":");
		dialog.setFilterPath("");
		String path = dialog.open();
		if (path == null) {
			return;
		}

		try {
			CompoundTerm command = new CompoundTermImpl(":", new Atom(
					"tracer_tcl"), new CompoundTermImpl("saros_cd", (new File(
					path)).getCanonicalPath()));
			interruptingRpc(command, "Failed to change working directory to '"
					+ path + "'.");
		} catch (IOException ioe) {
			System.err
					.println("Exception caught whilst changing working directory: "
							+ ioe.getMessage());
			ioe.printStackTrace();
		}
	}

	protected void sourceCheckerMenuAction() {
		FileDialog dialog = new FileDialog(this.getSite().getShell(),
				SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL);
		dialog.setText(PreferenceMessages
				.getString("ECLiPSeStrings.SourceCheckSourceFile"));
		dialog.setFilterExtensions(new String[] { "*.ecl", "*.pl", "*" });
		dialog.setFilterNames(new String[] { "ECLiPSe Source File",
				"Prolog Source File", "All Files" });
		String filename = dialog.open();
		if (filename == null) {
			return;
		}

		try {
			CompoundTerm command = new CompoundTermImpl(":", new Atom("lint"),
					new CompoundTermImpl("saros_lint", (new File(filename))
							.getCanonicalPath()));
			interruptingRpc(command, "Failed to check ECLiPSe source file '"
					+ filename + "'.");
		} catch (IOException ioe) {
			System.err
					.println("Exception caught whilst checking ECLiPSe source file: "
							+ ioe.getMessage());
			ioe.printStackTrace();
		}
	}

	protected void crossReferenceMenuAction() {
		FileDialog dialog = new FileDialog(this.getSite().getShell(),
				SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL);
		dialog.setText(PreferenceMessages
				.getString("ECLiPSeStrings.CrossReferenceSourceFile"));
		dialog.setFilterExtensions(new String[] { "*.ecl", "*.pl", "*" });
		dialog.setFilterNames(new String[] { "ECLiPSe Source File",
				"Prolog Source File", "All Files" });
		String filename = dialog.open();
		if (filename == null) {
			return;
		}

		try {
			ArrayList options = new ArrayList();
			options.add(new CompoundTermImpl(":", new Atom("output"), new Atom(
					"graphviz")));
			CompoundTerm command = new CompoundTermImpl(":", new Atom("xref"),
					new CompoundTermImpl("saros_xref", (new File(filename))
							.getCanonicalPath(), options));
			interruptingRpc(command,
					"Failed to cross reference ECLiPSe source file '"
							+ filename + "'.");
		} catch (IOException ioe) {
			System.err
					.println("Exception caught whilst cross referencing ECLiPSe source file: "
							+ ioe.getMessage());
			ioe.printStackTrace();
		}
	}

	protected void useModuleMenuAction() {
		FileDialog dialog = new FileDialog(this.getSite().getShell(),
				SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL);
		dialog
				.setText(PreferenceMessages
						.getString("ECLiPSeStrings.UseModule"));
		dialog.setFilterExtensions(new String[] { "*.ecl", "*.pl", "*" });
		dialog.setFilterNames(new String[] { "ECLiPSe Source File",
				"Prolog Source File", "All Files" });
		String filename = dialog.open();
		if (filename == null) {
			return;
		}

		try {
			CompoundTerm command = new CompoundTermImpl(":", new Atom(
					"tracer_tcl"), new CompoundTermImpl("saros_use_module",
					(new File(filename)).getCanonicalPath()));
			interruptingRpc(command, "Failed to use module '" + filename + "'.");
		} catch (IOException ioe) {
			System.err
					.println("Exception caught whilst loading ECLiPSe module: "
							+ ioe.getMessage());
			ioe.printStackTrace();
		}
	}

	private void compileMenuAction() {
		FileDialog dialog = new FileDialog(this.getSite().getShell(),
				SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL);
		dialog.setText(PreferenceMessages
				.getString("ECLiPSeStrings.CompileSourceFile"));
		dialog.setFilterExtensions(new String[] { "*.ecl", "*.pl", "*" });
		dialog.setFilterNames(new String[] { "ECLiPSe Source File",
				"Prolog Source File", "All Files" });
		String filename = dialog.open();
		if (filename == null) {
			return;
		}

		try {
			CompoundTerm command = new CompoundTermImpl(":", new Atom(
					"tracer_tcl"), new CompoundTermImpl("saros_compile",
					(new File(filename)).getCanonicalPath()));
			interruptingRpc(command, "Failed to compile ECliPSe source file '"
					+ filename + "'.");
		} catch (IOException ioe) {
			System.err
					.println("Exception caught whilst compiling ECLiPSe source file: "
							+ ioe.getMessage());
			ioe.printStackTrace();
		}
	}

	public void setFocus() {
		if (queryCombo != null) {
			queryCombo.setFocus();
		}
	}

	/**
	 * This method initializes queryGroup
	 * 
	 */
	private void createQueryGroup() {
		GridData gridData4 = new GridData();
		gridData4.grabExcessHorizontalSpace = false;
		gridData4.horizontalAlignment = org.eclipse.swt.layout.GridData.BEGINNING;
		GridData gridData3 = new GridData();
		gridData3.horizontalAlignment = org.eclipse.swt.layout.GridData.FILL;
		gridData3.grabExcessHorizontalSpace = true;
		GridData gridData2 = new org.eclipse.swt.layout.GridData();
		gridData2.horizontalSpan = 1;
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 3;
		GridData gridData = new GridData();
		gridData.horizontalAlignment = org.eclipse.swt.layout.GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		queryGroup = new Group(top, SWT.NONE);
		queryGroup
				.setText(PreferenceMessages.getString("ECLiPSeStrings.Query"));
		queryGroup.setLayout(gridLayout);
		queryGroup.setLayoutData(gridData);
		moduleCombo = new Combo(queryGroup, SWT.DROP_DOWN | SWT.READ_ONLY);
		moduleCombo.setBackground(Display.getDefault().getSystemColor(
				SWT.COLOR_WIDGET_BACKGROUND));
		moduleCombo.setLayoutData(gridData4);
		moduleSeperatorLabel = new Label(queryGroup, SWT.NONE);
		moduleSeperatorLabel.setText(":");
		moduleSeperatorLabel.setLayoutData(gridData2);
		queryCombo = new Combo(queryGroup, SWT.NONE);
		queryCombo.setLayoutData(gridData3);
	}

	private void createCoolbar() {
		GridData gridData = new GridData();
		gridData.grabExcessHorizontalSpace = true;
		gridData.horizontalAlignment = org.eclipse.swt.layout.GridData.FILL;
		coolbar = new CoolBar(top, SWT.BORDER);
		coolbar.setLayoutData(gridData);
		createQueryCoolItem();
		createTracerButtonCoolItem();
		createTracerJumpCoolItem();
	}

	private void createQueryCoolItem() {
		CoolItem coolItem = new CoolItem(coolbar, SWT.NONE);
		ToolBar toolbar = new ToolBar(coolbar, SWT.NONE);
		runTypeToolItem = new ToolItem(toolbar, SWT.DROP_DOWN);
		runTypeToolItem.setText(PreferenceMessages
				.getString("ECLiPSeStrings.TimeProfile"));
		runTypeToolItem.setData(PreferenceConstants.call);
		moreToolItem = new ToolItem(toolbar, SWT.PUSH);
		moreToolItem.setText(PreferenceMessages
				.getString("ECLiPSeStrings.More"));
		moreToolItem.setData(PreferenceConstants.More);
		moreToolItem.setEnabled(false);
		toolbar.pack();
		runTypeToolItem.setText(PreferenceMessages
				.getString("ECLiPSeStrings.Run"));
		Point size = toolbar.getSize();
		coolItem.setControl(toolbar);
		coolItem.setSize(coolItem.computeSize(size.x, size.y));
	}

	private void createTracerButtonCoolItem() {
		CoolItem coolItem = new CoolItem(coolbar, SWT.NONE);
		ToolBar toolbar = new ToolBar(coolbar, SWT.NONE);
		creepToolItem = new ToolItem(toolbar, SWT.PUSH);
		creepToolItem.setText(PreferenceMessages
				.getString("ECLiPSeStrings.Creep"));
		creepToolItem.setData(PreferenceConstants.Creep);
		creepToolItem.setEnabled(false);
		creepToolItem.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				sendTracerCommand(
						null,
						assembleTracerCommand(PreferenceConstants.tracerCreepCommand));
			}
		});
		skipToolItem = new ToolItem(toolbar, SWT.PUSH);
		skipToolItem.setText(PreferenceMessages
				.getString("ECLiPSeStrings.Skip"));
		skipToolItem.setData(PreferenceConstants.Skip);
		skipToolItem.setEnabled(false);
		skipToolItem.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				sendTracerCommand(
						null,
						assembleTracerCommand(PreferenceConstants.tracerSkipCommand));
			}
		});
		upToolItem = new ToolItem(toolbar, SWT.PUSH);
		upToolItem.setText(PreferenceMessages.getString("ECLiPSeStrings.Up"));
		upToolItem.setData(PreferenceConstants.Up);
		upToolItem.setEnabled(false);
		upToolItem.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				int depth;
				if (callStackData == null) {
					depth = 0;
				} else {
					depth = callStackData.size() - 1;
				}
				sendTracerCommand(
						assembleFilterCommand(null, new CompoundTermImpl("..",
								new Integer(0), new Integer(depth > 0 ? depth
										: 0)), null, null, null),
						assembleTracerCommand(PreferenceConstants.tracerToDepthCommand));
			}
		});
		leapToolItem = new ToolItem(toolbar, SWT.PUSH);
		leapToolItem.setText(PreferenceMessages
				.getString("ECLiPSeStrings.Leap"));
		leapToolItem.setData(PreferenceConstants.Leap);
		leapToolItem.setEnabled(false);
		leapToolItem.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				sendTracerCommand(
						null,
						assembleTracerCommand(PreferenceConstants.tracerLeapCommand));
			}
		});
		filterToolItem = new ToolItem(toolbar, SWT.PUSH);
		filterToolItem.setText(PreferenceMessages
				.getString("ECLiPSeStrings.Filter"));
		filterToolItem.setData(PreferenceConstants.Filter);
		filterToolItem.setEnabled(false);
		filterToolItem.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				CompoundTerm term = assembleFilterCommand();
				if (term == null) {
					return;
				}
				sendTracerCommand(
						term,
						assembleTracerCommand(PreferenceConstants.tracerFilterCommand));
			}
		});
		abortToolItem = new ToolItem(toolbar, SWT.PUSH);
		abortToolItem.setText(PreferenceMessages
				.getString("ECLiPSeStrings.Abort"));
		abortToolItem.setData(PreferenceConstants.Abort);
		abortToolItem.setEnabled(false);
		abortToolItem.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				sendTracerCommand(
						null,
						assembleTracerCommand(PreferenceConstants.tracerAbortCommand));
			}
		});
		nodebugToolItem = new ToolItem(toolbar, SWT.PUSH);
		nodebugToolItem.setText(PreferenceMessages
				.getString("ECLiPSeStrings.Nodebug"));
		nodebugToolItem.setData(PreferenceConstants.Nodebug);
		nodebugToolItem.setEnabled(false);
		nodebugToolItem.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				sendTracerCommand(
						null,
						assembleTracerCommand(PreferenceConstants.tracerNodebugCommand));
			}
		});
		toolbar.pack();
		Point size = toolbar.getSize();
		coolItem.setControl(toolbar);
		coolItem.setSize(coolItem.computeSize(size.x, size.y));
	}

	private void createTracerJumpCoolItem() {
		CoolItem coolItem = new CoolItem(coolbar, SWT.NONE);
		ToolBar toolbar = new ToolBar(coolbar, SWT.NONE);
		//
		toInvocToolItem = new ToolItem(toolbar, SWT.PUSH);
		toInvocToolItem.setText(PreferenceMessages
				.getString("ECLiPSeStrings.ToInvoc")
				+ ":");
		toInvocToolItem.setData(PreferenceConstants.ToInvoc);
		toInvocToolItem.setEnabled(false);
		toInvocToolItem.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				sendTracerCommand(
						assembleFilterCommand(
								new Integer(toInvocText.getText()), null, null,
								null, null),
						assembleTracerCommand(PreferenceConstants.tracerToInvocCommand));
			}
		});
		//
		ToolItem toolItem = new ToolItem(toolbar, SWT.SEPARATOR);
		toInvocText = new Text(toolbar, SWT.BORDER);
		toInvocText.setEnabled(false);
		toInvocText.setText("9999");
		toInvocText.pack();
		Point textSize = toInvocText.getSize();
		toolItem.setWidth(textSize.x);
		toolItem.setControl(toInvocText);
		toInvocText.setText("0");
		//
		toDepthToolItem = new ToolItem(toolbar, SWT.PUSH);
		toDepthToolItem.setText(PreferenceMessages
				.getString("ECLiPSeStrings.ToDepth")
				+ ":");
		toDepthToolItem.setData(PreferenceConstants.ToDepth);
		toDepthToolItem.setEnabled(false);
		toDepthToolItem.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				sendTracerCommand(
						assembleFilterCommand(null, new CompoundTermImpl("..",
								new Integer(toDepthFromText.getText()),
								new Integer(toDepthToText.getText())), null,
								null, null),
						assembleTracerCommand(PreferenceConstants.tracerToDepthCommand));
			}
		});
		//
		toolItem = new ToolItem(toolbar, SWT.SEPARATOR);
		toDepthFromText = new Text(toolbar, SWT.BORDER);
		toDepthFromText.setEnabled(false);
		toDepthFromText.setText("0");
		toDepthFromText.setSize(textSize);
		toolItem.setWidth(textSize.x);
		toolItem.setControl(toDepthFromText);
		//
		new ToolItem(toolbar, SWT.SEPARATOR);
		//
		toolItem = new ToolItem(toolbar, SWT.SEPARATOR);
		toDepthToText = new Text(toolbar, SWT.BORDER);
		toDepthToText.setEnabled(false);
		toDepthToText.setText("9999");
		toDepthToText.setSize(textSize);
		toolItem.setWidth(textSize.x);
		toolItem.setControl(toDepthToText);
		//
		toPortToolItem = new ToolItem(toolbar, SWT.PUSH);
		toPortToolItem.setText(PreferenceMessages
				.getString("ECLiPSeStrings.ToPort")
				+ ":");
		toPortToolItem.setData(PreferenceConstants.ToPort);
		toPortToolItem.setEnabled(false);
		toPortToolItem.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				if (toPortCombo.getText().equals(
						PreferenceMessages
								.getString("ECLiPSeStrings.NotCurrent"))) {
					sendTracerCommand(
							null,
							assembleTracerCommand(PreferenceConstants.tracerToPortCommand));
				} else {
					sendTracerCommand(
							assembleFilterCommand(null, null, new Atom(
									(String) toPortCombo.getData()), null,
									new Atom("dontcare")),
							assembleTracerCommand(PreferenceConstants.tracerToSpecificPortCommand));
				}
			}
		});
		//
		toolItem = new ToolItem(toolbar, SWT.SEPARATOR);
		toPortCombo = new Combo(toolbar, SWT.BORDER | SWT.READ_ONLY);
		toPortCombo.setBackground(Display.getDefault().getSystemColor(
				SWT.COLOR_WIDGET_BACKGROUND));
		toPortCombo.add(PreferenceMessages
				.getString("ECLiPSeStrings.NotCurrent"));
		toPortCombo.setText(PreferenceMessages
				.getString("ECLiPSeStrings.NotCurrent"));
		toPortCombo.add(PreferenceMessages.getString("ECLiPSeStrings.call"));
		toPortCombo.add(PreferenceMessages.getString("ECLiPSeStrings.exit"));
		toPortCombo.add(PreferenceMessages.getString("ECLiPSeStrings.*exit"));
		toPortCombo.add(PreferenceMessages.getString("ECLiPSeStrings.redo"));
		toPortCombo.add(PreferenceMessages.getString("ECLiPSeStrings.fail"));
		toPortCombo.add(PreferenceMessages.getString("ECLiPSeStrings.resume"));
		toPortCombo.add(PreferenceMessages.getString("ECLiPSeStrings.leave"));
		toPortCombo.add(PreferenceMessages.getString("ECLiPSeStrings.delay"));
		toPortCombo.add(PreferenceMessages.getString("ECLiPSeStrings.next"));
		toPortCombo.add(PreferenceMessages.getString("ECLiPSeStrings.unify"));
		toPortCombo.add(PreferenceMessages.getString("ECLiPSeStrings.spyterm"));
		toPortCombo.add(PreferenceMessages.getString("ECLiPSeStrings.modify"));
		toPortCombo.add(PreferenceMessages.getString("ECLiPSeStrings.else"));
		toPortCombo.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				if (toPortCombo.getText().equals(
						PreferenceMessages.getString("ECLiPSeStrings.call"))) {
					toPortCombo.setData(PreferenceConstants.call);
				} else if (toPortCombo.getText().equals(
						PreferenceMessages.getString("ECLiPSeStrings.exit"))) {
					toPortCombo.setData(PreferenceConstants.exit);
				} else if (toPortCombo.getText().equals(
						PreferenceMessages.getString("ECLiPSeStrings.*exit"))) {
					toPortCombo.setData(PreferenceConstants.starExit);
				} else if (toPortCombo.getText().equals(
						PreferenceMessages.getString("ECLiPSeStrings.redo"))) {
					toPortCombo.setData(PreferenceConstants.redo);

				} else if (toPortCombo.getText().equals(
						PreferenceMessages.getString("ECLiPSeStrings.fail"))) {
					toPortCombo.setData(PreferenceConstants.fail);

				} else if (toPortCombo.getText().equals(
						PreferenceMessages.getString("ECLiPSeStrings.resume"))) {
					toPortCombo.setData(PreferenceConstants.resume);

				} else if (toPortCombo.getText().equals(
						PreferenceMessages.getString("ECLiPSeStrings.leave"))) {
					toPortCombo.setData(PreferenceConstants.leave);

				} else if (toPortCombo.getText().equals(
						PreferenceMessages.getString("ECLiPSeStrings.delay"))) {
					toPortCombo.setData(PreferenceConstants.delay);

				} else if (toPortCombo.getText().equals(
						PreferenceMessages.getString("ECLiPSeStrings.next"))) {
					toPortCombo.setData(PreferenceConstants.next);

				} else if (toPortCombo.getText().equals(
						PreferenceMessages.getString("ECLiPSeStrings.unify"))) {
					toPortCombo.setData(PreferenceConstants.unify);

				} else if (toPortCombo.getText().equals(
						PreferenceMessages.getString("ECLiPSeStrings.spyterm"))) {
					toPortCombo.setData(PreferenceConstants.spyterm);

				} else if (toPortCombo.getText().equals(
						PreferenceMessages.getString("ECLiPSeStrings.modify"))) {
					toPortCombo.setData(PreferenceConstants.modify);

				} else if (toPortCombo.getText().equals(
						PreferenceMessages.getString("ECLiPSeStrings.else"))) {
					toPortCombo.setData(PreferenceConstants.elseString);

				}
			}
		});
		toPortCombo.setEnabled(false);
		toPortCombo.pack();
		toolItem.setWidth(toPortCombo.getSize().x);
		toolItem.setControl(toPortCombo);
		//
		toolbar.pack();
		coolItem.setControl(toolbar);
		coolItem.setSize(toolbar.computeSize(SWT.DEFAULT, SWT.DEFAULT));
	}

	private void createResultsOutputErrorSashForm() {
		GridData gridData = new GridData();
		gridData.horizontalAlignment = org.eclipse.swt.layout.GridData.FILL;
		gridData.grabExcessVerticalSpace = true;
		gridData.verticalAlignment = org.eclipse.swt.layout.GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		resultsOutputErrorSashForm = new SashForm(top, SWT.NONE);
		resultsOutputErrorSashForm.setOrientation(org.eclipse.swt.SWT.VERTICAL);
		resultsOutputErrorSashForm.setLayoutData(gridData);
	}

	/**
	 * This method initializes resultsGroup
	 * 
	 */
	private void createResultsGroup() {
		GridData gridData6 = new GridData();
		gridData6.horizontalAlignment = org.eclipse.swt.layout.GridData.FILL;
		gridData6.grabExcessVerticalSpace = true;
		gridData6.verticalAlignment = org.eclipse.swt.layout.GridData.FILL;
		gridData6.grabExcessHorizontalSpace = true;
		GridData gridData5 = new GridData();
		gridData5.horizontalAlignment = org.eclipse.swt.layout.GridData.FILL;
		gridData5.grabExcessVerticalSpace = true;
		gridData5.verticalAlignment = org.eclipse.swt.layout.GridData.FILL;
		gridData5.grabExcessHorizontalSpace = true;
		resultsGroup = new Group(resultsOutputErrorSashForm, SWT.NONE);
		resultsGroup.setLayout(new GridLayout());
		resultsGroup.setText(PreferenceMessages
				.getString("ECLiPSeStrings.Results"));
		resultsGroup.setLayoutData(gridData5);
		resultsTextArea = new StyledText(resultsGroup, SWT.MULTI | SWT.WRAP
				| SWT.READ_ONLY | SWT.V_SCROLL | SWT.BORDER);
		resultsTextArea.setLayoutData(gridData6);
	}

	/**
	 * This method initializes outputErrorGroup
	 * 
	 */
	private void createOutputErrorGroup() {
		GridData gridData8 = new GridData();
		gridData8.grabExcessHorizontalSpace = true;
		gridData8.horizontalAlignment = org.eclipse.swt.layout.GridData.FILL;
		gridData8.verticalAlignment = org.eclipse.swt.layout.GridData.FILL;
		gridData8.grabExcessVerticalSpace = true;
		GridData gridData7 = new GridData();
		gridData7.grabExcessHorizontalSpace = true;
		gridData7.horizontalAlignment = org.eclipse.swt.layout.GridData.FILL;
		gridData7.verticalAlignment = org.eclipse.swt.layout.GridData.FILL;
		gridData7.grabExcessVerticalSpace = true;
		outputErrorGroup = new Group(resultsOutputErrorSashForm, SWT.NONE);
		outputErrorGroup.setText(PreferenceMessages
				.getString("ECLiPSeStrings.OutputErrorMessages"));
		outputErrorGroup.setLayout(new GridLayout());
		outputErrorGroup.setLayoutData(gridData7);
		outputErrorTextArea = new StyledText(outputErrorGroup, SWT.MULTI
				| SWT.WRAP | SWT.READ_ONLY | SWT.V_SCROLL | SWT.BORDER);
		outputErrorTextArea.setLayoutData(gridData8);
	}

	public void partClosed(IWorkbenchPart part) {
		if (part == this) {
			getSite().getWorkbenchWindow().getPartService().removePartListener(
					this);
			SarosViewNotifier.notifyViews(getSite().getPage(), this,
					SarosViewNotifier.DECONFIGURE);
			eclipseEngine.shutdown();
			Saros.getDefault().setExecutionEnvironmentSingleton(null);
		} else if (part instanceof SarosViewNotification) {
			((SarosViewNotification) part).viewClosed(this);
		}
	}

	public void partActivated(IWorkbenchPart part) {
		if (!isActiveEditor && getSite().getPage().getActivePart() == this) {
			isActiveEditor = true;
			SarosViewNotifier.notifyViews(getSite().getPage(), this,
					SarosViewNotifier.ACTIVATE);
		} else if (part instanceof ECLiPSeExecutionEnvironment) {
			isActiveEditor = false;
		}
	}

	public void partBroughtToTop(IWorkbenchPart part) {
	}

	public void partDeactivated(IWorkbenchPart part) {
	}

	public void partOpened(IWorkbenchPart part) {
	}

	public void widgetSelected(SelectionEvent event) {
		if (event.getSource() == moreToolItem) {
			// System.out.println("Selected: " + moreToolItem.getData());
			runTypeToolItem.setText(PreferenceMessages
					.getString("ECLiPSeStrings.Interrupt"));
			runTypeToolItem.setData(PreferenceConstants.intString);
			moreToolItem.setEnabled(false);
			eclipseEngine.requestMore();
		} else if (event.getSource() == moduleCombo) {
			String[] activeModules = moduleCombo.getItems();
			final String selectedModule = moduleCombo.getText();
			moduleCombo.removeAll();
			moduleCombo.add(selectedModule);
			Point size = moduleCombo
					.computeSize(SWT.DEFAULT, SWT.DEFAULT, true);
			moduleCombo.removeAll();
			for (int index = 0; index < activeModules.length; index++) {
				moduleCombo.add(activeModules[index]);
			}
			AsynchronousRPCresult r = new AsynchronousRPCresult() {
				public void run() {
					eclipseEngine.setToplevelModule(selectedModule);
				}
			};
			interruptingRpc(
					new CompoundTermImpl("set_flag",
							new Atom("toplevel_module"), new Atom(
									selectedModule)),
					"Unable to set toplevel module to '" + selectedModule + "'",
					r, null);
			moduleCombo.setText(selectedModule);
			GridData gridData4 = new GridData();
			gridData4.grabExcessHorizontalSpace = false;
			gridData4.horizontalAlignment = org.eclipse.swt.layout.GridData.BEGINNING;
			gridData4.minimumWidth = size.x;
			gridData4.widthHint = size.x;
			moduleCombo.setLayoutData(gridData4);
			moduleCombo.setSize(size);
			moduleCombo.getParent().layout(true);
		}
	}

	public void widgetDefaultSelected(SelectionEvent event) {
	}

	public void keyPressed(KeyEvent event) {
		if (event.getSource() == queryCombo && !queryCombo.getText().equals("")) {
			if (event.keyCode == 13) {
				runTypeToolItem.notifyListeners(SWT.Selection, new Event());
			} else if (event.keyCode == 16777296 && moreToolItem.isEnabled()) {
				Event fakeEvent = new Event();
				fakeEvent.widget = moreToolItem;
				moreToolItem.notifyListeners(SWT.Selection, fakeEvent);
			}
		}
	}

	public void keyReleased(KeyEvent event) {
	}

	private void enableTracer(boolean enable) {
		creepToolItem.setEnabled(enable);
		skipToolItem.setEnabled(enable);
		upToolItem.setEnabled(enable);
		leapToolItem.setEnabled(enable);
		filterToolItem.setEnabled(enable);
		abortToolItem.setEnabled(enable);
		nodebugToolItem.setEnabled(enable);
		toInvocText.setEnabled(enable);
		toDepthFromText.setEnabled(enable);
		toDepthToText.setEnabled(enable);
		toDepthFromText.setText("0");
		toDepthToText.setText("9999");
		toDepthFromText.setText("0");
		toInvocToolItem.setEnabled(enable);
		toDepthToolItem.setEnabled(enable);
		toPortToolItem.setEnabled(enable);
		toPortCombo.setEnabled(enable);
		runTypeToolItem.setEnabled(!enable);
	}

	class RunTypeDropdownSelectionListener extends SelectionAdapter {
		private Menu menu = null;

		/**
		 * Constructs a RunTypeDropdownSelectionListener
		 */
		public RunTypeDropdownSelectionListener() {
			menu = new Menu(runTypeToolItem.getParent().getShell());
		}

		/**
		 * Adds an item to the dropdown list
		 * 
		 * @param item
		 *            the item to add
		 */
		public void add(String item, String command) {
			MenuItem menuItem = new MenuItem(menu, SWT.NONE);
			menuItem.setText(item);
			menuItem.setData(command);
			menuItem.addSelectionListener(new SelectionAdapter() {
				public void widgetSelected(SelectionEvent event) {
					MenuItem selected = (MenuItem) event.widget;
					runTypeToolItem.setText(selected.getText());
					runTypeToolItem.setData(selected.getData());
				}
			});
		}

		/**
		 * Called when either the button itself or the dropdown arrow is clicked
		 * 
		 * @param event
		 *            the event that trigged this call
		 */
		public void widgetSelected(SelectionEvent event) {
			// Is gui in 'Interrupt' state?
			if (runTypeToolItem.getData().toString().equals(
					PreferenceConstants.intString)) {
				eclipseEngine.requestInterrupt();
				return;
			}

			// If they clicked the arrow, we show the list
			if (event.detail == SWT.ARROW) {
				// Determine where to put the dropdown list
				ToolItem item = (ToolItem) event.widget;
				Rectangle rect = item.getBounds();
				Point pt = item.getParent()
						.toDisplay(new Point(rect.x, rect.y));
				menu.setLocation(pt.x, pt.y + rect.height);
				menu.setVisible(true);
			} else {
				// The button was clicked
				String query = queryCombo.getText();
				if (query.equals("")) {
					return;
				}
				if (queryCombo.indexOf(query) == -1) {
					queryCombo.add(query);
				}
				// System.out.println("" + runTypeToolItem.getData() + "(" +
				// query + ")");
				String data = runTypeToolItem.getData().toString();
				runTypeToolItem.setText(PreferenceMessages
						.getString("ECLiPSeStrings.Interrupt"));
				runTypeToolItem.setData(PreferenceConstants.intString);
				moreToolItem.setEnabled(false);
				eclipseEngine.sendToplevelInCommand(data, query);
			}
		}
	}

	private CompoundTerm assembleTracerCommand(String command) {
		return new CompoundTermImpl(":", new Atom("tracer_tcl"),
				new CompoundTermImpl("set_tracer_command", command));
	}

	private CompoundTerm assembleFilterCommand() {
		CompoundTerm term = null;
		CompoundTerm goalMatchingTerm = null;

		Atom predicateFilter = null;
		if (tracerFilter.isAnyPredicateFilter()) {
			predicateFilter = new Atom("all");
		} else if (tracerFilter.isAnySpiedPredicateFilter()) {
			predicateFilter = new Atom("spied");
		} else if (tracerFilter.isSpecificPredicateInstanceFilter()) {
			String predMatch = tracerFilter.getGoalTemplateFilter();
			if (predMatch.equals("")) {
				predMatch = "_";
			}
			String predModule = tracerFilter.getCallingModuleFilter();
			if (predModule.equals("")) {
				predModule = "_";
			}
			String predDefModule = tracerFilter.getDefiningModuleFilter();
			if (predDefModule.equals("")) {
				predDefModule = "eclipse";
			}
			String predCondition = tracerFilter.getConditionFilter();
			if (predCondition.equals("")) {
				predCondition = "true";
			}
			goalMatchingTerm = new CompoundTermImpl(":",
					new Atom("tracer_tcl"), new CompoundTermImpl(
							"set_usepred_info", predMatch, predModule,
							predDefModule, predCondition, null));
			goalMatchingTerm = eclipseEngine
					.rpc(
							goalMatchingTerm,
							"Unable to set conditional goal filter. Please check goal template and condition for syntax error");
			if (goalMatchingTerm == null) {
				return null;
			}
			String status = ((Atom) ((CompoundTerm) goalMatchingTerm.arg(2))
					.arg(5)).functor();
			if (status.equals("spy_set")) {
				predicateFilter = new Atom("spied");
			} else if (status.equals("not_found")) {
				Status errStatus = new Status(IStatus.ERROR, "Saros", 0,
						"Goal template or module may be undefined.", null);
				ErrorDialog.openError(Saros.getDefault().getWorkbench()
						.getDisplay().getActiveShell(),
						"Saros Execution Environment Error",
						"Filter Error: Failed to set conditional goal filter.",
						errStatus);

				return null;
			} else {
				predicateFilter = new Atom("all");
			}
		}

		term = assembleFilterCommand(new CompoundTermImpl("..", new Integer(
				tracerFilter.getFromInvocationFilter()), new Integer(
				tracerFilter.getToInvocationFilter())), new CompoundTermImpl(
				"..", new Integer(tracerFilter.getFromDepthFilter()),
				new Integer(tracerFilter.getToDepthFilter())), tracerFilter
				.getPortFilters(), predicateFilter, new Atom("dontcare"));

		term = new CompoundTermImpl(",", new CompoundTermImpl("@",
				new CompoundTermImpl("setval", new Atom("filter_hits"),
						new Integer(0)), new Atom("tracer_tcl")), term);

		return new CompoundTermImpl(",", new CompoundTermImpl(":", new Atom(
				"tracer_tcl"), new CompoundTermImpl("prepare_filter",
				new Integer(tracerFilter.getConditionCounterFilter()))), term);
	}

	private CompoundTerm assembleFilterCommand(Object arg1, Object arg2,
			Object arg3, Object arg4, Object arg5) {
		return new CompoundTermImpl(":", new Atom("sepia_kernel"),
				new CompoundTermImpl("configure_prefilter", arg1, arg2, arg3,
						arg4, arg5));
	}

	public void sendTracerCommand(CompoundTerm preCommand, CompoundTerm command) {
		eclipseEngine.sendTracerCommand(preCommand, command);
	}

	public QueueListener getOutputStreamListener() {
		return new ECLiPSeStdStreamDataListener(outputErrorTextArea,
				SWT.COLOR_BLUE, false, false);
	}

	public QueueListener getErrorStreamListener() {
		return new ECLiPSeStdStreamDataListener(outputErrorTextArea,
				SWT.COLOR_RED, false, false);
	}

	public QueueListener getWarningStreamListener() {
		return new ECLiPSeStdStreamDataListener(outputErrorTextArea,
				SWT.COLOR_DARK_MAGENTA, false, false);
	}

	public QueueListener getAnswerStreamListener() {
		return new ECLiPSeStdStreamDataListener(resultsTextArea,
				SWT.COLOR_BLUE, false, false);
	}

	public QueueListener getStatisticsStreamListener() {
		return (statisticsStreamListener = new ECLiPSeStdStreamDataListener(
				null, SWT.COLOR_BLUE, true, true));
	}

	public int getToplevelState() {
		return engineState;
	}

	public void requestInterrupt() {
		eclipseEngine.requestInterrupt();
	}

	public void sendToplevelInCommand(String command, String query) {
		eclipseEngine.sendToplevelInCommand(command, query);
	}

	public void reflectMoreStatus() {
		Runnable r = new Runnable() {
			public void run() {
				moreToolItem.setEnabled(true);
			}
		};
		Display.getDefault().asyncExec(r);
	}

	public void reflectStoppedStatus() {
		Runnable r = new Runnable() {
			public void run() {
				runTypeToolItem.setText(PreferenceMessages
						.getString("ECLiPSeStrings.Run"));
				runTypeToolItem.setData(PreferenceConstants.call);
				moreToolItem.setEnabled(false);
			}
		};
		Display.getDefault().asyncExec(r);
	}

	public void reflectTracerStatus(final boolean enabled) {
		if (enabled) {
			CompoundTerm term = new CompoundTermImpl("@", new CompoundTermImpl(
					"getval", new Atom("filter_hits"), null), new Atom(
					"tracer_tcl"));
			CompoundTerm result = eclipseEngine.rpc(term,
					"Unable to retrieve filter condition counter");
			if (result != null) {
				tracerFilter
						.setConditionMetCounter(((Integer) ((CompoundTerm) result
								.arg(1)).arg(2)).intValue());
			}
		}
		Runnable r = new Runnable() {
			public void run() {
				enableTracer(enabled);
			}
		};
		Display.getDefault().asyncExec(r);
	}

	public String requestInput() {
		RunnableInputDialog inputDialog = new RunnableInputDialog();
		Display.getDefault().syncExec(inputDialog);
		return inputDialog.getInput();
	}

	public void requestToplevelStateTransition(int state) {
		this.engineState = state;
	}

	public void sendEngineReconfigurationNotification() {
		final ECLiPSeExecutionEnvironment eev = this;
		Runnable r = new Runnable() {
			public void run() {
				SarosViewNotifier.notifyViews(getSite().getPage(), eev,
						SarosViewNotifier.CONFIGURE);
			}
		};
		Display.getDefault().asyncExec(r);
	}

	public void setActiveModules(String[] moduleList, String selectedModule) {
		if (moduleCombo.isDisposed()) {
			return;
		}
		moduleCombo.removeAll();
		moduleCombo.add(selectedModule);
		Point size = moduleCombo.computeSize(SWT.DEFAULT, SWT.DEFAULT, true);
		moduleCombo.removeAll();
		for (int index = 0; index < moduleList.length; index++) {
			moduleCombo.add(moduleList[index]);
		}
		moduleCombo.setText(selectedModule);
		GridData gridData4 = new GridData();
		gridData4.grabExcessHorizontalSpace = false;
		gridData4.horizontalAlignment = org.eclipse.swt.layout.GridData.BEGINNING;
		gridData4.minimumWidth = size.x;
		gridData4.widthHint = size.x;
		moduleCombo.setLayoutData(gridData4);
		moduleCombo.setSize(size);
		moduleCombo.getParent().layout(true);
	}

	public void interruptingRpc(CompoundTerm command, String failMessage) {
		eclipseEngine.interruptingRpc(command, failMessage);
	}

	public void interruptingRpc(CompoundTerm command, String failMessage,
			AsynchronousRPCresult successContinuation,
			Runnable failureContinuation) {
		eclipseEngine.interruptingRpc(command, failMessage,
				successContinuation, failureContinuation);
	}

	public void shutdown() {
		eclipseEngine.shutdown();
	}

	public void setStatisticsRefreshRate(Float refreshRate) {
		statisticsRefreshRate = refreshRate;
	}

	public Float getStatisticsRefreshRate() {
		return statisticsRefreshRate;
	}

	public void setStatisticsReportingEnabled(boolean enabled) {
		statisticsReportingEnabled = enabled;
	}

	public void setStatisticsCollecting(boolean enabled) {
		eclipseEngine.setStatisticsCollecting(enabled);
	}

	public boolean statisticsCollecting() {
		return eclipseEngine.statisticsCollecting();
	}

	public boolean statisticsReportingEnabled() {
		return statisticsReportingEnabled;
	}

	public void setStatisticsDisplay(Object display) {
		statisticsStreamListener.setTextArea((StyledText) display);
	}

	public EnginePreferences getInstallation() {
		return environment.getInstallation();
	}

	public void updateCompileScratchpadData(String data) {
		compileScratchpadData = data;
	}

	public String getCompileScratchpadData() {
		return compileScratchpadData;
	}

	public void updateStatisticsData(String data) {
	}

	public ArrayList getDelayedGoalsData() {
		return delayedGoals;
	}

	public void updateDelayedGoalsData(final ArrayList delayedGoals) {
		this.delayedGoals = delayedGoals;
		final ECLiPSeExecutionEnvironment eev = this;
		Runnable r = new Runnable() {
			public void run() {
				DelayedGoals delayedGoalsView = ((DelayedGoals) getSite()
						.getPage()
						.findView(
								"org.eclipseclp.saros.development.views.DelayedGoals"));
				if (delayedGoalsView != null) {
					delayedGoalsView.updateDelayedGoalsData(eev, delayedGoals);
				}
			}
		};
		Display.getDefault().asyncExec(r);
	}

	public String getTraceLogData() {
		return traceLogData;
	}

	public void traceSource(final ECLiPSeTraceLine traceLine) {
		if (traceLine.getFilename() == null) {
			return;
		}

		Runnable r = new Runnable() {
			public void run() {
				IWorkbenchPage page = PlatformUI.getWorkbench()
						.getActiveWorkbenchWindow().getActivePage();
				if (page == null) {
					return;
				}
				Path path = new Path(traceLine.getFilename());
				IWorkspaceRoot workspaceRoot = ResourcesPlugin.getWorkspace()
						.getRoot();
				try {
					ECLiPSeSourceEditor editor = (ECLiPSeSourceEditor) page
							.openEditor(new FileEditorInput(workspaceRoot
									.getFileForLocation(path)),
									ECLiPSeSourceEditor.ID, true);
					editor.setHighlightRange(traceLine.getSourceStart(),
							traceLine.getSourceEnd()
									- traceLine.getSourceStart(), true);
					editor.getSelectionProvider().setSelection(
							new TextSelection(traceLine.getSourceStart(),
									traceLine.getSourceEnd()
											- traceLine.getSourceStart()));
				} catch (PartInitException e) {
					Status status = new Status(
							IStatus.ERROR,
							"Saros",
							0,
							"Exception caught whilst opening source editor for tracing: ",
							e);
					ErrorDialog.openError(page.getWorkbenchWindow().getShell(),
							"Saros Development Plugin Error", e.getMessage(),
							status);
				}
			}
		};
		Display.getDefault().asyncExec(r);
	}

	public void updateCallStackData(final ECLiPSeTraceLine traceLine) {
		// traceLine == null : clear the call stack
		if (traceLine == null) {
			callStackData = new ArrayList();
		} else {
			int currentDisplayedDepth = callStackData.size() - 1;
			int currentDepth = traceLine.getDepth();
			if (currentDisplayedDepth == currentDepth - 1) {
				callStackData.add(traceLine);
			} else if (currentDisplayedDepth == currentDepth) {
				callStackData.set(currentDepth, traceLine);
			} else {
				if (currentDisplayedDepth > currentDepth) {
					while (currentDisplayedDepth > currentDepth) {
						callStackData.remove(currentDisplayedDepth);
						currentDisplayedDepth--;
					}
				} else if (currentDisplayedDepth < currentDepth) {
					while (currentDisplayedDepth < currentDepth) {
						currentDisplayedDepth++;
						callStackData.add(new ECLiPSeTraceLine(
								currentDisplayedDepth, null, "", -1, null, -1,
								null, -1, -1));
					}
				}
				callStackData.set(currentDepth, traceLine);
			}
		}

		if (autoRefreshTracer && traceLine != null) {
			requestCallStackRefresh();
		} else {
			final ECLiPSeExecutionEnvironment eev = this;
			Runnable r = new Runnable() {
				public void run() {
					CallStack callStackView = ((CallStack) getSite()
							.getPage()
							.findView(
									"org.eclipseclp.saros.development.views.CallStack"));
					if (callStackView != null) {
						callStackView.updateCallStackData(eev, callStackData);
					}
					autoRefreshDelayedGoals();
				}
			};
			Display.getDefault().asyncExec(r);
		}
	}

	public ArrayList getCallStackData() {
		return callStackData;
	}

	public void updateTraceLogData(final String traceText) {
		// traceText == null : clear the trace log
		if (traceText == null) {
			traceLogData = "";
		} else {
			traceLogData += traceText + "\n ";
		}

		final ECLiPSeExecutionEnvironment eev = this;
		Runnable r = new Runnable() {
			public void run() {
				TraceLog traceLogView = ((TraceLog) getSite()
						.getPage()
						.findView(
								"org.eclipseclp.saros.development.views.TraceLog"));
				if (traceLogView != null) {
					traceLogView.updateTraceLogData(eev, traceText);
				}
			}
		};
		Display.getDefault().asyncExec(r);
	}

	public void postShutdown(IWorkbench workbench) {
	}

	public boolean preShutdown(IWorkbench workbench, boolean forced) {
		eclipseEngine.shutdown();
		return true;
	}

	public boolean isAutoRefreshingDelayedGoals() {
		return autoRefreshDelayedGoals;
	}

	public boolean isAutoRefreshingTracer() {
		return autoRefreshTracer;
	}

	public void setAutoRefreshingDelayedGoals(boolean autoRefresh) {
		autoRefreshDelayedGoals = autoRefresh;
	}

	public void setAutoRefreshingTracer(boolean autoRefresh) {
		autoRefreshTracer = autoRefresh;
	}

	public void analyseGoalFailure(ECLiPSeTraceLine traceLine) {
		AsynchronousRPCresult r = new AsynchronousRPCresult() {
			public void run() {
				int fculprit = ((Integer) ((CompoundTerm) getResult().arg(2))
						.arg(1)).intValue();
				int invoc = ((Integer) ((CompoundTerm) getResult().arg(2))
						.arg(2)).intValue();

				if (fculprit > invoc) {
					MessageBox messageBox = new MessageBox(PlatformUI
							.getWorkbench().getActiveWorkbenchWindow()
							.getShell(), SWT.YES | SWT.NO | SWT.ICON_QUESTION);
					messageBox
							.setMessage("Most recent failure was caused by goal with invocation number '"
									+ fculprit
									+ "'. Do you wish to jump there now?");
					if (messageBox.open() == SWT.YES) {
						jumpInvocationNumber(fculprit);
					}
				} else {
					MessageBox messageBox = new MessageBox(PlatformUI
							.getWorkbench().getActiveWorkbenchWindow()
							.getShell(), SWT.OK | SWT.ICON_INFORMATION);
					messageBox
							.setMessage("Most recent failure was caused by goal with invocation number '"
									+ fculprit
									+ "'. To jump there, re-run the query with the tracer enabled and select \"Analyse Failure\" immediately.");
					messageBox.open();
				}
			}
		};
		interruptingRpc(new CompoundTermImpl(":", new Atom("sepia_kernel"),
				new CompoundTermImpl("failure_culprit", null, null)),
				"No fail culprit stored yet!", r, null);
	}

	public void forceGoalFailure(ECLiPSeTraceLine traceLine) {
		int invocation;
		if (traceLine == null) {
			invocation = 0;
		} else {
			invocation = traceLine.getInvocation();
			invocation = invocation > 0 ? invocation : 0;
		}
		sendTracerCommand(null, new CompoundTermImpl(":",
				new Atom("tracer_tcl"), new CompoundTermImpl(
						"set_tracer_command", new CompoundTermImpl(
								PreferenceConstants.tracerFailCommand,
								new Integer(invocation)))));
	}

	public void jumpInvocationDepth(ECLiPSeTraceLine traceLine) {
		int depth;
		if (traceLine == null) {
			depth = 0;
		} else {
			depth = traceLine.getDepth();
			depth = depth > 0 ? depth : 0;
		}
		if (traceLine != null && traceLine.getDepth() != -1) {
			sendTracerCommand(
					assembleFilterCommand(null, new CompoundTermImpl("..",
							new Integer(traceLine.getDepth()), new Integer(
									depth)), null, null, null),
					assembleTracerCommand(PreferenceConstants.tracerToDepthCommand));
		}
	}

	public void jumpInvocationNumber(ECLiPSeTraceLine traceLine) {
		int invocation;
		if (traceLine == null) {
			invocation = 0;
		} else {
			invocation = traceLine.getInvocation();
			invocation = invocation > 0 ? invocation : 0;
		}
		jumpInvocationNumber(traceLine.getInvocation());
	}

	private void jumpInvocationNumber(int invocation) {
		sendTracerCommand(assembleFilterCommand(new Integer(invocation), null,
				null, null, null),
				assembleTracerCommand(PreferenceConstants.tracerToInvocCommand));
	}

	public void inspectCurrentGoal() {
		IWorkbenchPage page = PlatformUI.getWorkbench()
				.getActiveWorkbenchWindow().getActivePage();
		try {
			Inspector inspector = (Inspector) page
					.showView("org.eclipseclp.saros.development.views.Inspector");
			inspector.inspectCurrentGoal(this);
		} catch (PartInitException e) {
			Status status = new Status(IStatus.ERROR, "Saros", 0,
					"Exception caught whilst opening the Term Inspector: ", e);
			ErrorDialog.openError(Saros.getDefault().getWorkbench()
					.getDisplay().getActiveShell(),
					"Saros Development Plugin Error", e.getMessage(), status);
		}
	}

	public void inspectInvokedGoal(int invocation) {
		IWorkbenchPage page = PlatformUI.getWorkbench()
				.getActiveWorkbenchWindow().getActivePage();
		try {
			Inspector inspector = (Inspector) page
					.showView("org.eclipseclp.saros.development.views.Inspector");
			inspector.inspectInvokedGoal(this, invocation);
		} catch (PartInitException e) {
			Status status = new Status(IStatus.ERROR, "Saros", 0,
					"Exception caught whilst opening the Term Inspector: ", e);
			ErrorDialog.openError(Saros.getDefault().getWorkbench()
					.getDisplay().getActiveShell(),
					"Saros Development Plugin Error", e.getMessage(), status);
		}
	}

	public ArrayList getInspectorData() {
		return inspectors;
	}

	private void refreshCurrentTraceLine(final ECLiPSeTraceLine traceLine) {
		final ECLiPSeInteractionInterface eev = this;
		AsynchronousRPCresult r = new AsynchronousRPCresult() {
			public void run() {
				CompoundTerm refreshedLine = (CompoundTerm) getResult().arg(2);
				traceLine.setDepth(((Integer) refreshedLine.arg(1)).intValue());
				traceLine.setStyle((String) refreshedLine.arg(2));
				traceLine.setLineText((String) refreshedLine.arg(3));
				traceLine.setInvocation(((Integer) refreshedLine.arg(4))
						.intValue());
				CallStack callStackView = ((CallStack) getSite()
						.getPage()
						.findView(
								"org.eclipseclp.saros.development.views.CallStack"));
				if (callStackView != null) {
					callStackView.updateCallStackData(eev, callStackData);
				}
			}
		};
		interruptingRpc(new CompoundTermImpl(":", new Atom("tracer_tcl"),
				new CompoundTermImpl("get_current_traceline", null, null, null,
						null)), "Unable to refresh current call stack goal.",
				r, null);
	}

	public void requestCallStackRefresh() {
		final ECLiPSeInteractionInterface eev = this;
		AsynchronousRPCresult r = new AsynchronousRPCresult() {
			public void run() {
				LinkedList refreshInfo = (LinkedList) ((CompoundTerm) getResult()
						.arg(2)).arg(1);
				ECLiPSeTraceLine traceLine;
				CompoundTerm refreshedLine;
				for (int cnt = 0; cnt < refreshInfo.size(); cnt++) {
					if (cnt == callStackData.size()) {
						break;
					}
					refreshedLine = (CompoundTerm) refreshInfo.get(cnt);
					traceLine = (ECLiPSeTraceLine) callStackData.get(cnt);
					traceLine.setDepth(((Integer) refreshedLine.arg(1))
							.intValue());
					traceLine.setInvocation(((Integer) refreshedLine.arg(2))
							.intValue());
					traceLine.setPriority(((Integer) refreshedLine.arg(3))
							.intValue());
					traceLine.setLineText((String) refreshedLine.arg(4));
				}
				CallStack callStackView = ((CallStack) getSite()
						.getPage()
						.findView(
								"org.eclipseclp.saros.development.views.CallStack"));
				if (callStackView != null) {
					callStackView.setCallStackData(eev, callStackData);
				}

				autoRefreshDelayedGoals();
			}
		};
		interruptingRpc(new CompoundTermImpl(":", new Atom("tracer_tcl"),
				new CompoundTermImpl("get_ancestors", (Object) null)),
				"Unable to refresh goal call stack.", r, null);
	}

	private void autoRefreshDelayedGoals() {
		if (autoRefreshDelayedGoals) {
			DelayedGoals delayedGoalsView = ((DelayedGoals) getSite()
					.getPage()
					.findView(
							"org.eclipseclp.saros.development.views.DelayedGoals"));
			if (delayedGoalsView != null) {
				delayedGoalsView.refreshDelayedGoals(this);
			}
		}
	}

	public boolean getMultipleExecutionEnvironmentInstanceSupport() {
		return Saros.getDefault()
				.getMultipleExecutionEnvironmentInstanceSupport();
	}

	public void activate() {
		final ECLiPSeExecutionEnvironment eev = this;
		Runnable r = new Runnable() {
			public void run() {
				enablePopupMenu(true);
				SarosViewNotifier.notifyViews(getSite().getPage(), eev,
						SarosViewNotifier.ACTIVATE);
			}
		};
		Display.getDefault().asyncExec(r);
	}

	public void deactivate() {
		final ECLiPSeExecutionEnvironment eev = this;
		Runnable r = new Runnable() {
			public void run() {
				enablePopupMenu(false);
				SarosViewNotifier.notifyViews(getSite().getPage(), eev,
						SarosViewNotifier.DEACTIVATE);
			}
		};
		Display.getDefault().asyncExec(r);
	}

	private void enablePopupMenu(boolean enable) {
		if (popupMenu.isDisposed()) {
			return;
		}
		MenuItem[] items = popupMenu.getItems();
		for (int cnt = 0; cnt < items.length; cnt++) {
			items[cnt].setEnabled(enable);
		}
	}
}
