/*******************************************************************************
 * Copyright (c) 2006 Cisco Technology, Inc and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Author: Andrew Cheadle, CrossCore Optimization Limited
 *
 * Contributors:
 *     Cisco Technology, Inc - Initial implementation
 *******************************************************************************/

package org.eclipseclp.saros.development.editors;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IPersistableElement;

public class ECLiPSeExecutionEnvironmentEditorInput extends Object implements
		IEditorInput {

	public ECLiPSeExecutionEnvironmentEditorInput(IResource resource) {
		this.resource = resource;
		this.isLaunchInput = false;
	}

	public ECLiPSeExecutionEnvironmentEditorInput(IResource resource,
			boolean isLaunchInput) {
		this.resource = resource;
		this.isLaunchInput = isLaunchInput;
	}
	
	public ECLiPSeExecutionEnvironmentEditorInput(IResource resource,
			boolean isLaunchInput, boolean isLaunchActionInput) {
		this.resource = resource;
		this.isLaunchInput = isLaunchInput;
		this.isLaunchActionInput = isLaunchActionInput;
	}

	public boolean exists() {
		if (resource != null) {
			return resource.exists();
		}

		return false;
	}

	public ImageDescriptor getImageDescriptor() {
		return null;
	}

	public String getName() {
		if (resource != null) {
			if (resource instanceof IFile) {
				return new String("Saros ECLiPSe Execution Environment: "
						+ resource.getProject().getName() + "/"
						+ resource.getName());
			}
			return new String("Saros ECLiPSe Execution Environment: "
					+ resource.getProject().getName());
		}

		return new String("Saros ECLiPSe Execution Environment");
	}

	public IPersistableElement getPersistable() {
		return null;
	}

	public String getToolTipText() {
		return getName();
	}

	public Object getAdapter(Class adapter) {
		if (adapter == IResource.class) {
			return resource;
		}
		return null;
	}

	public boolean equals(ECLiPSeExecutionEnvironmentEditorInput input) {
		if (resource != null) {
			return getName().equals(input.getName());
		}
		return false;
	}

	public int hashcode() {
		if (resource != null) {
			return resource.hashCode();
		}
		return 0;
	}

	public IProject getProject() {
		if (resource != null) {
			if (resource instanceof IProject) {
				return (IProject) resource;
			}
			return resource.getProject();
		}
		return null;
	}

	public IFile getFile() {
		if (resource != null && resource instanceof IFile) {
			return (IFile) resource;
		}
		return null;
	}

	public IResource getResource() {
		return resource;
	}

	public void setLaunchQuery(boolean hasLaunchQuery, String launchFile,
			String launchModule, String runMethod, String launchQuery) {
		this.hasLaunchQuery = hasLaunchQuery;
		this.launchFile = launchFile;
		this.launchModule = launchModule;
		this.launchQuery = launchQuery;
		this.launchRunMethod = runMethod;
	}

	public boolean isLaunchInput() {
		return isLaunchInput;
	}

	public boolean isLaunchActionInput() {
		return isLaunchActionInput;
	}

	public boolean hasLaunchQuery() {
		return hasLaunchQuery;
	}

	public String getLaunchFile() {
		return launchFile;
	}

	public String getLaunchModule() {
		return launchModule;
	}

	public String getLaunchQuery() {
		return launchQuery;
	}

	public String getLaunchRunMethod() {
		return launchRunMethod;
	}

	private IResource resource = null;

	private boolean isLaunchInput = false;

	private boolean isLaunchActionInput = false;

	private boolean hasLaunchQuery = false;

	private String launchFile = "";

	private String launchModule = "";

	private String launchQuery = "";

	private String launchRunMethod = "";
}
