/*******************************************************************************
 * Copyright (c) 2006 Cisco Technology, Inc and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Author: Andrew Cheadle, CrossCore Optimization Limited
 *
 * Contributors:
 *     Cisco Technology, Inc - Initial implementation
 *******************************************************************************/

package org.eclipseclp.saros.development.launchers;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.TimeZone;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipseclp.saros.development.Saros;
import org.eclipseclp.saros.development.views.SarosConsole;
import org.eclipseclp.saros.runtime.Environment;

public class ECLiPSeConsole {

	public ECLiPSeConsole(final boolean topLevel, final Environment environment) {
		Calendar calendar = Calendar.getInstance(TimeZone.getDefault());

		String DATE_FORMAT = "yyyy-MM-dd HH:mm:ss";
		SimpleDateFormat simpleDateFormat = new SimpleDateFormat(DATE_FORMAT);
		simpleDateFormat.setTimeZone(TimeZone.getDefault());
		final String consoleName = simpleDateFormat.format(calendar.getTime());
		IWorkbenchPage page = PlatformUI.getWorkbench()
				.getActiveWorkbenchWindow().getActivePage();
		try {
			page
					.showView("org.eclipseclp.saros.development.views.SarosConsole");
		} catch (PartInitException e) {
			Status status = new Status(IStatus.ERROR, "Saros", 0,
					"Exception caught whilst opening Saros Console view: ", e);
			ErrorDialog.openError(Saros.getDefault().getWorkbench()
					.getDisplay().getActiveShell(),
					"Saros Development Plugin Error", e.getMessage(), status);
			return;
		}

		SarosConsole console = (SarosConsole) page
				.findView("org.eclipseclp.saros.development.views.SarosConsole");
		console.createConsole(consoleName, topLevel, environment);
	}
}
