/*******************************************************************************
 * Copyright (c) 2006 Cisco Technology, Inc and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Author: Andrew Cheadle, CrossCore Optimization Limited
 *
 * Contributors:
 *     Cisco Technology, Inc - Initial implementation
 *******************************************************************************/

package org.eclipseclp.saros.development.launchers;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.debug.ui.ILaunchShortcut;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.part.FileEditorInput;
import org.eclipseclp.saros.runtime.Environment;

public class SarosConsoleShortcut implements ILaunchShortcut {

	public void launch(ISelection selection, String mode) {
		Environment environment = null;
		if (selection != null && selection.isEmpty() == false
				&& selection instanceof IStructuredSelection) {
			IStructuredSelection ssel = (IStructuredSelection) selection;
			if (ssel.size() > 1)
				return;
			Object obj = ssel.getFirstElement();
			if (obj instanceof IResource) {
				if (obj instanceof IFile) {
					environment = new Environment(Environment.FILE_PROPERTIES,
							((IFile) obj).getProject(), (IFile) obj);
				} else if (obj instanceof IProject) {
					environment = new Environment(
							Environment.PROJECT_PROPERTIES, (IProject) obj,
							null);
				}
			}
			if (environment != null) {
				new ECLiPSeConsole(true, environment);
			}
		}
	}

	public void launch(IEditorPart editor, String mode) {
		if (!(editor.getEditorInput() instanceof FileEditorInput)) {
			return;
		}
		IFile file = ((FileEditorInput) (editor.getEditorInput())).getFile();
		Environment environment = new Environment(Environment.FILE_PROPERTIES,
				file.getProject(), file);
		new ECLiPSeConsole(true, environment);
	}
}
