/*******************************************************************************
 * Copyright (c) 2006 Cisco Technology, Inc and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Author: Andrew Cheadle, CrossCore Optimization Limited
 *
 * Contributors:
 *     Cisco Technology, Inc - Initial implementation
 *******************************************************************************/

package org.eclipseclp.saros.development.launchers;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.debug.ui.ILaunchShortcut;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.FileEditorInput;
import org.eclipseclp.saros.development.Saros;
import org.eclipseclp.saros.development.editors.ECLiPSeExecutionEnvironment;
import org.eclipseclp.saros.development.editors.ECLiPSeExecutionEnvironmentEditorInput;

public class SarosShortcut implements ILaunchShortcut {

	public void launch(ISelection selection, String mode) {
		if (selection != null && selection.isEmpty() == false
				&& selection instanceof IStructuredSelection) {
			IStructuredSelection ssel = (IStructuredSelection) selection;
			if (ssel.size() > 1)
				return;
			Object obj = ssel.getFirstElement();
			if (obj instanceof IResource) {
				ECLiPSeExecutionEnvironmentEditorInput editorInput = null;
				if (obj instanceof IProject) {
					editorInput = new ECLiPSeExecutionEnvironmentEditorInput(
							(IResource) obj, false, true);
				} else {
					editorInput = new ECLiPSeExecutionEnvironmentEditorInput(
							(IResource) obj);
				}
				try {
					PlatformUI.getWorkbench().getActiveWorkbenchWindow()
							.getActivePage().openEditor(editorInput,
									ECLiPSeExecutionEnvironment.ID);
				} catch (PartInitException e) {
					Status status = new Status(
							IStatus.ERROR,
							"Saros",
							0,
							"Exception caught whilst launching execution environment: ",
							e);
					ErrorDialog.openError(Saros.getDefault().getWorkbench()
							.getDisplay().getActiveShell(),
							"Saros Development Plugin Error", e.getMessage(),
							status);
				}
			}
		}
	}

	public void launch(IEditorPart editor, String mode) {
		if (!(editor.getEditorInput() instanceof FileEditorInput)) {
			return;
		}
		IFile file = ((FileEditorInput) (editor.getEditorInput())).getFile();
		try {
			PlatformUI.getWorkbench().getActiveWorkbenchWindow()
					.getActivePage().openEditor(
							new ECLiPSeExecutionEnvironmentEditorInput(file),
							ECLiPSeExecutionEnvironment.ID);
		} catch (PartInitException e) {
			Status status = new Status(
					IStatus.ERROR,
					"Saros",
					0,
					"Exception caught whilst launching execution environment: ",
					e);
			ErrorDialog.openError(Saros.getDefault().getWorkbench()
					.getDisplay().getActiveShell(),
					"Saros Development Plugin Error", e.getMessage(), status);
		}
	}
}
