/*******************************************************************************
 * Copyright (c) 2006 Cisco Technology, Inc and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Author: Andrew Cheadle, CrossCore Optimization Limited
 *
 * Contributors:
 *     Cisco Technology, Inc - Initial implementation
 *******************************************************************************/

package org.eclipseclp.saros.development.perspectives;

import org.eclipse.ui.IFolderLayout;
import org.eclipse.ui.IPageLayout;
import org.eclipse.ui.IPerspectiveFactory;

public class SarosDevelopment implements IPerspectiveFactory {

	public void createInitialLayout(IPageLayout layout) {
		initialiseActions(layout);
		initialiseLayout(layout);
	}

	private void initialiseActions(IPageLayout layout) {
		layout
				.addNewWizardShortcut("org.eclipseclp.saros.development.wizards.NewProjectWizard");
		layout
				.addNewWizardShortcut("org.eclipseclp.saros.development.wizards.NewModuleWizard");
		layout.addShowViewShortcut(IPageLayout.ID_RES_NAV);
		layout.addShowViewShortcut(IPageLayout.ID_BOOKMARKS);
		layout.addShowViewShortcut(IPageLayout.ID_PROBLEM_VIEW);
		layout.addShowViewShortcut(IPageLayout.ID_TASK_LIST);
		layout
				.addPerspectiveShortcut("org.eclipseclp.saros.development.perspectives.SarosExecution");
		layout
				.addShowViewShortcut("org.eclipseclp.saros.development.views.Modules");
		layout
				.addShowViewShortcut("org.eclipseclp.saros.development.views.Predicates");
		layout
				.addShowViewShortcut("org.eclipseclp.saros.development.views.Clauses");
		layout
				.addShowViewShortcut("org.eclipseclp.saros.development.views.CallHierarchy");
	}

	private void initialiseLayout(IPageLayout layout) {
		// Get the editor area.
		String editorArea = layout.getEditorArea();

		// Top left: Resource Navigator view and Bookmarks view placeholder
		IFolderLayout topLeft = layout.createFolder("topLeft",
				IPageLayout.LEFT, 0.25f, editorArea);
		topLeft.addView(IPageLayout.ID_RES_NAV);
		topLeft.addPlaceholder(IPageLayout.ID_BOOKMARKS);

		// Top right:
		IFolderLayout topRight = layout.createFolder("topRight",
				IPageLayout.RIGHT, 0.75f, editorArea);
		topRight.addView("org.eclipseclp.saros.development.views.Modules");
		topRight.addView("org.eclipseclp.saros.development.views.Predicates");
		topRight.addView("org.eclipseclp.saros.development.views.Clauses");
		topRight
				.addView("org.eclipseclp.saros.development.views.CallHierarchy");

		IFolderLayout bottomRight = layout.createFolder("bottomRight",
				IPageLayout.BOTTOM, 0.66f, editorArea);
		// Bottom right: Problem view
		bottomRight.addView(IPageLayout.ID_PROBLEM_VIEW);
		// Bottom right: Task List view
		bottomRight.addView(IPageLayout.ID_TASK_LIST);
	}
}
