/*******************************************************************************
 * Copyright (c) 2006 Cisco Technology, Inc and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Author: Andrew Cheadle, CrossCore Optimization Limited
 *
 * Contributors:
 *     Cisco Technology, Inc - Initial implementation
 *******************************************************************************/

package org.eclipseclp.saros.development.views;

import java.util.ArrayList;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.part.ViewPart;
import org.eclipseclp.saros.core.preferences.EnginePreferences;
import org.eclipseclp.saros.core.preferences.PreferenceConstants;
import org.eclipseclp.saros.core.preferences.PreferenceMessages;
import org.eclipseclp.saros.core.preferences.PrintDepthOutputOptionsShell;
import org.eclipseclp.saros.development.editors.ECLiPSeExecutionEnvironment;
import org.eclipseclp.saros.runtime.AsynchronousRPCresult;
import org.eclipseclp.saros.runtime.ECLiPSeInteractionInterface;
import org.eclipseclp.saros.runtime.ECLiPSeTraceLine;
import com.parctechnologies.eclipse.Atom;
import com.parctechnologies.eclipse.CompoundTerm;
import com.parctechnologies.eclipse.CompoundTermImpl;

public class CallStack extends ViewPart implements SarosViewNotification,
		MouseListener {

	public void createPartControl(Composite parent) {
		callStackText = new StyledText(parent, SWT.MULTI | SWT.WRAP
				| SWT.V_SCROLL | SWT.READ_ONLY);
		popupMenu = new Menu(parent.getShell(), SWT.POP_UP);
		predicateInfoItem = new MenuItem(popupMenu, SWT.PUSH);
		predicateInfoItem.setEnabled(false);
		new MenuItem(popupMenu, SWT.SEPARATOR);
		refreshCallStackItem = new MenuItem(popupMenu, SWT.PUSH);
		refreshCallStackItem.setText(PreferenceMessages
				.getString("ECLiPSeStrings.Refresh"));
		refreshCallStackItem.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				activeInteraction.requestCallStackRefresh();
			}
		});
		eventRefreshItem = new MenuItem(popupMenu, SWT.CHECK);
		eventRefreshItem.setText(PreferenceMessages
				.getString("ECLiPSeStrings.RefreshEachTraceEvent"));
		eventRefreshItem.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				activeInteraction.setAutoRefreshingTracer(eventRefreshItem
						.getSelection());
			}
		});
		new MenuItem(popupMenu, SWT.SEPARATOR);
		spyGoalItem = new MenuItem(popupMenu, SWT.CHECK);
		spyGoalItem.setText(PreferenceMessages
				.getString("ECLiPSeStrings.SpyGoal"));
		spyGoalItem.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				spyCurrentGoal();
			}
		});
		inspectGoalItem = new MenuItem(popupMenu, SWT.PUSH);
		inspectGoalItem.setText(PreferenceMessages
				.getString("ECLiPSeStrings.InspectGoal"));
		inspectGoalItem.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				activeInteraction.inspectInvokedGoal(currentTraceLine
						.getInvocation());
			}
		});
		/*
		 * observeGoalItem = new MenuItem(popupMenu, SWT.PUSH);
		 * observeGoalItem.setText(PreferenceMessages
		 * .getString("ECLiPSeStrings.ObserveGoal"));
		 * observeGoalItem.addSelectionListener(new SelectionAdapter() { public
		 * void widgetSelected(SelectionEvent event) {
		 * activeInteraction.observeGoal(currentTraceLine); } });
		 */
		new MenuItem(popupMenu, SWT.SEPARATOR);
		forceGoalFailureItem = new MenuItem(popupMenu, SWT.PUSH);
		forceGoalFailureItem.setText(PreferenceMessages
				.getString("ECLiPSeStrings.ForceGoalFailure"));
		forceGoalFailureItem.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				activeInteraction.forceGoalFailure(currentTraceLine);
			}
		});
		analyseGoalFailureItem = new MenuItem(popupMenu, SWT.PUSH);
		analyseGoalFailureItem.setText(PreferenceMessages
				.getString("ECLiPSeStrings.AnalyseGoalFailure"));
		analyseGoalFailureItem.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				activeInteraction.analyseGoalFailure(currentTraceLine);
			}
		});
		new MenuItem(popupMenu, SWT.SEPARATOR);
		jumpInvocationItem = new MenuItem(popupMenu, SWT.PUSH);
		jumpInvocationItem.setText(PreferenceMessages
				.getString("ECLiPSeStrings.JumpInvocationNumber"));
		jumpInvocationItem.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				activeInteraction.jumpInvocationNumber(currentTraceLine);
			}
		});
		jumpDepthItem = new MenuItem(popupMenu, SWT.PUSH);
		jumpDepthItem.setText(PreferenceMessages
				.getString("ECLiPSeStrings.JumpInvocationDepth"));
		jumpDepthItem.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				activeInteraction.jumpInvocationDepth(currentTraceLine);
			}
		});
		new MenuItem(popupMenu, SWT.SEPARATOR);
		printOptionsItem = new MenuItem(popupMenu, SWT.PUSH);
		printOptionsItem.setText(PreferenceMessages
				.getString("ECLiPSeStrings.ChangeOutputOptions")
				+ "...");
		printOptionsItem.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				changeOutputOptions();
			}
		});
		callStackText.setDoubleClickEnabled(false);
		callStackText.addMouseListener(this);
		IWorkbenchPage page = getSite().getPage();
		if (page == null
				|| !(page.getActiveEditor() instanceof ECLiPSeExecutionEnvironment)) {
			enableWidgets(false);
		} else {
			editorActivated((ECLiPSeInteractionInterface) page
					.getActiveEditor());
		}
	}

	private void enableWidgets(boolean enable) {
		popupMenu.setEnabled(enable);
		eventRefreshItem.setEnabled(enable);
		refreshCallStackItem.setEnabled(enable);
		spyGoalItem.setEnabled(enable);
		inspectGoalItem.setEnabled(enable);
		// observeGoalItem.setEnabled(enable);
		forceGoalFailureItem.setEnabled(enable);
		analyseGoalFailureItem.setEnabled(enable);
		jumpInvocationItem.setEnabled(enable);
		jumpDepthItem.setEnabled(enable);
		printOptionsItem.setEnabled(enable);
	}

	public void setFocus() {
		callStackText.setFocus();
	}

	protected void clearText() {
		callStackText.replaceTextRange(0, callStackText.getCharCount(), "");
	}

	public void mouseDown(MouseEvent event) {
		if (event.button == 3) {
			if (popupMenu.getEnabled()) {
				int currentIndex = callStackText.getLineIndex(event.y);
				if (callStackData != null
						&& currentIndex < callStackData.size()) {
					currentTraceLine = (ECLiPSeTraceLine) callStackData
							.get(currentIndex);
					final int priority = currentTraceLine.getPriority();
					AsynchronousRPCresult r = new AsynchronousRPCresult() {
						public void run() {
							final CompoundTerm refreshedLine = (CompoundTerm) getResult()
									.arg(2);

							currentPredSpec = (String) refreshedLine.arg(3);
							if (currentPredSpec.equals("unknown")) {
								predicateInfoItem.setText("");
								spyGoalItem.setEnabled(false);
								spyGoalItem.setText("Spy Goal");
								currentPredSpec = null;
								currentPredModule = null;
							} else {
								final String module = ((Atom) refreshedLine
										.arg(5)).functor();
								predicateInfoItem
										.setText((String) refreshedLine.arg(4)
												+ " @ " + module + " <"
												+ priority + ">");
								spyGoalItem.setEnabled(true);
								spyGoalItem.setText("Spy " + currentPredSpec);
								if (refreshedLine.arg(10).equals(
										PreferenceConstants.on)) {
									spyGoalItem.setSelection(true);
								} else {
									spyGoalItem.setSelection(false);
								}
								currentPredModule = ((Atom) refreshedLine
										.arg(6)).functor();
							}

						}
					};
					Object args[] = {
							new Integer(currentTraceLine.getInvocation()),
							new Integer(1), null, null, null, null, null, null,
							null, null };
					activeInteraction.interruptingRpc(new CompoundTermImpl(":",
							new Atom("tracer_tcl"), new CompoundTermImpl(
									"saros_get_goal_info_by_invoc", args)),
							"Unable to obtain call stack goal info for '"
									+ currentTraceLine.getLineText() + "'.", r,
							null);

					jumpInvocationItem.setText(PreferenceMessages
							.getString("ECLiPSeStrings.JumpInvocationNumber")
							+ " " + currentTraceLine.getInvocation());
					jumpDepthItem.setText(PreferenceMessages
							.getString("ECLiPSeStrings.JumpInvocationDepth")
							+ " " + currentTraceLine.getDepth());
				} else {
					currentTraceLine = null;
					currentPredSpec = null;
					currentPredModule = null;
					spyGoalItem.setEnabled(false);
					// observeGoalItem.setEnabled(false);
					inspectGoalItem.setEnabled(false);
				}
			}
			popupMenu.setVisible(true);
		}
	}

	private void spyCurrentGoal() {
		if (currentPredSpec != null) {
			Atom spyValue;
			if (spyGoalItem.getSelection()) {
				spyValue = new Atom(PreferenceConstants.on);
			} else {
				spyValue = new Atom(PreferenceConstants.off);
			}
			CompoundTerm term = new CompoundTermImpl(":",
					new Atom("tracer_tcl"), new CompoundTermImpl(
							"set_flag_string", currentPredSpec,
							new Atom("spy"), spyValue, new Atom(
									currentPredModule)));
			activeInteraction.interruptingRpc(term,
					"Unable to set spy predicate property for '"
							+ currentPredSpec + "'.");
		}
	}

	public void mouseDoubleClick(MouseEvent event) {
		activeInteraction.inspectInvokedGoal(callStackText
				.getLineIndex(event.y));
	}

	public void mouseUp(MouseEvent event) {
	}

	private void changeOutputOptions() {
		EnginePreferences installation = (EnginePreferences) activeInteraction
				.getInstallation().clone();
		installation = (new PrintDepthOutputOptionsShell(installation))
				.open(this.getSite().getShell());

		String mode = installation
				.getStringFlag(PreferenceConstants.outputMode);
		CompoundTerm command = new CompoundTermImpl(":",
				new Atom("tracer_tcl"), new CompoundTermImpl(
						"set_tracer_output_modes", mode));
		activeInteraction.interruptingRpc(command,
				"Unable to set tracer output mode");

		Integer printDepth = new Integer(installation
				.getIntFlag(PreferenceConstants.printDepth));
		command = new CompoundTermImpl(":", new Atom("tracer_tcl"),
				new CompoundTermImpl("set_tracer_print_depth", printDepth));
		activeInteraction.interruptingRpc(command,
				"Unable to set tracer print depth");
	}

	public void updateCallStackData(
			ECLiPSeInteractionInterface eclipseInteraction,
			ArrayList callStackData) {
		if (eclipseInteraction == activeInteraction) {
			this.callStackData = callStackData;
			int currentDepth = callStackData.size() - 1;
			int currentDisplayedDepth = callStackText.getLineCount() - 1;

			if (currentDepth == -1) {
				clearText();
				currentTraceLine = null;
				currentPredSpec = null;
				currentPredModule = null;
				return;
			}

			ECLiPSeTraceLine lastTraceLine = (ECLiPSeTraceLine) callStackData
					.get(currentDepth);

			if (currentDisplayedDepth == currentDepth - 1) {
				callStackText.append(callStackText.getLineDelimiter()
						+ lastTraceLine.getLineText());
			} else if (currentDisplayedDepth == currentDepth) {
				int lineStart = callStackText
						.getOffsetAtLine(currentDisplayedDepth);
				callStackText.replaceTextRange(lineStart, callStackText
						.getCharCount()
						- lineStart, " " + lastTraceLine.getLineText());
			} else if (currentDisplayedDepth > currentDepth) {
				int lineStart = callStackText.getOffsetAtLine(currentDepth);
				callStackText.replaceTextRange(lineStart, callStackText
						.getCharCount()
						- lineStart, " " + lastTraceLine.getLineText()
						+ callStackText.getLineDelimiter());
			} else if (currentDisplayedDepth < currentDepth) {
				int count = currentDepth - currentDisplayedDepth;
				for (int cnt = 0; cnt < count; cnt++) {
					callStackText.append(callStackText.getLineDelimiter());
				}
				callStackText.append(" " + lastTraceLine.getLineText()
						+ callStackText.getLineDelimiter());
			}
		}
	}

	public void setCallStackData(
			ECLiPSeInteractionInterface eclipseInteraction,
			ArrayList callStackData) {
		if (eclipseInteraction == activeInteraction) {
			this.callStackData = callStackData;
			currentTraceLine = null;
			currentPredSpec = null;
			currentPredModule = null;
			clearText();
			for (int cnt = 0; cnt < callStackData.size(); cnt++) {
				callStackText.append(" "
						+ ((ECLiPSeTraceLine) callStackData.get(cnt))
								.getLineText()
						+ callStackText.getLineDelimiter());
			}
		}
	}

	public void configureEngine(ECLiPSeInteractionInterface eclipseInteraction) {
	}

	public void editorActivated(ECLiPSeInteractionInterface eclipseInteraction) {
		if (eclipseInteraction == activeInteraction
				&& activeInteraction != null) {
			return;
		}
		activeInteraction = eclipseInteraction;
		eventRefreshItem.setSelection(eclipseInteraction
				.isAutoRefreshingTracer());
		setCallStackData(eclipseInteraction, eclipseInteraction
				.getCallStackData());
		enableWidgets(true);
	}

	public void editorDeactivated(ECLiPSeInteractionInterface eclipseInteraction) {
		deconfigureEngine(eclipseInteraction);
	}

	public void deconfigureEngine(ECLiPSeInteractionInterface eclipseInteraction) {
		if (activeInteraction == eclipseInteraction) {
			enableWidgets(false);
			clearText();
			callStackData = null;
			currentTraceLine = null;
			currentPredSpec = null;
			currentPredModule = null;
		}
	}

	public void viewClosed(ECLiPSeInteractionInterface eclipseInteraction) {
	}

	StyledText callStackText = null;

	private Menu popupMenu = null;

	private MenuItem predicateInfoItem = null;

	private MenuItem eventRefreshItem = null;

	private MenuItem refreshCallStackItem = null;

	private MenuItem spyGoalItem = null;

	private MenuItem inspectGoalItem = null;

	private MenuItem observeGoalItem = null;

	private MenuItem forceGoalFailureItem = null;

	private MenuItem analyseGoalFailureItem = null;

	private MenuItem jumpInvocationItem = null;

	private MenuItem jumpDepthItem = null;

	private MenuItem printOptionsItem = null;

	private ECLiPSeInteractionInterface activeInteraction = null;

	private ArrayList callStackData = null;

	private ECLiPSeTraceLine currentTraceLine = null;

	private String currentPredSpec = null;

	private String currentPredModule = null;
}
