/*******************************************************************************
 * Copyright (c) 2006 Cisco Technology, Inc and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Author: Andrew Cheadle, CrossCore Optimization Limited
 *
 * Contributors:
 *     Cisco Technology, Inc - Initial implementation
 *******************************************************************************/

package org.eclipseclp.saros.development.views;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.TabFolder;
import org.eclipse.swt.widgets.TabItem;
import org.eclipse.ui.part.ViewPart;
import org.eclipseclp.saros.core.preferences.PreferenceMessages;
import org.eclipseclp.saros.runtime.ECLiPSeConsoleEngineRuntime;
import org.eclipseclp.saros.runtime.Environment;

public class SarosConsole extends ViewPart {

	public void createPartControl(Composite parent) {
		tabFolder = new TabFolder(parent, SWT.BORDER);
	}

	public void setFocus() {
	}

	public void createConsole(final String name, final boolean interactive,
			final Environment environment) {
		Runnable r = new Runnable() {
			public void run() {
				final TabItem tabItem = new TabItem(tabFolder, SWT.NONE);
				tabItem.setText(name);
				int styleMask = SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL
						| SWT.BORDER;
				if (!interactive) {
					styleMask |= SWT.READ_ONLY;
				}
				StyledText console = new StyledText(tabFolder, styleMask);
				tabItem.setControl(console);
				Menu popupMenu = new Menu(console.getShell(), SWT.POP_UP);
				MenuItem item = new MenuItem(popupMenu, SWT.PUSH);
				item.setText(PreferenceMessages
						.getString("ECLiPSeStrings.Terminate"));
				console.setMenu(popupMenu);
				tabFolder.setSelection(tabItem);
				console.setFocus();
				final ECLiPSeConsoleEngineRuntime engine = new ECLiPSeConsoleEngineRuntime(
						interactive, environment, console);
				item.addSelectionListener(new SelectionAdapter() {
					public void widgetSelected(SelectionEvent event) {
						engine.terminate();
						tabItem.dispose();
					}
				});
			}
		};
		Display.getDefault().asyncExec(r);
	}

	private TabFolder tabFolder = null;
}
