/*******************************************************************************
 * Copyright (c) 2006 Cisco Technology, Inc and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Author: Andrew Cheadle, CrossCore Optimization Limited
 *
 * Contributors:
 *     Cisco Technology, Inc - Initial implementation
 *******************************************************************************/

package org.eclipseclp.saros.development.views;

import org.eclipse.ui.IWorkbenchPage;
import org.eclipseclp.saros.runtime.ECLiPSeInteractionInterface;

public class SarosViewNotifier {

	public final static int ACTIVATE = 0;

	public final static int DEACTIVATE = 1;

	public final static int CLOSED = 2;

	public final static int CONFIGURE = 3;

	public final static int DECONFIGURE = 4;

	public static void notifyViews(IWorkbenchPage page,
			ECLiPSeInteractionInterface eclipseInteraction, int mode) {
		for (int index = 0; index < viewsToNotify.length; index++) {
			notifyView((SarosViewNotification) page
					.findView(viewsToNotify[index]), eclipseInteraction, mode);
		}
	}

	private static void notifyView(SarosViewNotification vni,
			ECLiPSeInteractionInterface eclipseInteraction, int mode) {
		if (vni == null) {
			return;
		}

		switch (mode) {
		case ACTIVATE:
			vni.editorActivated(eclipseInteraction);
			break;
		case DEACTIVATE:
			vni.editorDeactivated(eclipseInteraction);
			break;
		case CONFIGURE:
			vni.configureEngine(eclipseInteraction);
			break;
		case DECONFIGURE:
			vni.deconfigureEngine(eclipseInteraction);
			break;
		case CLOSED:
			vni.viewClosed(eclipseInteraction);
			break;
		}
	}

	private static String[] viewsToNotify = {
			"org.eclipseclp.saros.development.views.CallStack",
			"org.eclipseclp.saros.development.views.CompileScratchpad",
			"org.eclipseclp.saros.development.views.DelayedGoals",
			"org.eclipseclp.saros.development.views.Inspector",
			"org.eclipseclp.saros.development.views.PredicateBrowser",
			"org.eclipseclp.saros.development.views.StatisticsSummary",
			"org.eclipseclp.saros.development.views.TraceLog" };
}
