/*******************************************************************************
 * Copyright (c) 2006 Cisco Technology, Inc and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Author: Andrew Cheadle, CrossCore Optimization Limited
 *
 * Contributors:
 *     Cisco Technology, Inc - Initial implementation
 *******************************************************************************/

package org.eclipseclp.saros.documentation;

import java.io.ByteArrayInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.util.Locale;
import java.io.File;
import java.net.MalformedURLException;
import java.io.FileInputStream;

import org.eclipseclp.saros.core.preferences.PreferenceManager;

import org.eclipse.help.IHelpContentProducer;

public class DocumentLocator implements IHelpContentProducer {

	private String currentManualPath = "";

	public InputStream getInputStream(String pluginID, String href,
			Locale locale) {
		String generatedHtml = null;
		// Fake document hrefs produced by toc.xml - keep in-sync
		try {
			if (href.equals("_ec_userman.html")) {
				currentManualPath = PreferenceManager
						.getDefaultInstallationPath()
						+ "/doc/userman";
				return new FileInputStream(new File(currentManualPath,
						"umsroot.html"));
			} else if (href.equals("_ec_libman.html")) {
				currentManualPath = PreferenceManager
						.getDefaultInstallationPath()
						+ "/doc/libman";
				return new FileInputStream(new File(currentManualPath,
						"libman.html"));
			} else if (href.equals("_ec_refman.html")) {
				currentManualPath = PreferenceManager
						.getDefaultInstallationPath()
						+ "/doc/bips";
				return new FileInputStream(new File(currentManualPath,
						"index.html"));
			} else if (href.equals("_ec_embedding.html")) {
				currentManualPath = PreferenceManager
						.getDefaultInstallationPath()
						+ "/doc/embedding";
				return new FileInputStream(new File(currentManualPath,
						"embroot.html"));
			} else if (href.equals("_ec_javainterface.html")) {
				currentManualPath = PreferenceManager
						.getDefaultInstallationPath()
						+ "/doc/javadoc/JavaEclipseInterface";
				return new FileInputStream(new File(currentManualPath,
						"index.html"));
			} else if (href.equals("_ec_visualisation.html")) {
				currentManualPath = PreferenceManager
						.getDefaultInstallationPath()
						+ "/doc/visualisation";
				return new FileInputStream(new File(currentManualPath,
						"visualisation.html"));
			} else if (href.equals("_ec_obsman.html")) {
				currentManualPath = PreferenceManager
						.getDefaultInstallationPath()
						+ "/doc/obsman";
				return new FileInputStream(new File(currentManualPath,
						"obsman.html"));
			} else if (href.equals("_ec_tutorial.html")) {
				currentManualPath = PreferenceManager
						.getDefaultInstallationPath()
						+ "/doc/tutorial";
				return new FileInputStream(new File(currentManualPath,
						"tutorial.html"));
			} else if (href.equals("_ec_applications.html")) {
				currentManualPath = PreferenceManager
						.getDefaultInstallationPath()
						+ "/doc/applications";
				return new FileInputStream(new File(currentManualPath,
						"tutorial.html"));
			} else if (href.equals("userman.pdf")) {
				return new FileInputStream(new File(PreferenceManager
						.getDefaultInstallationPath(), "doc/userman.pdf"));
			} else if (href.equals("libman.pdf")) {
				return new FileInputStream(new File(PreferenceManager
						.getDefaultInstallationPath(), "doc/libman.pdf"));
			} else if (href.equals("embedding.pdf")) {
				return new FileInputStream(new File(PreferenceManager
						.getDefaultInstallationPath(), "doc/embedding.pdf"));
			} else if (href.equals("visualisation.pdf")) {
				return new FileInputStream(new File(PreferenceManager
						.getDefaultInstallationPath(), "doc/visualisation.pdf"));
			} else if (href.equals("obsman.pdf")) {
				return new FileInputStream(new File(PreferenceManager
						.getDefaultInstallationPath(), "doc/obsman.pdf"));
			} else if (href.equals("tutorial.pdf")) {
				return new FileInputStream(new File(PreferenceManager
						.getDefaultInstallationPath(), "doc/tutorial.pdf"));
			} else if (href.equals("applications.pdf")) {
				return new FileInputStream(new File(PreferenceManager
						.getDefaultInstallationPath(), "doc/applications.pdf"));
			} else if (href.equals("userman.html")) {
				generatedHtml = generateHtmlUserman();
			} else if (href.equals("libman.html")) {
				generatedHtml = generateHtmlLibman();
			} else if (href.equals("refman.html")) {
				generatedHtml = generateHtmlRefman();
			} else if (href.equals("embedding.html")) {
				generatedHtml = generateHtmlEmbedding();
			} else if (href.equals("javainterface.html")) {
				generatedHtml = generateHtmlJavaInterface();
			} else if (href.equals("visualisation.html")) {
				generatedHtml = generateHtmlVisualisation();
			} else if (href.equals("obsman.html")) {
				generatedHtml = generateHtmlObsoleteman();
			} else if (href.equals("tutorial.html")) {
				generatedHtml = generateHtmlTutorial();
			} else if (href.equals("applications.html")) {
				generatedHtml = generateHtmlApplications();
			} else if (href.equals("usermanpdf.html")) {
				generatedHtml = generatePdfUserman();
			} else if (href.equals("libmanpdf.html")) {
				generatedHtml = generatePdfLibman();
			} else if (href.equals("embeddingpdf.html")) {
				generatedHtml = generatePdfEmbedding();
			} else if (href.equals("visualisationpdf.html")) {
				generatedHtml = generatePdfVisualisation();
			} else if (href.equals("obsmanpdf.html")) {
				generatedHtml = generatePdfObsolete();
			} else if (href.equals("tutorialpdf.html")) {
				generatedHtml = generatePdfTutorial();
			} else if (href.equals("applicationspdf.html")) {
				generatedHtml = generatePdfApplications();
			} else {
				return new FileInputStream(new File(currentManualPath, href));
			}
		} catch (FileNotFoundException fnfe) {
			return null;
		}

		if (generatedHtml == null) {
			return null;
		}
		return new ByteArrayInputStream(generatedHtml.getBytes());
	}

	private String generateHtmlUserman() {
		String[] installationNames = PreferenceManager.getInstallationNames();
		String[] installationPaths = PreferenceManager.getInstallationPaths();
		if (installationNames.length == 0) {
			return null;
		}
		String generatedHtml = "<HTML><HEAD><TITLE>The ECLiPSe Constraint Logic Programming System</TITLE></HEAD>"
				+ "<BODY><H2>User Manual for installation:</H2><BR><H3><UL>";

		for (int engine = 0; engine < installationNames.length; engine++) {
			File path = new File(installationPaths[engine],
					"doc/userman/umsroot.html");
			try {
				generatedHtml = generatedHtml.concat("<LI><A HREF=\""
						+ path.toURL() + "\">" + installationNames[engine]
						+ "</A>");
			} catch (MalformedURLException mue) {
				System.err.println(mue.getMessage());
			}
		}

		generatedHtml = generatedHtml.concat("</UL></H3></BODY></HTML>");
		return generatedHtml;
	}

	private String generateHtmlLibman() {
		String[] installationNames = PreferenceManager.getInstallationNames();
		String[] installationPaths = PreferenceManager.getInstallationPaths();
		if (installationNames.length == 0) {
			return null;
		}
		String generatedHtml = "<HTML><HEAD><TITLE>The ECLiPSe Constraint Logic Programming System</TITLE></HEAD>"
				+ "<BODY><H2>Constraint Library Manual for installation:</H2><BR><H3><UL>";

		for (int engine = 0; engine < installationNames.length; engine++) {
			File path = new File(installationPaths[engine],
					"doc/libman/libman.html");
			try {
				generatedHtml = generatedHtml.concat("<LI><A HREF=\""
						+ path.toURL() + "\">" + installationNames[engine]
						+ "</A>");
			} catch (MalformedURLException mue) {
				System.err.println(mue.getMessage());
			}
		}

		generatedHtml = generatedHtml.concat("</UL></H3></BODY></HTML>");
		return generatedHtml;
	}

	private String generateHtmlRefman() {
		String[] installationNames = PreferenceManager.getInstallationNames();
		String[] installationPaths = PreferenceManager.getInstallationPaths();
		if (installationNames.length == 0) {
			return null;
		}
		String generatedHtml = "<HTML><HEAD><TITLE>The ECLiPSe Constraint Logic Programming System</TITLE></HEAD>"
				+ "<BODY><H2>Reference Manual for installation:</H2><BR><H3><UL>";

		for (int engine = 0; engine < installationNames.length; engine++) {
			File path = new File(installationPaths[engine],
					"doc/bips/index.html");
			try {
				generatedHtml = generatedHtml.concat("<LI><A HREF=\""
						+ path.toURL() + "\">" + installationNames[engine]
						+ "</A>");
			} catch (MalformedURLException mue) {
				System.err.println(mue.getMessage());
			}
		}

		generatedHtml = generatedHtml.concat("</UL></H3></BODY></HTML>");
		return generatedHtml;
	}

	private String generateHtmlEmbedding() {
		String[] installationNames = PreferenceManager.getInstallationNames();
		String[] installationPaths = PreferenceManager.getInstallationPaths();
		if (installationNames.length == 0) {
			return null;
		}
		String generatedHtml = "<HTML><HEAD><TITLE>The ECLiPSe Constraint Logic Programming System</TITLE></HEAD>"
				+ "<BODY><H2>Embedding and Interfacing Manual for installation:</H2><BR><H3><UL>";

		for (int engine = 0; engine < installationNames.length; engine++) {
			File path = new File(installationPaths[engine],
					"doc/embedding/embroot.html");
			try {
				generatedHtml = generatedHtml.concat("<LI><A HREF=\""
						+ path.toURL() + "\">" + installationNames[engine]
						+ "</A>");
			} catch (MalformedURLException mue) {
				System.err.println(mue.getMessage());
			}
		}

		generatedHtml = generatedHtml.concat("</UL></H3></BODY></HTML>");
		return generatedHtml;
	}

	private String generateHtmlJavaInterface() {
		String[] installationNames = PreferenceManager.getInstallationNames();
		String[] installationPaths = PreferenceManager.getInstallationPaths();
		if (installationNames.length == 0) {
			return null;
		}
		String generatedHtml = "<HTML><HEAD><TITLE>The ECLiPSe Constraint Logic Programming System</TITLE></HEAD>"
				+ "<BODY><H2>Java-ECLiPSe Interface API documentation for installation:</H2><BR><H3><UL>";

		for (int engine = 0; engine < installationNames.length; engine++) {
			File path = new File(installationPaths[engine],
					"doc/javadoc/JavaEclipseInterface/index.html");
			try {
				generatedHtml = generatedHtml.concat("<LI><A HREF=\""
						+ path.toURL() + "\">" + installationNames[engine]
						+ "</A>");
			} catch (MalformedURLException mue) {
				System.err.println(mue.getMessage());
			}
		}

		generatedHtml = generatedHtml.concat("</UL></H3></BODY></HTML>");
		return generatedHtml;
	}

	private String generateHtmlVisualisation() {
		String[] installationNames = PreferenceManager.getInstallationNames();
		String[] installationPaths = PreferenceManager.getInstallationPaths();
		if (installationNames.length == 0) {
			return null;
		}
		String generatedHtml = "<HTML><HEAD><TITLE>The ECLiPSe Constraint Logic Programming System</TITLE></HEAD>"
				+ "<BODY><H2>Visualisation Tools Manual for installation:</H2><BR><H3><UL>";

		for (int engine = 0; engine < installationNames.length; engine++) {
			File path = new File(installationPaths[engine],
					"doc/visualisation/visualisation.html");
			try {
				generatedHtml = generatedHtml.concat("<LI><A HREF=\""
						+ path.toURL() + "\">" + installationNames[engine]
						+ "</A>");
			} catch (MalformedURLException mue) {
				System.err.println(mue.getMessage());
			}
		}

		generatedHtml = generatedHtml.concat("</UL></H3></BODY></HTML>");
		return generatedHtml;
	}

	private String generateHtmlObsoleteman() {
		String[] installationNames = PreferenceManager.getInstallationNames();
		String[] installationPaths = PreferenceManager.getInstallationPaths();
		if (installationNames.length == 0) {
			return null;
		}
		String generatedHtml = "<HTML><HEAD><TITLE>The ECLiPSe Constraint Logic Programming System</TITLE></HEAD>"
				+ "<BODY><H2>Obsolete Libraries Manual for installation:</H2><BR><H3><UL>";

		for (int engine = 0; engine < installationNames.length; engine++) {
			File path = new File(installationPaths[engine],
					"doc/obsman/obsman.html");
			try {
				generatedHtml = generatedHtml.concat("<LI><A HREF=\""
						+ path.toURL() + "\">" + installationNames[engine]
						+ "</A>");
			} catch (MalformedURLException mue) {
				System.err.println(mue.getMessage());
			}
		}

		generatedHtml = generatedHtml.concat("</UL></H3></BODY></HTML>");
		return generatedHtml;
	}

	private String generateHtmlTutorial() {
		String[] installationNames = PreferenceManager.getInstallationNames();
		String[] installationPaths = PreferenceManager.getInstallationPaths();
		if (installationNames.length == 0) {
			return null;
		}
		String generatedHtml = "<HTML><HEAD><TITLE>The ECLiPSe Constraint Logic Programming System</TITLE></HEAD>"
				+ "<BODY><H2>ECLiPSe Tutorial Introduction for installation:</H2><BR><H3><UL>";

		for (int engine = 0; engine < installationNames.length; engine++) {
			File path = new File(installationPaths[engine],
					"doc/tutorial/tutorial.html");
			try {
				generatedHtml = generatedHtml.concat("<LI><A HREF=\""
						+ path.toURL() + "\">" + installationNames[engine]
						+ "</A>");
			} catch (MalformedURLException mue) {
				System.err.println(mue.getMessage());
			}
		}

		generatedHtml = generatedHtml.concat("</UL></H3></BODY></HTML>");
		return generatedHtml;
	}

	private String generateHtmlApplications() {
		String[] installationNames = PreferenceManager.getInstallationNames();
		String[] installationPaths = PreferenceManager.getInstallationPaths();
		if (installationNames.length == 0) {
			return null;
		}
		String generatedHtml = "<HTML><HEAD><TITLE>The ECLiPSe Constraint Logic Programming System</TITLE></HEAD>"
				+ "<BODY><H2>Application Development Manual for installation:</H2><BR><H3><UL>";

		for (int engine = 0; engine < installationNames.length; engine++) {
			File path = new File(installationPaths[engine],
					"doc/applications/tutorial.html");
			try {
				generatedHtml = generatedHtml.concat("<LI><A HREF=\""
						+ path.toURL() + "\">" + installationNames[engine]
						+ "</A>");
			} catch (MalformedURLException mue) {
				System.err.println(mue.getMessage());
			}
		}

		generatedHtml = generatedHtml.concat("</UL></H3></BODY></HTML>");
		return generatedHtml;
	}

	private String generatePdfUserman() {
		String[] installationNames = PreferenceManager.getInstallationNames();
		String[] installationPaths = PreferenceManager.getInstallationPaths();
		if (installationNames.length == 0) {
			return null;
		}
		String generatedHtml = "<HTML><HEAD><TITLE>The ECLiPSe Constraint Logic Programming System</TITLE></HEAD>"
				+ "<BODY><H2>User Manual for installation:</H2><BR><H3><UL>";

		for (int engine = 0; engine < installationNames.length; engine++) {
			File path = new File(installationPaths[engine], "doc/userman.pdf");
			try {
				generatedHtml = generatedHtml.concat("<LI><A HREF=\""
						+ path.toURL() + "\">" + installationNames[engine]
						+ "</A>");
			} catch (MalformedURLException mue) {
				System.err.println(mue.getMessage());
			}
		}

		generatedHtml = generatedHtml.concat("</UL></H3></BODY></HTML>");
		return generatedHtml;
	}

	private String generatePdfLibman() {
		String[] installationNames = PreferenceManager.getInstallationNames();
		String[] installationPaths = PreferenceManager.getInstallationPaths();
		if (installationNames.length == 0) {
			return null;
		}
		String generatedHtml = "<HTML><HEAD><TITLE>The ECLiPSe Constraint Logic Programming System</TITLE></HEAD>"
				+ "<BODY><H2>Constraint Library Manual for installation:</H2><BR><H3><UL>";

		for (int engine = 0; engine < installationNames.length; engine++) {
			File path = new File(installationPaths[engine], "doc/libman.pdf");
			try {
				generatedHtml = generatedHtml.concat("<LI><A HREF=\""
						+ path.toURL() + "\">" + installationNames[engine]
						+ "</A>");
			} catch (MalformedURLException mue) {
				System.err.println(mue.getMessage());
			}
		}

		generatedHtml = generatedHtml.concat("</UL></H3></BODY></HTML>");
		return generatedHtml;
	}

	private String generatePdfEmbedding() {
		String[] installationNames = PreferenceManager.getInstallationNames();
		String[] installationPaths = PreferenceManager.getInstallationPaths();
		if (installationNames.length == 0) {
			return null;
		}
		String generatedHtml = "<HTML><HEAD><TITLE>The ECLiPSe Constraint Logic Programming System</TITLE></HEAD>"
				+ "<BODY><H2>Embedding and Interfacing Manual for installation:</H2><BR><H3><UL>";

		for (int engine = 0; engine < installationNames.length; engine++) {
			File path = new File(installationPaths[engine], "doc/embedding.pdf");
			try {
				generatedHtml = generatedHtml.concat("<LI><A HREF=\""
						+ path.toURL() + "\">" + installationNames[engine]
						+ "</A>");
			} catch (MalformedURLException mue) {
				System.err.println(mue.getMessage());
			}
		}

		generatedHtml = generatedHtml.concat("</UL></H3></BODY></HTML>");
		return generatedHtml;
	}

	private String generatePdfVisualisation() {
		String[] installationNames = PreferenceManager.getInstallationNames();
		String[] installationPaths = PreferenceManager.getInstallationPaths();
		if (installationNames.length == 0) {
			return null;
		}
		String generatedHtml = "<HTML><HEAD><TITLE>The ECLiPSe Constraint Logic Programming System</TITLE></HEAD>"
				+ "<BODY><H2>Visualisation Tools Manual for installation:</H2><BR><H3><UL>";

		for (int engine = 0; engine < installationNames.length; engine++) {
			File path = new File(installationPaths[engine],
					"doc/visualisation.pdf");
			try {
				generatedHtml = generatedHtml.concat("<LI><A HREF=\""
						+ path.toURL() + "\">" + installationNames[engine]
						+ "</A>");
			} catch (MalformedURLException mue) {
				System.err.println(mue.getMessage());
			}
		}

		generatedHtml = generatedHtml.concat("</UL></H3></BODY></HTML>");
		return generatedHtml;
	}

	private String generatePdfObsolete() {
		String[] installationNames = PreferenceManager.getInstallationNames();
		String[] installationPaths = PreferenceManager.getInstallationPaths();
		if (installationNames.length == 0) {
			return null;
		}
		String generatedHtml = "<HTML><HEAD><TITLE>The ECLiPSe Constraint Logic Programming System</TITLE></HEAD>"
				+ "<BODY><H2>Obsolete Libraries User Manual for installation:</H2><BR><H3><UL>";

		for (int engine = 0; engine < installationNames.length; engine++) {
			File path = new File(installationPaths[engine], "doc/obsman.pdf");
			try {
				generatedHtml = generatedHtml.concat("<LI><A HREF=\""
						+ path.toURL() + "\">" + installationNames[engine]
						+ "</A>");
			} catch (MalformedURLException mue) {
				System.err.println(mue.getMessage());
			}
		}

		generatedHtml = generatedHtml.concat("</UL></H3></BODY></HTML>");
		return generatedHtml;
	}

	private String generatePdfTutorial() {
		String[] installationNames = PreferenceManager.getInstallationNames();
		String[] installationPaths = PreferenceManager.getInstallationPaths();
		if (installationNames.length == 0) {
			return null;
		}
		String generatedHtml = "<HTML><HEAD><TITLE>The ECLiPSe Constraint Logic Programming System</TITLE></HEAD>"
				+ "<BODY><H2>ECLiPSe Tutorial Introduction for installation:</H2><BR><H3><UL>";

		for (int engine = 0; engine < installationNames.length; engine++) {
			File path = new File(installationPaths[engine], "doc/tutorial.pdf");
			try {
				generatedHtml = generatedHtml.concat("<LI><A HREF=\""
						+ path.toURL() + "\">" + installationNames[engine]
						+ "</A>");
			} catch (MalformedURLException mue) {
				System.err.println(mue.getMessage());
			}
		}

		generatedHtml = generatedHtml.concat("</UL></H3></BODY></HTML>");
		return generatedHtml;
	}

	private String generatePdfApplications() {
		String[] installationNames = PreferenceManager.getInstallationNames();
		String[] installationPaths = PreferenceManager.getInstallationPaths();
		if (installationNames.length == 0) {
			return null;
		}
		String generatedHtml = "<HTML><HEAD><TITLE>The ECLiPSe Constraint Logic Programming System</TITLE></HEAD>"
				+ "<BODY><H2>Application Development Manual for installation:</H2><BR><H3><UL>";

		for (int engine = 0; engine < installationNames.length; engine++) {
			File path = new File(installationPaths[engine],
					"doc/applications.pdf");
			try {
				generatedHtml = generatedHtml.concat("<LI><A HREF=\""
						+ path.toURL() + "\">" + installationNames[engine]
						+ "</A>");
			} catch (MalformedURLException mue) {
				System.err.println(mue.getMessage());
			}
		}

		generatedHtml = generatedHtml.concat("</UL></H3></BODY></HTML>");
		return generatedHtml;
	}
}
