/*******************************************************************************
 * Copyright (c) 2006 Cisco Technology, Inc and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Author: Andrew Cheadle, CrossCore Optimization Limited
 *
 * Contributors:
 *     Cisco Technology, Inc - Initial implementation
 *******************************************************************************/

package org.eclipseclp.saros.runtime;

import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.custom.StyleRange;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Display;

public class RunnableTextAreaUpdater implements Runnable {

	public RunnableTextAreaUpdater(String text, StyledText textArea,
			int colour, boolean clearOnUpdate) {
		this.text = text;
		this.textArea = textArea;
		this.colour = colour;
		this.clearOnUpdate = clearOnUpdate;
	}

	public void run() {
		if (textArea == null) {
			return;
		}

		StyleRange[] styleRanges = null;
		if (clearOnUpdate) {
			styleRanges = new StyleRange[1];

			styleRanges[0] = new StyleRange();
			styleRanges[0].start = 0;
			styleRanges[0].length = text.length();
			styleRanges[0].foreground = Display.getDefault().getSystemColor(
					colour);
			textArea.replaceTextRange(0, textArea.getCharCount(), text);
		} else {
			styleRanges = new StyleRange[2];

			styleRanges[0] = new StyleRange();
			styleRanges[0].start = 0;
			styleRanges[0].length = textArea.getCharCount();
			styleRanges[0].foreground = Display.getDefault().getSystemColor(
					SWT.COLOR_BLACK);
			// textArea.setStyleRange(styleRange);

			styleRanges[1] = new StyleRange();
			styleRanges[1].start = textArea.getCharCount();
			styleRanges[1].length = text.length();
			styleRanges[1].foreground = Display.getDefault().getSystemColor(
					colour);
			textArea.append(text);
		}
		charCount = textArea.getCharCount();
		textArea.setStyleRanges(styleRanges);
		textArea.setCaretOffset(charCount);
		textArea.showSelection();
	}

	public int getCharCount() {
		return charCount;
	}

	public void setTextArea(StyledText textArea) {
		this.textArea = textArea;
	}

	private String text = null;

	private StyledText textArea = null;

	private int colour = SWT.COLOR_BLACK;

	private int charCount = 0;

	private boolean clearOnUpdate = false;

}
